@@ -6,7 +6,7 @@ const lookup = require('dns').lookup;
 const bench = common.createBenchmark(main, {
   name: ['127.0.0.1', '::1'],
   all: ['true', 'false'],
-  n: [5e6]
+  n: [5e6],
 });
 
 function main({ name, n, all }) {
