@@ -10,6 +10,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <iomanip>
+#include <set>
 #include <sstream>
 
 #include "src/base/functional.h"
@@ -28,16 +29,23 @@
 #include "src/wasm/wasm-limits.h"
 #endif  // V8_ENABLE_WEBASSEMBLY
 
-namespace v8 {
-namespace internal {
+namespace v8::internal {
 
-// Define all of our flags.
-#define FLAG_MODE_DEFINE
-#include "src/flags/flag-definitions.h"  // NOLINT(build/include)
+// Define {v8_flags}, declared in flags.h.
+FlagValues v8_flags;
+
+// {v8_flags} needs to be aligned to a memory page, and the size needs to be a
+// multiple of a page size. This is required for memory-protection of the memory
+// holding the {v8_flags} struct.
+// Both is guaranteed by the {alignas(kMinimumOSPageSize)} annotation on
+// {FlagValues}.
+static_assert(alignof(FlagValues) == kMinimumOSPageSize);
+static_assert(sizeof(FlagValues) % kMinimumOSPageSize == 0);
 
 // Define all of our flags default values.
 #define FLAG_MODE_DEFINE_DEFAULTS
 #include "src/flags/flag-definitions.h"  // NOLINT(build/include)
+#undef FLAG_MODE_DEFINE_DEFAULTS
 
 namespace {
 
@@ -47,6 +55,27 @@ struct Flag;
 Flag* FindFlagByPointer(const void* ptr);
 Flag* FindFlagByName(const char* name);
 
+// Helper struct for printing normalized flag names.
+struct FlagName {
+  const char* name;
+  bool negated;
+
+  constexpr FlagName(const char* name, bool negated)
+      : name(name), negated(negated) {
+    DCHECK_NE('\0', name[0]);
+    DCHECK_NE('!', name[0]);
+  }
+
+  constexpr explicit FlagName(const char* name)
+      : FlagName(name[0] == '!' ? name + 1 : name, name[0] == '!') {}
+};
+
+std::ostream& operator<<(std::ostream& os, FlagName flag_name) {
+  os << (flag_name.negated ? "--no-" : "--");
+  for (const char* p = flag_name.name; *p; ++p) os << NormalizeChar(*p);
+  return os;
+}
+
 // This structure represents a single entry in the flag system, with a pointer
 // to the actual flag, default value, comment, etc.  This is designed to be POD
 // initialized as to avoid requiring static constructors.
@@ -64,6 +93,10 @@ struct Flag {
 
   enum class SetBy { kDefault, kWeakImplication, kImplication, kCommandLine };
 
+  constexpr bool IsAnyImplication(Flag::SetBy set_by) {
+    return set_by == SetBy::kWeakImplication || set_by == SetBy::kImplication;
+  }
+
   FlagType type_;       // What type of flag, bool, int, or string.
   const char* name_;    // Name of the flag, ex "my_flag".
   void* valptr_;        // Pointer to the global flag variable.
@@ -71,7 +104,12 @@ struct Flag {
   const char* cmt_;     // A comment about the flags purpose.
   bool owns_ptr_;       // Does the flag own its string value?
   SetBy set_by_ = SetBy::kDefault;
+  // Name of the flag implying this flag, if any.
   const char* implied_by_ = nullptr;
+#ifdef DEBUG
+  // Pointer to the flag implying this flag, if any.
+  const Flag* implied_by_ptr_ = nullptr;
+#endif
 
   FlagType type() const { return type_; }
 
@@ -81,154 +119,136 @@ struct Flag {
 
   bool PointsTo(const void* ptr) const { return valptr_ == ptr; }
 
-  bool bool_variable() const {
-    DCHECK(type_ == TYPE_BOOL);
-    return *reinterpret_cast<bool*>(valptr_);
+#ifdef DEBUG
+  bool ImpliedBy(const void* ptr) const {
+    const Flag* current = this->implied_by_ptr_;
+    while (current != nullptr) {
+      if (current->PointsTo(ptr)) return true;
+      current = current->implied_by_ptr_;
+    }
+    return false;
   }
+#endif
+
+  bool bool_variable() const { return GetValue<TYPE_BOOL, bool>(); }
 
   void set_bool_variable(bool value, SetBy set_by) {
-    DCHECK(type_ == TYPE_BOOL);
-    bool change_flag = *reinterpret_cast<bool*>(valptr_) != value;
-    change_flag = CheckFlagChange(set_by, change_flag);
-    if (change_flag) *reinterpret_cast<bool*>(valptr_) = value;
+    SetValue<TYPE_BOOL, bool>(value, set_by);
   }
 
-  MaybeBoolFlag maybe_bool_variable() const {
-    DCHECK(type_ == TYPE_MAYBE_BOOL);
-    return *reinterpret_cast<MaybeBoolFlag*>(valptr_);
+  base::Optional<bool> maybe_bool_variable() const {
+    return GetValue<TYPE_MAYBE_BOOL, base::Optional<bool>>();
   }
 
-  void set_maybe_bool_variable(MaybeBoolFlag value, SetBy set_by) {
-    DCHECK(type_ == TYPE_MAYBE_BOOL);
-    bool change_flag = *reinterpret_cast<MaybeBoolFlag*>(valptr_) != value;
-    change_flag = CheckFlagChange(set_by, change_flag);
-    if (change_flag) *reinterpret_cast<MaybeBoolFlag*>(valptr_) = value;
+  void set_maybe_bool_variable(base::Optional<bool> value, SetBy set_by) {
+    SetValue<TYPE_MAYBE_BOOL, base::Optional<bool>>(value, set_by);
   }
 
-  int int_variable() const {
-    DCHECK(type_ == TYPE_INT);
-    return *reinterpret_cast<int*>(valptr_);
-  }
+  int int_variable() const { return GetValue<TYPE_INT, int>(); }
 
   void set_int_variable(int value, SetBy set_by) {
-    DCHECK(type_ == TYPE_INT);
-    bool change_flag = *reinterpret_cast<int*>(valptr_) != value;
-    change_flag = CheckFlagChange(set_by, change_flag);
-    if (change_flag) *reinterpret_cast<int*>(valptr_) = value;
+    SetValue<TYPE_INT, int>(value, set_by);
   }
 
   unsigned int uint_variable() const {
-    DCHECK(type_ == TYPE_UINT);
-    return *reinterpret_cast<unsigned int*>(valptr_);
+    return GetValue<TYPE_UINT, unsigned int>();
   }
 
   void set_uint_variable(unsigned int value, SetBy set_by) {
-    DCHECK(type_ == TYPE_UINT);
-    bool change_flag = *reinterpret_cast<unsigned int*>(valptr_) != value;
-    change_flag = CheckFlagChange(set_by, change_flag);
-    if (change_flag) *reinterpret_cast<unsigned int*>(valptr_) = value;
+    SetValue<TYPE_UINT, unsigned int>(value, set_by);
   }
 
-  uint64_t uint64_variable() const {
-    DCHECK(type_ == TYPE_UINT64);
-    return *reinterpret_cast<uint64_t*>(valptr_);
-  }
+  uint64_t uint64_variable() const { return GetValue<TYPE_UINT64, uint64_t>(); }
 
   void set_uint64_variable(uint64_t value, SetBy set_by) {
-    DCHECK(type_ == TYPE_UINT64);
-    bool change_flag = *reinterpret_cast<uint64_t*>(valptr_) != value;
-    change_flag = CheckFlagChange(set_by, change_flag);
-    if (change_flag) *reinterpret_cast<uint64_t*>(valptr_) = value;
+    SetValue<TYPE_UINT64, uint64_t>(value, set_by);
   }
 
-  double float_variable() const {
-    DCHECK(type_ == TYPE_FLOAT);
-    return *reinterpret_cast<double*>(valptr_);
-  }
+  double float_variable() const { return GetValue<TYPE_FLOAT, double>(); }
 
   void set_float_variable(double value, SetBy set_by) {
-    DCHECK(type_ == TYPE_FLOAT);
-    bool change_flag = *reinterpret_cast<double*>(valptr_) != value;
-    change_flag = CheckFlagChange(set_by, change_flag);
-    if (change_flag) *reinterpret_cast<double*>(valptr_) = value;
+    SetValue<TYPE_FLOAT, double>(value, set_by);
   }
 
-  size_t size_t_variable() const {
-    DCHECK(type_ == TYPE_SIZE_T);
-    return *reinterpret_cast<size_t*>(valptr_);
-  }
+  size_t size_t_variable() const { return GetValue<TYPE_SIZE_T, size_t>(); }
 
   void set_size_t_variable(size_t value, SetBy set_by) {
-    DCHECK(type_ == TYPE_SIZE_T);
-    bool change_flag = *reinterpret_cast<size_t*>(valptr_) != value;
-    change_flag = CheckFlagChange(set_by, change_flag);
-    if (change_flag) *reinterpret_cast<size_t*>(valptr_) = value;
+    SetValue<TYPE_SIZE_T, size_t>(value, set_by);
   }
 
   const char* string_value() const {
-    DCHECK(type_ == TYPE_STRING);
-    return *reinterpret_cast<const char**>(valptr_);
+    return GetValue<TYPE_STRING, const char*>();
   }
 
-  void set_string_value(const char* value, bool owns_ptr, SetBy set_by) {
-    DCHECK(type_ == TYPE_STRING);
-    const char** ptr = reinterpret_cast<const char**>(valptr_);
-    bool change_flag = (*ptr == nullptr) != (value == nullptr) ||
-                       (*ptr && value && std::strcmp(*ptr, value) != 0);
+  void set_string_value(const char* new_value, bool owns_new_value,
+                        SetBy set_by) {
+    DCHECK_EQ(TYPE_STRING, type_);
+    DCHECK_IMPLIES(owns_new_value, new_value != nullptr);
+    auto* flag_value = reinterpret_cast<FlagValue<const char*>*>(valptr_);
+    const char* old_value = *flag_value;
+    DCHECK_IMPLIES(owns_ptr_, old_value != nullptr);
+    bool change_flag =
+        old_value ? !new_value || std::strcmp(old_value, new_value) != 0
+                  : !!new_value;
     change_flag = CheckFlagChange(set_by, change_flag);
     if (change_flag) {
-      if (owns_ptr_ && *ptr != nullptr) DeleteArray(*ptr);
-      *ptr = value;
-      owns_ptr_ = owns_ptr;
+      if (owns_ptr_) DeleteArray(old_value);
+      *flag_value = new_value;
+      owns_ptr_ = owns_new_value;
     } else {
-      if (owns_ptr && value != nullptr) DeleteArray(value);
+      if (owns_new_value) DeleteArray(new_value);
     }
   }
 
+  template <typename T>
+  T GetDefaultValue() const {
+    return *reinterpret_cast<const T*>(defptr_);
+  }
+
   bool bool_default() const {
-    DCHECK(type_ == TYPE_BOOL);
-    return *reinterpret_cast<const bool*>(defptr_);
+    DCHECK_EQ(TYPE_BOOL, type_);
+    return GetDefaultValue<bool>();
   }
 
   int int_default() const {
-    DCHECK(type_ == TYPE_INT);
-    return *reinterpret_cast<const int*>(defptr_);
+    DCHECK_EQ(TYPE_INT, type_);
+    return GetDefaultValue<int>();
   }
 
   unsigned int uint_default() const {
-    DCHECK(type_ == TYPE_UINT);
-    return *reinterpret_cast<const unsigned int*>(defptr_);
+    DCHECK_EQ(TYPE_UINT, type_);
+    return GetDefaultValue<unsigned int>();
   }
 
   uint64_t uint64_default() const {
-    DCHECK(type_ == TYPE_UINT64);
-    return *reinterpret_cast<const uint64_t*>(defptr_);
+    DCHECK_EQ(TYPE_UINT64, type_);
+    return GetDefaultValue<uint64_t>();
   }
 
   double float_default() const {
-    DCHECK(type_ == TYPE_FLOAT);
-    return *reinterpret_cast<const double*>(defptr_);
+    DCHECK_EQ(TYPE_FLOAT, type_);
+    return GetDefaultValue<double>();
   }
 
   size_t size_t_default() const {
-    DCHECK(type_ == TYPE_SIZE_T);
-    return *reinterpret_cast<const size_t*>(defptr_);
+    DCHECK_EQ(TYPE_SIZE_T, type_);
+    return GetDefaultValue<size_t>();
   }
 
   const char* string_default() const {
-    DCHECK(type_ == TYPE_STRING);
-    return *reinterpret_cast<const char* const*>(defptr_);
+    DCHECK_EQ(TYPE_STRING, type_);
+    return GetDefaultValue<const char*>();
   }
 
   static bool ShouldCheckFlagContradictions() {
-    if (FLAG_allow_overwriting_for_next_flag) {
+    if (v8_flags.allow_overwriting_for_next_flag) {
       // Setting the flag manually to false before calling Reset() avoids this
       // becoming re-entrant.
-      FLAG_allow_overwriting_for_next_flag = false;
-      FindFlagByPointer(&FLAG_allow_overwriting_for_next_flag)->Reset();
+      v8_flags.allow_overwriting_for_next_flag = false;
+      FindFlagByPointer(&v8_flags.allow_overwriting_for_next_flag)->Reset();
       return false;
     }
-    return FLAG_abort_on_contradictory_flags && !FLAG_fuzzing;
+    return v8_flags.abort_on_contradictory_flags && !v8_flags.fuzzing;
   }
 
   // {change_flag} indicates if we're going to change the flag value.
@@ -242,6 +262,28 @@ struct Flag {
       return false;
     }
     if (ShouldCheckFlagContradictions()) {
+      static constexpr const char kHint[] =
+          "If a test variant caused this, it might be necessary to specify "
+          "additional contradictory flags in "
+          "tools/testrunner/local/variants.py.";
+      struct FatalError : public std::ostringstream {
+        // MSVC complains about non-returning destructor; disable that.
+        MSVC_SUPPRESS_WARNING(4722)
+        ~FatalError() { FATAL("%s.\n%s", str().c_str(), kHint); }
+      };
+      // Readonly flags cannot change value.
+      if (change_flag && IsReadOnly()) {
+        // Exit instead of abort for certain testing situations.
+        if (v8_flags.exit_on_contradictory_flags) base::OS::ExitProcess(0);
+        if (implied_by == nullptr) {
+          FatalError{} << "Contradictory value for readonly flag "
+                       << FlagName{name()};
+        } else {
+          DCHECK(IsAnyImplication(new_set_by));
+          FatalError{} << "Contradictory value for readonly flag "
+                       << FlagName{name()} << " implied by " << implied_by;
+        }
+      }
       // For bool flags, we only check for a conflict if the value actually
       // changes. So specifying the same flag with the same value multiple times
       // is allowed.
@@ -253,74 +295,104 @@ struct Flag {
       bool is_bool_flag = type_ == TYPE_MAYBE_BOOL || type_ == TYPE_BOOL;
       bool check_implications = change_flag;
       bool check_command_line_flags = change_flag || !is_bool_flag;
-      const char* hint =
-          "To fix this, it might be necessary to specify additional "
-          "contradictory flags in tools/testrunner/local/variants.py.";
       switch (set_by_) {
         case SetBy::kDefault:
           break;
         case SetBy::kWeakImplication:
           if (new_set_by == SetBy::kWeakImplication && check_implications) {
-            FATAL(
-                "Contradictory weak flag implications from --%s and --%s for "
-                "flag %s\n%s",
-                implied_by_, implied_by, name(), hint);
+            FatalError{} << "Contradictory weak flag implications from "
+                         << FlagName{implied_by_} << " and "
+                         << FlagName{implied_by} << " for flag "
+                         << FlagName{name()};
           }
           break;
         case SetBy::kImplication:
           if (new_set_by == SetBy::kImplication && check_implications) {
-            FATAL(
-                "Contradictory flag implications from --%s and --%s for flag "
-                "%s\n%s",
-                implied_by_, implied_by, name(), hint);
+            FatalError{} << "Contradictory flag implications from "
+                         << FlagName{implied_by_} << " and "
+                         << FlagName{implied_by} << " for flag "
+                         << FlagName{name()};
           }
           break;
         case SetBy::kCommandLine:
           if (new_set_by == SetBy::kImplication && check_command_line_flags) {
+            // Exit instead of abort for certain testing situations.
+            if (v8_flags.exit_on_contradictory_flags) base::OS::ExitProcess(0);
             if (is_bool_flag) {
-              FATAL(
-                  "Flag --%s: value implied by --%s conflicts with explicit "
-                  "specification\n%s",
-                  name(), implied_by, hint);
+              FatalError{} << "Flag " << FlagName{name()}
+                           << ": value implied by " << FlagName{implied_by}
+                           << " conflicts with explicit specification";
             } else {
-              FATAL(
-                  "Flag --%s is implied by --%s but also specified "
-                  "explicitly.\n%s",
-                  name(), implied_by, hint);
+              FatalError{} << "Flag " << FlagName{name()} << " is implied by "
+                           << FlagName{implied_by}
+                           << " but also specified explicitly";
             }
           } else if (new_set_by == SetBy::kCommandLine &&
                      check_command_line_flags) {
+            // Exit instead of abort for certain testing situations.
+            if (v8_flags.exit_on_contradictory_flags) base::OS::ExitProcess(0);
             if (is_bool_flag) {
-              FATAL(
-                  "Command-line provided flag --%s specified as both true and "
-                  "false.\n%s",
-                  name(), hint);
+              FatalError{} << "Command-line provided flag " << FlagName{name()}
+                           << " specified as both true and false";
             } else {
-              FATAL(
-                  "Command-line provided flag --%s specified multiple "
-                  "times.\n%s",
-                  name(), hint);
+              FatalError{} << "Command-line provided flag " << FlagName{name()}
+                           << " specified multiple times";
             }
           }
           break;
       }
     }
+    if (change_flag && IsReadOnly()) {
+      // Readonly flags must never change value.
+      return false;
+    }
     set_by_ = new_set_by;
-    if (new_set_by == SetBy::kImplication ||
-        new_set_by == SetBy::kWeakImplication) {
+    if (IsAnyImplication(new_set_by)) {
       DCHECK_NOT_NULL(implied_by);
       implied_by_ = implied_by;
+#ifdef DEBUG
+      // This only works when implied_by is a flag_name or !flag_name, but it
+      // can also be a condition e.g. flag_name > 3. Since this is only used for
+      // checks in DEBUG mode, we will just ignore the more complex conditions
+      // for now - that will just lead to a nullptr which won't be followed.
+      implied_by_ptr_ = static_cast<Flag*>(
+          FindFlagByName(implied_by[0] == '!' ? implied_by + 1 : implied_by));
+      DCHECK_NE(implied_by_ptr_, this);
+#endif
     }
     return change_flag;
   }
 
+  bool IsReadOnly() const {
+    // See the FLAG_READONLY definition for FLAG_MODE_META.
+    return valptr_ == nullptr;
+  }
+
+  template <FlagType flag_type, typename T>
+  T GetValue() const {
+    DCHECK_EQ(flag_type, type_);
+    if (IsReadOnly()) return GetDefaultValue<T>();
+    return *reinterpret_cast<const FlagValue<T>*>(valptr_);
+  }
+
+  template <FlagType flag_type, typename T>
+  void SetValue(T new_value, SetBy set_by) {
+    DCHECK_EQ(flag_type, type_);
+    bool change_flag = GetValue<flag_type, T>() != new_value;
+    change_flag = CheckFlagChange(set_by, change_flag);
+    if (change_flag) {
+      DCHECK(!IsReadOnly());
+      *reinterpret_cast<FlagValue<T>*>(valptr_) = new_value;
+    }
+  }
+
   // Compare this flag's current value against the default.
   bool IsDefault() const {
     switch (type_) {
       case TYPE_BOOL:
         return bool_variable() == bool_default();
       case TYPE_MAYBE_BOOL:
-        return maybe_bool_variable().has_value == false;
+        return maybe_bool_variable().has_value() == false;
       case TYPE_INT:
         return int_variable() == int_default();
       case TYPE_UINT:
@@ -342,6 +414,11 @@ struct Flag {
     UNREACHABLE();
   }
 
+  void ReleaseDynamicAllocations() {
+    if (type_ != TYPE_STRING) return;
+    if (owns_ptr_) DeleteArray(string_value());
+  }
+
   // Set a flag back to it's default value.
   void Reset() {
     switch (type_) {
@@ -349,8 +426,7 @@ struct Flag {
         set_bool_variable(bool_default(), SetBy::kDefault);
         break;
       case TYPE_MAYBE_BOOL:
-        set_maybe_bool_variable(MaybeBoolFlag::Create(false, false),
-                                SetBy::kDefault);
+        set_maybe_bool_variable(base::nullopt, SetBy::kDefault);
         break;
       case TYPE_INT:
         set_int_variable(int_default(), SetBy::kDefault);
@@ -379,9 +455,10 @@ struct Flag {
 Flag flags[] = {
 #define FLAG_MODE_META
 #include "src/flags/flag-definitions.h"  // NOLINT(build/include)
+#undef FLAG_MODE_META
 };
 
-const size_t num_flags = sizeof(flags) / sizeof(*flags);
+constexpr size_t kNumFlags = arraysize(flags);
 
 bool EqualNames(const char* a, const char* b) {
   for (int i = 0; NormalizeChar(a[i]) == NormalizeChar(b[i]); i++) {
@@ -393,21 +470,19 @@ bool EqualNames(const char* a, const char* b) {
 }
 
 Flag* FindFlagByName(const char* name) {
-  for (size_t i = 0; i < num_flags; ++i) {
+  for (size_t i = 0; i < kNumFlags; ++i) {
     if (EqualNames(name, flags[i].name())) return &flags[i];
   }
   return nullptr;
 }
 
 Flag* FindFlagByPointer(const void* ptr) {
-  for (size_t i = 0; i < num_flags; ++i) {
+  for (size_t i = 0; i < kNumFlags; ++i) {
     if (flags[i].PointsTo(ptr)) return &flags[i];
   }
   return nullptr;
 }
 
-}  // namespace
-
 static const char* Type2String(Flag::FlagType type) {
   switch (type) {
     case Flag::TYPE_BOOL:
@@ -427,37 +502,22 @@ static const char* Type2String(Flag::FlagType type) {
     case Flag::TYPE_STRING:
       return "string";
   }
-  UNREACHABLE();
-}
-
-// Helper struct for printing normalize Flag names.
-struct FlagName {
-  explicit FlagName(const Flag& flag) : flag(flag) {}
-  const Flag& flag;
-};
-
-std::ostream& operator<<(std::ostream& os, const FlagName& flag_name) {
-  for (const char* c = flag_name.flag.name(); *c != '\0'; ++c) {
-    os << NormalizeChar(*c);
-  }
-  return os;
 }
 
 // Helper for printing flag values.
-struct FlagValue {
-  explicit FlagValue(const Flag& flag) : flag(flag) {}
+struct PrintFlagValue {
   const Flag& flag;
 };
 
-std::ostream& operator<<(std::ostream& os, const FlagValue& flag_value) {
+std::ostream& operator<<(std::ostream& os, PrintFlagValue flag_value) {
   const Flag& flag = flag_value.flag;
   switch (flag.type()) {
     case Flag::TYPE_BOOL:
       os << (flag.bool_variable() ? "true" : "false");
       break;
     case Flag::TYPE_MAYBE_BOOL:
-      os << (flag.maybe_bool_variable().has_value
-                 ? (flag.maybe_bool_variable().value ? "true" : "false")
+      os << (flag.maybe_bool_variable().has_value()
+                 ? (flag.maybe_bool_variable().value() ? "true" : "false")
                  : "unset");
       break;
     case Flag::TYPE_INT:
@@ -486,37 +546,87 @@ std::ostream& operator<<(std::ostream& os, const FlagValue& flag_value) {
 
 std::ostream& operator<<(std::ostream& os, const Flag& flag) {
   if (flag.type() == Flag::TYPE_BOOL) {
-    os << (flag.bool_variable() ? "--" : "--no") << FlagName(flag);
+    os << FlagName{flag.name(), !flag.bool_variable()};
   } else {
-    os << "--" << FlagName(flag) << "=" << FlagValue(flag);
+    os << FlagName{flag.name()} << "=" << PrintFlagValue{flag};
   }
   return os;
 }
 
-namespace {
+static std::atomic<uint32_t> flag_hash{0};
+static std::atomic<bool> flags_frozen{false};
 
-static std::atomic<uint32_t> flag_hash(0);
-
-void ComputeFlagListHash() {
+uint32_t ComputeFlagListHash() {
   std::ostringstream modified_args_as_string;
   if (COMPRESS_POINTERS_BOOL) modified_args_as_string << "ptr-compr";
   if (DEBUG_BOOL) modified_args_as_string << "debug";
+
+#ifdef DEBUG
+  // These two sets are used to check that we don't leave out any flags
+  // implied by --predictable in the list below.
+  std::set<const char*> flags_implied_by_predictable;
+  std::set<const char*> flags_ignored_because_of_predictable;
+#endif
+
   for (const Flag& flag : flags) {
     if (flag.IsDefault()) continue;
+#ifdef DEBUG
+    if (flag.ImpliedBy(&v8_flags.predictable)) {
+      flags_implied_by_predictable.insert(flag.name());
+    }
+#endif
     // We want to be able to flip --profile-deserialization without
     // causing the code cache to get invalidated by this hash.
-    if (flag.PointsTo(&FLAG_profile_deserialization)) continue;
-    // Skip FLAG_random_seed to allow predictable code caching.
-    if (flag.PointsTo(&FLAG_random_seed)) continue;
+    if (flag.PointsTo(&v8_flags.profile_deserialization)) continue;
+    // Skip v8_flags.random_seed and v8_flags.predictable to allow predictable
+    // code caching.
+    if (flag.PointsTo(&v8_flags.random_seed)) continue;
+    if (flag.PointsTo(&v8_flags.predictable)) continue;
+
+    // The following flags are implied by --predictable (some negated).
+    if (flag.PointsTo(&v8_flags.concurrent_sparkplug) ||
+        flag.PointsTo(&v8_flags.concurrent_recompilation) ||
+        flag.PointsTo(&v8_flags.parallel_scavenge) ||
+        flag.PointsTo(&v8_flags.concurrent_marking) ||
+        flag.PointsTo(&v8_flags.concurrent_array_buffer_sweeping) ||
+        flag.PointsTo(&v8_flags.parallel_marking) ||
+        flag.PointsTo(&v8_flags.concurrent_sweeping) ||
+        flag.PointsTo(&v8_flags.parallel_compaction) ||
+        flag.PointsTo(&v8_flags.parallel_pointer_update) ||
+        flag.PointsTo(&v8_flags.memory_reducer) ||
+        flag.PointsTo(&v8_flags.cppheap_concurrent_marking) ||
+        flag.PointsTo(&v8_flags.cppheap_incremental_marking) ||
+        flag.PointsTo(&v8_flags.single_threaded_gc)) {
+#ifdef DEBUG
+      if (flag.ImpliedBy(&v8_flags.predictable)) {
+        flags_ignored_because_of_predictable.insert(flag.name());
+      }
+#endif
+      continue;
+    }
     modified_args_as_string << flag;
   }
+
+#ifdef DEBUG
+  for (const char* name : flags_implied_by_predictable) {
+    if (flags_ignored_because_of_predictable.find(name) ==
+        flags_ignored_because_of_predictable.end()) {
+      PrintF(
+          "%s should be added to the list of "
+          "flags_ignored_because_of_predictable\n",
+          name);
+      UNREACHABLE();
+    }
+  }
+#endif
+
   std::string args(modified_args_as_string.str());
   // Generate a hash that is not 0.
   uint32_t hash = static_cast<uint32_t>(base::hash_range(
                       args.c_str(), args.c_str() + args.length())) |
                   1;
   DCHECK_NE(hash, 0);
-  flag_hash.store(hash, std::memory_order_relaxed);
+  return hash;
 }
 
 }  // namespace
@@ -637,8 +747,7 @@ int FlagList::SetFlagsFromCommandLine(int* argc, char** argv, bool remove_flags,
           flag->set_bool_variable(!negated, Flag::SetBy::kCommandLine);
           break;
         case Flag::TYPE_MAYBE_BOOL:
-          flag->set_maybe_bool_variable(MaybeBoolFlag::Create(true, !negated),
-                                        Flag::SetBy::kCommandLine);
+          flag->set_maybe_bool_variable(!negated, Flag::SetBy::kCommandLine);
           break;
         case Flag::TYPE_INT:
           flag->set_int_variable(static_cast<int>(strtol(value, &endp, 10)),
@@ -707,7 +816,7 @@ int FlagList::SetFlagsFromCommandLine(int* argc, char** argv, bool remove_flags,
     }
   }
 
-  if (FLAG_help) {
+  if (v8_flags.help) {
     if (help_options.HasUsage()) {
       PrintF(stdout, "%s", help_options.usage());
     }
@@ -781,10 +890,27 @@ int FlagList::SetFlagsFromString(const char* str, size_t len) {
 }
 
 // static
-void FlagList::ResetAllFlags() {
+void FlagList::FreezeFlags() {
+  // Disallow changes via the API by setting {flags_frozen}.
+  flags_frozen.store(true, std::memory_order_relaxed);
+  // Also memory-protect the memory that holds the flag values. This makes it
+  // impossible for attackers to overwrite values, except if they find a way to
+  // first unprotect the memory again.
+  // Note that for string flags we only protect the pointer itself, but not the
+  // string storage. TODO(12887): Fix this.
+  base::OS::SetDataReadOnly(&v8_flags, sizeof(v8_flags));
+}
+
+// static
+bool FlagList::IsFrozen() {
+  return flags_frozen.load(std::memory_order_relaxed);
+}
+
+// static
+void FlagList::ReleaseDynamicAllocations() {
   flag_hash = 0;
-  for (size_t i = 0; i < num_flags; ++i) {
-    flags[i].Reset();
+  for (size_t i = 0; i < kNumFlags; ++i) {
+    flags[i].ReleaseDynamicAllocations();
   }
 }
 
@@ -805,7 +931,7 @@ void FlagList::PrintHelp() {
   os << "Options:\n";
 
   for (const Flag& f : flags) {
-    os << "  --" << FlagName(f) << " (" << f.comment() << ")\n"
+    os << "  " << FlagName{f.name()} << " (" << f.comment() << ")\n"
        << "        type: " << Type2String(f.type()) << "  default: " << f
        << "\n";
   }
@@ -821,43 +947,125 @@ void FlagList::PrintValues() {
 
 namespace {
 
-template <class A, class B>
-bool TriggerImplication(bool premise, const char* premise_name,
-                        A* conclusion_pointer, B value, bool weak_implication) {
-  if (!premise) return false;
-  bool change_flag = *conclusion_pointer != implicit_cast<A>(value);
-  Flag* conclusion_flag = FindFlagByPointer(conclusion_pointer);
-  change_flag = conclusion_flag->CheckFlagChange(
-      weak_implication ? Flag::SetBy::kWeakImplication
-                       : Flag::SetBy::kImplication,
-      change_flag, premise_name);
-  if (change_flag) *conclusion_pointer = value;
-  return change_flag;
-}
+class ImplicationProcessor {
+ public:
+  // Returns {true} if any flag value was changed.
+  bool EnforceImplications() {
+    bool changed = false;
+#define FLAG_MODE_DEFINE_IMPLICATIONS
+#include "src/flags/flag-definitions.h"  // NOLINT(build/include)
+#undef FLAG_MODE_DEFINE_IMPLICATIONS
+    CheckForCycle();
+    return changed;
+  }
+
+ private:
+  // Called from {DEFINE_*_IMPLICATION} in flag-definitions.h.
+  template <class T>
+  bool TriggerImplication(bool premise, const char* premise_name,
+                          FlagValue<T>* conclusion_value,
+                          const char* conclusion_name, T value,
+                          bool weak_implication) {
+    if (!premise) return false;
+    Flag* conclusion_flag = FindFlagByName(conclusion_name);
+    if (!conclusion_flag->CheckFlagChange(
+            weak_implication ? Flag::SetBy::kWeakImplication
+                             : Flag::SetBy::kImplication,
+            conclusion_value->value() != value, premise_name)) {
+      return false;
+    }
+    if (V8_UNLIKELY(num_iterations_ >= kMaxNumIterations)) {
+      cycle_ << "\n" << FlagName{premise_name} << " -> ";
+      if constexpr (std::is_same_v<T, bool>) {
+        cycle_ << FlagName{conclusion_flag->name(), !value};
+      } else {
+        cycle_ << FlagName{conclusion_flag->name()} << " = " << value;
+      }
+    }
+    *conclusion_value = value;
+    return true;
+  }
+
+  // Called from {DEFINE_*_IMPLICATION} in flag-definitions.h, when the
+  // conclusion flag is read-only (note this is the const overload of the
+  // function just above).
+  template <class T>
+  bool TriggerImplication(bool premise, const char* premise_name,
+                          const FlagValue<T>* conclusion_value,
+                          const char* conclusion_name, T value,
+                          bool weak_implication) {
+    if (!premise) return false;
+    Flag* conclusion_flag = FindFlagByName(conclusion_name);
+    // Because this is the `const FlagValue*` overload:
+    DCHECK(conclusion_flag->IsReadOnly());
+    if (!conclusion_flag->CheckFlagChange(
+            weak_implication ? Flag::SetBy::kWeakImplication
+                             : Flag::SetBy::kImplication,
+            conclusion_value->value() != value, premise_name)) {
+      return false;
+    }
+    // Must equal the default value, otherwise CheckFlagChange should've
+    // returned false.
+    DCHECK_EQ(value, conclusion_flag->GetDefaultValue<T>());
+    return true;
+  }
+
+  void CheckForCycle() {
+    // Make sure flag implications reach a fixed point within
+    // {kMaxNumIterations} iterations.
+    if (++num_iterations_ < kMaxNumIterations) return;
+
+    if (num_iterations_ == kMaxNumIterations) {
+      // Start cycle detection.
+      DCHECK(cycle_.str().empty());
+      cycle_start_hash_ = ComputeFlagListHash();
+      return;
+    }
+
+    DCHECK_NE(0, cycle_start_hash_);
+    // We accept spurious but highly unlikely hash collisions here. This is
+    // only a debug output anyway.
+    if (ComputeFlagListHash() == cycle_start_hash_) {
+      DCHECK(!cycle_.str().empty());
+      // {cycle_} starts with a newline.
+      FATAL("Cycle in flag implications:%s", cycle_.str().c_str());
+    }
+    // We must have found a cycle within another {kMaxNumIterations}.
+    DCHECK_GE(2 * kMaxNumIterations, num_iterations_);
+  }
+
+  static constexpr size_t kMaxNumIterations = kNumFlags;
+  size_t num_iterations_ = 0;
+  // After {kMaxNumIterations} we use the following two fields for finding
+  // cycles in flags.
+  uint32_t cycle_start_hash_;
+  std::ostringstream cycle_;
+};
 
 }  // namespace
 
 // static
 void FlagList::EnforceFlagImplications() {
-  flag_hash = 0;
-  bool changed;
-  do {
-    changed = false;
-#define FLAG_MODE_DEFINE_IMPLICATIONS
-#include "src/flags/flag-definitions.h"  // NOLINT(build/include)
-#undef FLAG_MODE_DEFINE_IMPLICATIONS
-  } while (changed);
+  for (ImplicationProcessor proc; proc.EnforceImplications();) {
+    // Continue processing (recursive) implications. The processor has an
+    // internal limit to avoid endless recursion.
+  }
 }
 
 // static
 uint32_t FlagList::Hash() {
-  if (flag_hash.load() == 0) ComputeFlagListHash();
-  return flag_hash.load();
+  if (uint32_t hash = flag_hash.load(std::memory_order_relaxed)) return hash;
+  uint32_t hash = ComputeFlagListHash();
+  flag_hash.store(hash, std::memory_order_relaxed);
+  return hash;
 }
 
-#undef FLAG_MODE_DEFINE
-#undef FLAG_MODE_DEFINE_DEFAULTS
-#undef FLAG_MODE_META
+// static
+void FlagList::ResetFlagHash() {
+  // If flags are frozen, we should not need to reset the hash since we cannot
+  // change flag values anyway.
+  CHECK(!IsFrozen());
+  flag_hash = 0;
+}
 
-}  // namespace internal
-}  // namespace v8
+}  // namespace v8::internal
