@@ -5,6 +5,7 @@
 #ifndef V8_IC_STUB_CACHE_H_
 #define V8_IC_STUB_CACHE_H_
 
+#include "include/v8-callbacks.h"
 #include "src/objects/name.h"
 #include "src/objects/tagged-value.h"
 
@@ -16,7 +17,6 @@ namespace internal {
 // need explicit invalidation when a prototype chain is modified, since the
 // handlers verify the chain.
 
-
 class SCTableReference {
  public:
   Address address() const { return address_; }
@@ -78,28 +78,25 @@ class V8_EXPORT_PRIVATE StubCache {
 
   Isolate* isolate() { return isolate_; }
 
-  // Setting the entry size such that the index is shifted by Name::kHashShift
-  // is convenient; shifting down the length field (to extract the hash code)
-  // automatically discards the hash bit field.
-  static const int kCacheIndexShift = Name::kHashShift;
+  // Setting kCacheIndexShift to Name::HashBits::kShift is convenient because it
+  // causes the bit field inside the hash field to get shifted out implicitly.
+  // Note that kCacheIndexShift must not get too large, because
+  // sizeof(Entry) needs to be a multiple of 1 << kCacheIndexShift (see
+  // the static_assert below, in {entry(...)}).
+  static const int kCacheIndexShift = Name::HashBits::kShift;
 
   static const int kPrimaryTableBits = 11;
   static const int kPrimaryTableSize = (1 << kPrimaryTableBits);
   static const int kSecondaryTableBits = 9;
   static const int kSecondaryTableSize = (1 << kSecondaryTableBits);
 
-  // We compute the hash code for a map as follows:
-  //   <code> = <address> ^ (<address> >> kMapKeyShift)
-  static const int kMapKeyShift = kPrimaryTableBits + kCacheIndexShift;
-
-  // Some magic number used in the secondary hash computation.
-  static const int kSecondaryMagic = 0xb16ca6e5;
-
   static int PrimaryOffsetForTesting(Name name, Map map);
-  static int SecondaryOffsetForTesting(Name name, int seed);
+  static int SecondaryOffsetForTesting(Name name, Map map);
 
   // The constructor is made public only for the purposes of testing.
   explicit StubCache(Isolate* isolate);
+  StubCache(const StubCache&) = delete;
+  StubCache& operator=(const StubCache&) = delete;
 
  private:
   // The stub cache has a primary and secondary level.  The two levels have
@@ -117,15 +114,18 @@ class V8_EXPORT_PRIVATE StubCache {
   // Hash algorithm for the secondary table.  This algorithm is replicated in
   // assembler for every architecture.  Returns an index into the table that
   // is scaled by 1 << kCacheIndexShift.
-  static int SecondaryOffset(Name name, int seed);
+  static int SecondaryOffset(Name name, Map map);
 
   // Compute the entry for a given offset in exactly the same way as
   // we do in generated code.  We generate an hash code that already
-  // ends in Name::kHashShift 0s.  Then we multiply it so it is a multiple
+  // ends in Name::HashBits::kShift 0s.  Then we multiply it so it is a multiple
   // of sizeof(Entry).  This makes it easier to avoid making mistakes
   // in the hashed offset computations.
   static Entry* entry(Entry* table, int offset) {
-    const int multiplier = sizeof(*table) >> Name::kHashShift;
+    // The size of {Entry} must be a multiple of 1 << kCacheIndexShift.
+    static_assert((sizeof(*table) >> kCacheIndexShift) << kCacheIndexShift ==
+                  sizeof(*table));
+    const int multiplier = sizeof(*table) >> kCacheIndexShift;
     return reinterpret_cast<Entry*>(reinterpret_cast<Address>(table) +
                                     offset * multiplier);
   }
@@ -137,8 +137,6 @@ class V8_EXPORT_PRIVATE StubCache {
 
   friend class Isolate;
   friend class SCTableReference;
-
-  DISALLOW_COPY_AND_ASSIGN(StubCache);
 };
 }  // namespace internal
 }  // namespace v8
