@@ -68,30 +68,47 @@ using protocol::Runtime::PrivatePropertyDescriptor;
 using protocol::Runtime::PropertyDescriptor;
 using protocol::Runtime::RemoteObject;
 
+// static
+void EvaluateCallback::sendSuccess(
+    std::weak_ptr<EvaluateCallback> callback, InjectedScript* injectedScript,
+    std::unique_ptr<protocol::Runtime::RemoteObject> result,
+    protocol::Maybe<protocol::Runtime::ExceptionDetails> exceptionDetails) {
+  std::shared_ptr<EvaluateCallback> cb = callback.lock();
+  if (!cb) return;
+  injectedScript->deleteEvaluateCallback(cb);
+  CHECK_EQ(cb.use_count(), 1);
+  cb->sendSuccess(std::move(result), std::move(exceptionDetails));
+}
+
+// static
+void EvaluateCallback::sendFailure(std::weak_ptr<EvaluateCallback> callback,
+                                   InjectedScript* injectedScript,
+                                   const protocol::DispatchResponse& response) {
+  std::shared_ptr<EvaluateCallback> cb = callback.lock();
+  if (!cb) return;
+  injectedScript->deleteEvaluateCallback(cb);
+  CHECK_EQ(cb.use_count(), 1);
+  cb->sendFailure(response);
+}
+
 class InjectedScript::ProtocolPromiseHandler {
  public:
-  static bool add(V8InspectorSessionImpl* session,
+  static void add(V8InspectorSessionImpl* session,
                   v8::Local<v8::Context> context, v8::Local<v8::Value> value,
                   int executionContextId, const String16& objectGroup,
-                  WrapMode wrapMode, bool replMode,
-                  EvaluateCallback* callback) {
-    v8::Local<v8::Promise::Resolver> resolver;
-    if (!v8::Promise::Resolver::New(context).ToLocal(&resolver)) {
-      callback->sendFailure(Response::InternalError());
-      return false;
-    }
-    if (!resolver->Resolve(context, value).FromMaybe(false)) {
-      callback->sendFailure(Response::InternalError());
-      return false;
-    }
+                  WrapMode wrapMode, bool replMode, bool throwOnSideEffect,
+                  std::weak_ptr<EvaluateCallback> callback) {
+    InjectedScript::ContextScope scope(session, executionContextId);
+    Response response = scope.initialize();
+    if (!response.IsSuccess()) return;
 
     v8::MaybeLocal<v8::Promise> originalPromise =
         value->IsPromise() ? value.As<v8::Promise>()
                            : v8::MaybeLocal<v8::Promise>();
     V8InspectorImpl* inspector = session->inspector();
     ProtocolPromiseHandler* handler = new ProtocolPromiseHandler(
-        session, executionContextId, objectGroup, wrapMode, replMode, callback,
-        originalPromise);
+        session, executionContextId, objectGroup, wrapMode, replMode,
+        throwOnSideEffect, callback, originalPromise);
     v8::Local<v8::Value> wrapper = handler->m_wrapper.Get(inspector->isolate());
     v8::Local<v8::Function> thenCallbackFunction =
         v8::Function::New(context, thenCallback, wrapper, 0,
@@ -101,13 +118,36 @@ class InjectedScript::ProtocolPromiseHandler {
         v8::Function::New(context, catchCallback, wrapper, 0,
                           v8::ConstructorBehavior::kThrow)
             .ToLocalChecked();
-    v8::Local<v8::Promise> promise = resolver->GetPromise();
+
+    v8::Local<v8::Promise> promise;
+    v8::Local<v8::Promise::Resolver> resolver;
+    if (value->IsPromise()) {
+      // If value is a promise, we can chain the handlers directly onto `value`.
+      promise = value.As<v8::Promise>();
+    } else {
+      // Otherwise we do `Promise.resolve(value)`.
+      CHECK(!replMode);
+      if (!v8::Promise::Resolver::New(context).ToLocal(&resolver)) {
+        EvaluateCallback::sendFailure(callback, scope.injectedScript(),
+                                      Response::InternalError());
+        return;
+      }
+      if (!resolver->Resolve(context, value).FromMaybe(false)) {
+        EvaluateCallback::sendFailure(callback, scope.injectedScript(),
+                                      Response::InternalError());
+        return;
+      }
+      promise = resolver->GetPromise();
+    }
+
     if (promise->Then(context, thenCallbackFunction, catchCallbackFunction)
             .IsEmpty()) {
-      callback->sendFailure(Response::InternalError());
-      return false;
+      // Re-initialize after returning from JS.
+      Response response = scope.initialize();
+      if (!response.IsSuccess()) return;
+      EvaluateCallback::sendFailure(callback, scope.injectedScript(),
+                                    Response::InternalError());
     }
-    return true;
   }
 
  private:
@@ -143,7 +183,8 @@ class InjectedScript::ProtocolPromiseHandler {
   ProtocolPromiseHandler(V8InspectorSessionImpl* session,
                          int executionContextId, const String16& objectGroup,
                          WrapMode wrapMode, bool replMode,
-                         EvaluateCallback* callback,
+                         bool throwOnSideEffect,
+                         std::weak_ptr<EvaluateCallback> callback,
                          v8::MaybeLocal<v8::Promise> maybeEvaluationResult)
       : m_inspector(session->inspector()),
         m_sessionId(session->sessionId()),
@@ -152,6 +193,7 @@ class InjectedScript::ProtocolPromiseHandler {
         m_objectGroup(objectGroup),
         m_wrapMode(wrapMode),
         m_replMode(replMode),
+        m_throwOnSideEffect(throwOnSideEffect),
         m_callback(std::move(callback)),
         m_wrapper(m_inspector->isolate(),
                   v8::External::New(m_inspector->isolate(), this)) {
@@ -183,24 +225,22 @@ class InjectedScript::ProtocolPromiseHandler {
     Response response = scope.initialize();
     if (!response.IsSuccess()) return;
 
-    std::unique_ptr<EvaluateCallback> callback =
-        scope.injectedScript()->takeEvaluateCallback(m_callback);
-    if (!callback) return;
-
     // In REPL mode the result is additionally wrapped in an object.
     // The evaluation result can be found at ".repl_result".
     v8::Local<v8::Value> result = value;
     if (m_replMode) {
       v8::Local<v8::Object> object;
       if (!result->ToObject(scope.context()).ToLocal(&object)) {
-        callback->sendFailure(response);
+        EvaluateCallback::sendFailure(m_callback, scope.injectedScript(),
+                                      response);
         return;
       }
 
       v8::Local<v8::String> name =
           GetDotReplResultString(m_inspector->isolate());
       if (!object->Get(scope.context(), name).ToLocal(&result)) {
-        callback->sendFailure(response);
+        EvaluateCallback::sendFailure(m_callback, scope.injectedScript(),
+                                      response);
         return;
       }
     }
@@ -213,11 +253,13 @@ class InjectedScript::ProtocolPromiseHandler {
     response = scope.injectedScript()->wrapObject(result, m_objectGroup,
                                                   m_wrapMode, &wrappedValue);
     if (!response.IsSuccess()) {
-      callback->sendFailure(response);
+      EvaluateCallback::sendFailure(m_callback, scope.injectedScript(),
+                                    response);
       return;
     }
-    callback->sendSuccess(std::move(wrappedValue),
-                          Maybe<protocol::Runtime::ExceptionDetails>());
+    EvaluateCallback::sendSuccess(m_callback, scope.injectedScript(),
+                                  std::move(wrappedValue),
+                                  Maybe<protocol::Runtime::ExceptionDetails>());
   }
 
   void catchCallback(v8::Local<v8::Value> result) {
@@ -227,14 +269,12 @@ class InjectedScript::ProtocolPromiseHandler {
     InjectedScript::ContextScope scope(session, m_executionContextId);
     Response response = scope.initialize();
     if (!response.IsSuccess()) return;
-    std::unique_ptr<EvaluateCallback> callback =
-        scope.injectedScript()->takeEvaluateCallback(m_callback);
-    if (!callback) return;
     std::unique_ptr<protocol::Runtime::RemoteObject> wrappedValue;
     response = scope.injectedScript()->wrapObject(result, m_objectGroup,
                                                   m_wrapMode, &wrappedValue);
     if (!response.IsSuccess()) {
-      callback->sendFailure(response);
+      EvaluateCallback::sendFailure(m_callback, scope.injectedScript(),
+                                    response);
       return;
     }
     v8::Isolate* isolate = session->inspector()->isolate();
@@ -249,18 +289,22 @@ class InjectedScript::ProtocolPromiseHandler {
     // we try to capture a fresh stack trace.
     if (maybeMessage.ToLocal(&message)) {
       v8::Local<v8::Value> exception = result;
-      session->inspector()->client()->dispatchError(scope.context(), message,
-                                                    exception);
+      if (!m_throwOnSideEffect) {
+        session->inspector()->client()->dispatchError(scope.context(), message,
+                                                      exception);
+      }
       protocol::PtrMaybe<protocol::Runtime::ExceptionDetails> exceptionDetails;
       response = scope.injectedScript()->createExceptionDetails(
           message, exception, m_objectGroup, &exceptionDetails);
       if (!response.IsSuccess()) {
-        callback->sendFailure(response);
+        EvaluateCallback::sendFailure(m_callback, scope.injectedScript(),
+                                      response);
         return;
       }
 
-      callback->sendSuccess(std::move(wrappedValue),
-                            std::move(exceptionDetails));
+      EvaluateCallback::sendSuccess(m_callback, scope.injectedScript(),
+                                    std::move(wrappedValue),
+                                    std::move(exceptionDetails));
       return;
     }
 
@@ -301,7 +345,8 @@ class InjectedScript::ProtocolPromiseHandler {
     response = scope.injectedScript()->addExceptionToDetails(
         result, exceptionDetails.get(), m_objectGroup);
     if (!response.IsSuccess()) {
-      callback->sendFailure(response);
+      EvaluateCallback::sendFailure(m_callback, scope.injectedScript(),
+                                    response);
       return;
     }
     if (stack)
@@ -310,7 +355,9 @@ class InjectedScript::ProtocolPromiseHandler {
     if (stack && !stack->isEmpty())
       exceptionDetails->setScriptId(
           String16::fromInteger(stack->topScriptId()));
-    callback->sendSuccess(std::move(wrappedValue), std::move(exceptionDetails));
+    EvaluateCallback::sendSuccess(m_callback, scope.injectedScript(),
+                                  std::move(wrappedValue),
+                                  std::move(exceptionDetails));
   }
 
   void sendPromiseCollected() {
@@ -320,10 +367,9 @@ class InjectedScript::ProtocolPromiseHandler {
     InjectedScript::ContextScope scope(session, m_executionContextId);
     Response response = scope.initialize();
     if (!response.IsSuccess()) return;
-    std::unique_ptr<EvaluateCallback> callback =
-        scope.injectedScript()->takeEvaluateCallback(m_callback);
-    if (!callback) return;
-    callback->sendFailure(Response::ServerError("Promise was collected"));
+    EvaluateCallback::sendFailure(
+        m_callback, scope.injectedScript(),
+        Response::ServerError("Promise was collected"));
   }
 
   V8InspectorImpl* m_inspector;
@@ -333,7 +379,8 @@ class InjectedScript::ProtocolPromiseHandler {
   String16 m_objectGroup;
   WrapMode m_wrapMode;
   bool m_replMode;
-  EvaluateCallback* m_callback;
+  bool m_throwOnSideEffect;
+  std::weak_ptr<EvaluateCallback> m_callback;
   v8::Global<v8::External> m_wrapper;
   v8::Global<v8::Promise> m_evaluationResult;
 };
@@ -576,8 +623,8 @@ Response InjectedScript::wrapObjectMirror(
   }
   if (wrapMode == WrapMode::kGenerateWebDriverValue) {
     int maxDepth = 1;
-    std::unique_ptr<protocol::Runtime::WebDriverValue> webDriverValue;
-    response = mirror.buildWebDriverValue(context, maxDepth, &webDriverValue);
+    std::unique_ptr<protocol::Runtime::WebDriverValue> webDriverValue =
+        mirror.buildWebDriverValue(context, maxDepth);
     if (!response.IsSuccess()) return response;
     (*result)->setWebDriverValue(std::move(webDriverValue));
   }
@@ -639,7 +686,7 @@ std::unique_ptr<protocol::Runtime::RemoteObject> InjectedScript::wrapTable(
       auto filtered = std::make_unique<Array<PropertyPreview>>();
       for (const String16& column : selectedColumns) {
         if (columnMap.find(column) == columnMap.end()) continue;
-        filtered->push_back(columnMap[column]->clone());
+        filtered->push_back(columnMap[column]->Clone());
       }
       columnPreview->setProperties(std::move(filtered));
     }
@@ -651,37 +698,46 @@ std::unique_ptr<protocol::Runtime::RemoteObject> InjectedScript::wrapTable(
 void InjectedScript::addPromiseCallback(
     V8InspectorSessionImpl* session, v8::MaybeLocal<v8::Value> value,
     const String16& objectGroup, WrapMode wrapMode, bool replMode,
-    std::unique_ptr<EvaluateCallback> callback) {
+    bool throwOnSideEffect, std::shared_ptr<EvaluateCallback> callback) {
+  m_evaluateCallbacks.insert(callback);
+  // After stashing the shared_ptr in `m_evaluateCallback`, we reset `callback`.
+  // `ProtocolPromiseHandler:add` can take longer than the life time of this
+  // `InjectedScript` and we don't want `callback` to survive that.
+  std::weak_ptr<EvaluateCallback> weak_callback = callback;
+  callback.reset();
+  CHECK_EQ(weak_callback.use_count(), 1);
+
   if (value.IsEmpty()) {
-    callback->sendFailure(Response::InternalError());
+    EvaluateCallback::sendFailure(weak_callback, this,
+                                  Response::InternalError());
     return;
   }
-  v8::MicrotasksScope microtasksScope(m_context->isolate(),
+
+  v8::MicrotasksScope microtasksScope(m_context->context(),
                                       v8::MicrotasksScope::kRunMicrotasks);
-  if (ProtocolPromiseHandler::add(session, m_context->context(),
-                                  value.ToLocalChecked(),
-                                  m_context->contextId(), objectGroup, wrapMode,
-                                  replMode, callback.get())) {
-    m_evaluateCallbacks.insert(callback.release());
-  }
+  ProtocolPromiseHandler::add(session, m_context->context(),
+                              value.ToLocalChecked(), m_context->contextId(),
+                              objectGroup, wrapMode, replMode,
+                              throwOnSideEffect, weak_callback);
+  // Do not add any code here! `this` might be invalid.
+  // `ProtocolPromiseHandler::add` calls into JS which could kill this
+  // `InjectedScript`.
 }
 
 void InjectedScript::discardEvaluateCallbacks() {
-  for (auto& callback : m_evaluateCallbacks) {
-    callback->sendFailure(
+  while (!m_evaluateCallbacks.empty()) {
+    EvaluateCallback::sendFailure(
+        *m_evaluateCallbacks.begin(), this,
         Response::ServerError("Execution context was destroyed."));
-    delete callback;
   }
-  m_evaluateCallbacks.clear();
+  CHECK(m_evaluateCallbacks.empty());
 }
 
-std::unique_ptr<EvaluateCallback> InjectedScript::takeEvaluateCallback(
-    EvaluateCallback* callback) {
+void InjectedScript::deleteEvaluateCallback(
+    std::shared_ptr<EvaluateCallback> callback) {
   auto it = m_evaluateCallbacks.find(callback);
-  if (it == m_evaluateCallbacks.end()) return nullptr;
-  std::unique_ptr<EvaluateCallback> value(*it);
+  CHECK_NE(it, m_evaluateCallbacks.end());
   m_evaluateCallbacks.erase(it);
-  return value;
 }
 
 Response InjectedScript::findObject(const RemoteObjectId& objectId,
@@ -840,7 +896,7 @@ Response InjectedScript::createExceptionDetails(
 
 Response InjectedScript::wrapEvaluateResult(
     v8::MaybeLocal<v8::Value> maybeResultValue, const v8::TryCatch& tryCatch,
-    const String16& objectGroup, WrapMode wrapMode,
+    const String16& objectGroup, WrapMode wrapMode, bool throwOnSideEffect,
     std::unique_ptr<protocol::Runtime::RemoteObject>* result,
     Maybe<protocol::Runtime::ExceptionDetails>* exceptionDetails) {
   v8::Local<v8::Value> resultValue;
@@ -858,8 +914,10 @@ Response InjectedScript::wrapEvaluateResult(
       return Response::ServerError("Execution was terminated");
     }
     v8::Local<v8::Value> exception = tryCatch.Exception();
-    m_context->inspector()->client()->dispatchError(
-        m_context->context(), tryCatch.Message(), exception);
+    if (!throwOnSideEffect) {
+      m_context->inspector()->client()->dispatchError(
+          m_context->context(), tryCatch.Message(), exception);
+    }
     Response response =
         wrapObject(exception, objectGroup,
                    exception->IsNativeError() ? WrapMode::kNoPreview
@@ -914,6 +972,11 @@ Response InjectedScript::Scope::initialize() {
 void InjectedScript::Scope::installCommandLineAPI() {
   DCHECK(m_injectedScript && !m_context.IsEmpty() &&
          !m_commandLineAPIScope.get());
+  V8InspectorSessionImpl* session =
+      m_inspector->sessionById(m_contextGroupId, m_sessionId);
+  if (session->clientTrustLevel() != V8Inspector::kFullyTrusted) {
+    return;
+  }
   m_commandLineAPIScope.reset(new V8Console::CommandLineAPIScope(
       m_context, m_injectedScript->commandLineAPI(), m_context->Global()));
 }
