@@ -5,6 +5,8 @@
 #ifndef V8_INSPECTOR_REMOTE_OBJECT_ID_H_
 #define V8_INSPECTOR_REMOTE_OBJECT_ID_H_
 
+#include <memory>
+
 #include "src/inspector/protocol/Forward.h"
 
 namespace v8_inspector {
@@ -13,16 +15,18 @@ using protocol::Response;
 
 class RemoteObjectIdBase {
  public:
+  uint64_t isolateId() const { return m_isolateId; }
   int contextId() const { return m_injectedScriptId; }
 
  protected:
   RemoteObjectIdBase();
   ~RemoteObjectIdBase() = default;
 
-  std::unique_ptr<protocol::DictionaryValue> parseInjectedScriptId(
-      const String16&);
+  bool parseId(const String16&);
 
+  uint64_t m_isolateId;
   int m_injectedScriptId;
+  int m_id;
 };
 
 class RemoteObjectId final : public RemoteObjectIdBase {
@@ -31,10 +35,7 @@ class RemoteObjectId final : public RemoteObjectIdBase {
   ~RemoteObjectId() = default;
   int id() const { return m_id; }
 
- private:
-  RemoteObjectId();
-
-  int m_id;
+  static String16 serialize(uint64_t isolateId, int injectedScriptId, int id);
 };
 
 class RemoteCallFrameId final : public RemoteObjectIdBase {
@@ -42,14 +43,10 @@ class RemoteCallFrameId final : public RemoteObjectIdBase {
   static Response parse(const String16&, std::unique_ptr<RemoteCallFrameId>*);
   ~RemoteCallFrameId() = default;
 
-  int frameOrdinal() const { return m_frameOrdinal; }
+  int frameOrdinal() const { return m_id; }
 
-  static String16 serialize(int injectedScriptId, int frameOrdinal);
-
- private:
-  RemoteCallFrameId();
-
-  int m_frameOrdinal;
+  static String16 serialize(uint64_t isolateId, int injectedScriptId,
+                            int frameOrdinal);
 };
 
 }  // namespace v8_inspector
