@@ -31,20 +31,6 @@ FeedbackSlot FeedbackVectorSpec::AddSlot(FeedbackSlotKind kind) {
   return FeedbackSlot(slot);
 }
 
-FeedbackSlot FeedbackVectorSpec::AddTypeProfileSlot() {
-  FeedbackSlot slot = AddSlot(FeedbackSlotKind::kTypeProfile);
-  CHECK_EQ(FeedbackVectorSpec::kTypeProfileSlotIndex,
-           FeedbackVector::GetIndex(slot));
-  return slot;
-}
-
-bool FeedbackVectorSpec::HasTypeProfileSlot() const {
-  FeedbackSlot slot =
-      FeedbackVector::ToSlot(FeedbackVectorSpec::kTypeProfileSlotIndex);
-  if (slot_count() <= slot.ToInt()) return false;
-  return GetKind(slot) == FeedbackSlotKind::kTypeProfile;
-}
-
 static bool IsPropertyNameFeedback(MaybeObject feedback) {
   HeapObject heap_object;
   if (!feedback->GetHeapObjectIfStrong(&heap_object)) return false;
@@ -184,25 +170,15 @@ const char* FeedbackMetadata::Kind2String(FeedbackSlotKind kind) {
       return "DefineKeyedOwnPropertyInLiteral";
     case FeedbackSlotKind::kLiteral:
       return "Literal";
-    case FeedbackSlotKind::kTypeProfile:
-      return "TypeProfile";
     case FeedbackSlotKind::kForIn:
       return "ForIn";
     case FeedbackSlotKind::kInstanceOf:
       return "InstanceOf";
     case FeedbackSlotKind::kCloneObject:
       return "CloneObject";
-    case FeedbackSlotKind::kKindsNumber:
-      break;
+    case FeedbackSlotKind::kJumpLoop:
+      return "JumpLoop";
   }
-  UNREACHABLE();
-}
-
-bool FeedbackMetadata::HasTypeProfileSlot() const {
-  FeedbackSlot slot =
-      FeedbackVector::ToSlot(FeedbackVectorSpec::kTypeProfileSlotIndex);
-  return slot.ToInt() < slot_count() &&
-         GetKind(slot) == FeedbackSlotKind::kTypeProfile;
 }
 
 FeedbackSlotKind FeedbackVector::GetKind(FeedbackSlot slot) const {
@@ -216,14 +192,6 @@ FeedbackSlotKind FeedbackVector::GetKind(FeedbackSlot slot,
   return metadata(tag).GetKind(slot);
 }
 
-FeedbackSlot FeedbackVector::GetTypeProfileSlot() const {
-  DCHECK(metadata().HasTypeProfileSlot());
-  FeedbackSlot slot =
-      FeedbackVector::ToSlot(FeedbackVectorSpec::kTypeProfileSlotIndex);
-  DCHECK_EQ(FeedbackSlotKind::kTypeProfile, GetKind(slot));
-  return slot;
-}
-
 // static
 Handle<ClosureFeedbackCellArray> ClosureFeedbackCellArray::New(
     Isolate* isolate, Handle<SharedFunctionInfo> shared) {
@@ -247,6 +215,7 @@ Handle<ClosureFeedbackCellArray> ClosureFeedbackCellArray::New(
 Handle<FeedbackVector> FeedbackVector::New(
     Isolate* isolate, Handle<SharedFunctionInfo> shared,
     Handle<ClosureFeedbackCellArray> closure_feedback_cell_array,
+    Handle<FeedbackCell> parent_feedback_cell,
     IsCompiledScope* is_compiled_scope) {
   DCHECK(is_compiled_scope->is_compiled());
   Factory* factory = isolate->factory();
@@ -255,14 +224,15 @@ Handle<FeedbackVector> FeedbackVector::New(
                                              isolate);
   const int slot_count = feedback_metadata->slot_count();
 
-  Handle<FeedbackVector> vector =
-      factory->NewFeedbackVector(shared, closure_feedback_cell_array);
+  Handle<FeedbackVector> vector = factory->NewFeedbackVector(
+      shared, closure_feedback_cell_array, parent_feedback_cell);
 
   DCHECK_EQ(vector->length(), slot_count);
 
   DCHECK_EQ(vector->shared_function_info(), *shared);
   DCHECK_EQ(vector->tiering_state(), TieringState::kNone);
-  DCHECK(!vector->maybe_has_optimized_code());
+  DCHECK(!vector->maybe_has_maglev_code());
+  DCHECK(!vector->maybe_has_turbofan_code());
   DCHECK_EQ(vector->invocation_count(), 0);
   DCHECK_EQ(vector->profiler_ticks(), 0);
   DCHECK(vector->maybe_optimized_code()->IsCleared());
@@ -282,6 +252,7 @@ Handle<FeedbackVector> FeedbackVector::New(
       case FeedbackSlotKind::kLoadGlobalNotInsideTypeof:
       case FeedbackSlotKind::kStoreGlobalSloppy:
       case FeedbackSlotKind::kStoreGlobalStrict:
+      case FeedbackSlotKind::kJumpLoop:
         vector->Set(slot, HeapObjectReference::ClearedValue(isolate),
                     SKIP_WRITE_BARRIER);
         break;
@@ -309,13 +280,11 @@ Handle<FeedbackVector> FeedbackVector::New(
       case FeedbackSlotKind::kSetKeyedStrict:
       case FeedbackSlotKind::kStoreInArrayLiteral:
       case FeedbackSlotKind::kDefineKeyedOwnPropertyInLiteral:
-      case FeedbackSlotKind::kTypeProfile:
       case FeedbackSlotKind::kInstanceOf:
         vector->Set(slot, *uninitialized_sentinel, SKIP_WRITE_BARRIER);
         break;
 
       case FeedbackSlotKind::kInvalid:
-      case FeedbackSlotKind::kKindsNumber:
         UNREACHABLE();
     }
     for (int j = 1; j < entry_size; j++) {
@@ -324,17 +293,15 @@ Handle<FeedbackVector> FeedbackVector::New(
     i += entry_size;
   }
 
-  Handle<FeedbackVector> result = Handle<FeedbackVector>::cast(vector);
-  if (!isolate->is_best_effort_code_coverage() ||
-      isolate->is_collecting_type_profile()) {
-    AddToVectorsForProfilingTools(isolate, result);
+  if (!isolate->is_best_effort_code_coverage()) {
+    AddToVectorsForProfilingTools(isolate, vector);
   }
-  return result;
+  parent_feedback_cell->set_value(*vector, kReleaseStore);
+  return vector;
 }
 
-namespace {
-
-Handle<FeedbackVector> NewFeedbackVectorForTesting(
+// static
+Handle<FeedbackVector> FeedbackVector::NewForTesting(
     Isolate* isolate, const FeedbackVectorSpec* spec) {
   Handle<FeedbackMetadata> metadata = FeedbackMetadata::New(isolate, spec);
   Handle<SharedFunctionInfo> shared =
@@ -345,20 +312,20 @@ Handle<FeedbackVector> NewFeedbackVectorForTesting(
   shared->set_raw_outer_scope_info_or_feedback_metadata(*metadata);
   Handle<ClosureFeedbackCellArray> closure_feedback_cell_array =
       ClosureFeedbackCellArray::New(isolate, shared);
+  Handle<FeedbackCell> parent_cell = isolate->factory()->NewNoClosuresCell(
+      isolate->factory()->undefined_value());
 
   IsCompiledScope is_compiled_scope(shared->is_compiled_scope(isolate));
   return FeedbackVector::New(isolate, shared, closure_feedback_cell_array,
-                             &is_compiled_scope);
+                             parent_cell, &is_compiled_scope);
 }
 
-}  // namespace
-
 // static
 Handle<FeedbackVector> FeedbackVector::NewWithOneBinarySlotForTesting(
     Zone* zone, Isolate* isolate) {
   FeedbackVectorSpec one_slot(zone);
   one_slot.AddBinaryOpICSlot();
-  return NewFeedbackVectorForTesting(isolate, &one_slot);
+  return NewForTesting(isolate, &one_slot);
 }
 
 // static
@@ -366,14 +333,13 @@ Handle<FeedbackVector> FeedbackVector::NewWithOneCompareSlotForTesting(
     Zone* zone, Isolate* isolate) {
   FeedbackVectorSpec one_slot(zone);
   one_slot.AddCompareICSlot();
-  return NewFeedbackVectorForTesting(isolate, &one_slot);
+  return NewForTesting(isolate, &one_slot);
 }
 
 // static
 void FeedbackVector::AddToVectorsForProfilingTools(
     Isolate* isolate, Handle<FeedbackVector> vector) {
-  DCHECK(!isolate->is_best_effort_code_coverage() ||
-         isolate->is_collecting_type_profile());
+  DCHECK(!isolate->is_best_effort_code_coverage());
   if (!vector->shared_function_info().IsSubjectToDebugging()) return;
   Handle<ArrayList> list = Handle<ArrayList>::cast(
       isolate->factory()->feedback_vectors_for_profiling_tools());
@@ -386,30 +352,48 @@ void FeedbackVector::SaturatingIncrementProfilerTicks() {
   if (ticks < Smi::kMaxValue) set_profiler_ticks(ticks + 1);
 }
 
-void FeedbackVector::SetOptimizedCode(Handle<CodeT> code) {
-  DCHECK(CodeKindIsOptimizedJSFunction(code->kind()));
+void FeedbackVector::SetOptimizedCode(Code code) {
+  DCHECK(CodeKindIsOptimizedJSFunction(code.kind()));
   // We should set optimized code only when there is no valid optimized code.
   DCHECK(!has_optimized_code() ||
          optimized_code().marked_for_deoptimization() ||
-         FLAG_stress_concurrent_inlining_attach_code);
+         (CodeKindCanTierUp(optimized_code().kind()) &&
+          optimized_code().kind() < code.kind()) ||
+         v8_flags.stress_concurrent_inlining_attach_code);
   // TODO(mythria): We could see a CompileOptimized state here either from
-  // tests that use %OptimizeFunctionOnNextCall, --always-opt or because we
+  // tests that use %OptimizeFunctionOnNextCall, --always-turbofan or because we
   // re-mark the function for non-concurrent optimization after an OSR. We
   // should avoid these cases and also check that marker isn't
   // TieringState::kRequestTurbofan*.
-  set_maybe_optimized_code(HeapObjectReference::Weak(*code), kReleaseStore);
+  set_maybe_optimized_code(HeapObjectReference::Weak(code));
   int32_t state = flags();
+  // TODO(leszeks): Reconsider whether this could clear the tiering state vs.
+  // the callers doing so.
   state = TieringStateBits::update(state, TieringState::kNone);
-  state = MaybeHasOptimizedCodeBit::update(state, true);
+  if (code.is_maglevved()) {
+    DCHECK(!MaybeHasTurbofanCodeBit::decode(state));
+    state = MaybeHasMaglevCodeBit::update(state, true);
+  } else {
+    DCHECK(code.is_turbofanned());
+    state = MaybeHasTurbofanCodeBit::update(state, true);
+    state = MaybeHasMaglevCodeBit::update(state, false);
+  }
   set_flags(state);
 }
 
 void FeedbackVector::ClearOptimizedCode() {
   DCHECK(has_optimized_code());
-  DCHECK(maybe_has_optimized_code());
-  set_maybe_optimized_code(HeapObjectReference::ClearedValue(GetIsolate()),
-                           kReleaseStore);
-  set_maybe_has_optimized_code(false);
+  DCHECK(maybe_has_maglev_code() || maybe_has_turbofan_code());
+  set_maybe_optimized_code(HeapObjectReference::ClearedValue(GetIsolate()));
+  set_maybe_has_maglev_code(false);
+  set_maybe_has_turbofan_code(false);
+}
+
+void FeedbackVector::SetOptimizedOsrCode(FeedbackSlot slot, Code code) {
+  DCHECK(CodeKindIsOptimizedJSFunction(code.kind()));
+  DCHECK(!slot.IsInvalid());
+  Set(slot, HeapObjectReference::Weak(code));
+  set_maybe_has_optimized_osr_code(true);
 }
 
 void FeedbackVector::reset_tiering_state() {
@@ -424,8 +408,11 @@ void FeedbackVector::set_tiering_state(TieringState state) {
 
 void FeedbackVector::reset_flags() {
   set_flags(TieringStateBits::encode(TieringState::kNone) |
+            LogNextExecutionBit::encode(false) |
+            MaybeHasMaglevCodeBit::encode(false) |
+            MaybeHasTurbofanCodeBit::encode(false) |
             OsrTieringStateBit::encode(TieringState::kNone) |
-            MaybeHasOptimizedCodeBit::encode(false));
+            MaybeHasOptimizedOsrCodeBit::encode(false));
 }
 
 TieringState FeedbackVector::osr_tiering_state() {
@@ -434,29 +421,30 @@ TieringState FeedbackVector::osr_tiering_state() {
 
 void FeedbackVector::set_osr_tiering_state(TieringState marker) {
   DCHECK(marker == TieringState::kNone || marker == TieringState::kInProgress);
-  STATIC_ASSERT(TieringState::kNone <= OsrTieringStateBit::kMax);
-  STATIC_ASSERT(TieringState::kInProgress <= OsrTieringStateBit::kMax);
+  static_assert(TieringState::kNone <= OsrTieringStateBit::kMax);
+  static_assert(TieringState::kInProgress <= OsrTieringStateBit::kMax);
   int32_t state = flags();
   state = OsrTieringStateBit::update(state, marker);
   set_flags(state);
 }
 
 void FeedbackVector::EvictOptimizedCodeMarkedForDeoptimization(
-    SharedFunctionInfo shared, const char* reason) {
-  MaybeObject slot = maybe_optimized_code(kAcquireLoad);
+    Isolate* isolate, SharedFunctionInfo shared, const char* reason) {
+  MaybeObject slot = maybe_optimized_code();
   if (slot->IsCleared()) {
-    set_maybe_has_optimized_code(false);
+    set_maybe_has_maglev_code(false);
+    set_maybe_has_turbofan_code(false);
     return;
   }
 
-  Code code = FromCodeT(CodeT::cast(slot->GetHeapObject()));
+  Code code = Code::cast(slot->GetHeapObject());
   if (code.marked_for_deoptimization()) {
-    Deoptimizer::TraceEvictFromOptimizedCodeCache(shared, reason);
+    Deoptimizer::TraceEvictFromOptimizedCodeCache(isolate, shared, reason);
     ClearOptimizedCode();
   }
 }
 
-bool FeedbackVector::ClearSlots(Isolate* isolate) {
+bool FeedbackVector::ClearSlots(Isolate* isolate, ClearBehavior behavior) {
   if (!shared_function_info().HasFeedbackMetadata()) return false;
   MaybeObject uninitialized_sentinel = MaybeObject::FromObject(
       FeedbackVector::RawUninitializedSentinel(isolate));
@@ -469,7 +457,7 @@ bool FeedbackVector::ClearSlots(Isolate* isolate) {
     MaybeObject obj = Get(slot);
     if (obj != uninitialized_sentinel) {
       FeedbackNexus nexus(*this, slot);
-      feedback_updated |= nexus.Clear();
+      feedback_updated |= nexus.Clear(behavior);
     }
   }
   return feedback_updated;
@@ -550,22 +538,19 @@ void FeedbackNexus::ConfigureUninitialized() {
     case FeedbackSlotKind::kStoreGlobalSloppy:
     case FeedbackSlotKind::kStoreGlobalStrict:
     case FeedbackSlotKind::kLoadGlobalNotInsideTypeof:
-    case FeedbackSlotKind::kLoadGlobalInsideTypeof: {
+    case FeedbackSlotKind::kLoadGlobalInsideTypeof:
       SetFeedback(HeapObjectReference::ClearedValue(isolate),
                   SKIP_WRITE_BARRIER, UninitializedSentinel(),
                   SKIP_WRITE_BARRIER);
       break;
-    }
     case FeedbackSlotKind::kCloneObject:
-    case FeedbackSlotKind::kCall: {
+    case FeedbackSlotKind::kCall:
       SetFeedback(UninitializedSentinel(), SKIP_WRITE_BARRIER, Smi::zero(),
                   SKIP_WRITE_BARRIER);
       break;
-    }
-    case FeedbackSlotKind::kInstanceOf: {
+    case FeedbackSlotKind::kInstanceOf:
       SetFeedback(UninitializedSentinel(), SKIP_WRITE_BARRIER);
       break;
-    }
     case FeedbackSlotKind::kSetNamedSloppy:
     case FeedbackSlotKind::kSetNamedStrict:
     case FeedbackSlotKind::kSetKeyedSloppy:
@@ -576,33 +561,40 @@ void FeedbackNexus::ConfigureUninitialized() {
     case FeedbackSlotKind::kLoadProperty:
     case FeedbackSlotKind::kLoadKeyed:
     case FeedbackSlotKind::kHasKeyed:
-    case FeedbackSlotKind::kDefineKeyedOwnPropertyInLiteral: {
+    case FeedbackSlotKind::kDefineKeyedOwnPropertyInLiteral:
       SetFeedback(UninitializedSentinel(), SKIP_WRITE_BARRIER,
                   UninitializedSentinel(), SKIP_WRITE_BARRIER);
       break;
-    }
+    case FeedbackSlotKind::kJumpLoop:
+      SetFeedback(HeapObjectReference::ClearedValue(isolate),
+                  SKIP_WRITE_BARRIER);
+      break;
     default:
       UNREACHABLE();
   }
 }
 
-bool FeedbackNexus::Clear() {
+bool FeedbackNexus::Clear(ClearBehavior behavior) {
   bool feedback_updated = false;
 
   switch (kind()) {
-    case FeedbackSlotKind::kTypeProfile:
-      // We don't clear these kinds ever.
-      break;
-
     case FeedbackSlotKind::kCompareOp:
     case FeedbackSlotKind::kForIn:
     case FeedbackSlotKind::kBinaryOp:
-      // We don't clear these, either.
+      if (V8_LIKELY(behavior == ClearBehavior::kDefault)) {
+        // We don't clear these, either.
+      } else if (!IsCleared()) {
+        DCHECK_EQ(behavior, ClearBehavior::kClearAll);
+        SetFeedback(Smi::zero(), SKIP_WRITE_BARRIER);
+        feedback_updated = true;
+      }
       break;
 
     case FeedbackSlotKind::kLiteral:
-      SetFeedback(Smi::zero(), SKIP_WRITE_BARRIER);
-      feedback_updated = true;
+      if (!IsCleared()) {
+        SetFeedback(Smi::zero(), SKIP_WRITE_BARRIER);
+        feedback_updated = true;
+      }
       break;
 
     case FeedbackSlotKind::kSetNamedSloppy:
@@ -623,6 +615,7 @@ bool FeedbackNexus::Clear() {
     case FeedbackSlotKind::kInstanceOf:
     case FeedbackSlotKind::kDefineKeyedOwnPropertyInLiteral:
     case FeedbackSlotKind::kCloneObject:
+    case FeedbackSlotKind::kJumpLoop:
       if (!IsCleared()) {
         ConfigureUninitialized();
         feedback_updated = true;
@@ -630,7 +623,6 @@ bool FeedbackNexus::Clear() {
       break;
 
     case FeedbackSlotKind::kInvalid:
-    case FeedbackSlotKind::kKindsNumber:
       UNREACHABLE();
   }
   return feedback_updated;
@@ -692,7 +684,8 @@ InlineCacheState FeedbackNexus::ic_state() const {
     case FeedbackSlotKind::kStoreGlobalSloppy:
     case FeedbackSlotKind::kStoreGlobalStrict:
     case FeedbackSlotKind::kLoadGlobalNotInsideTypeof:
-    case FeedbackSlotKind::kLoadGlobalInsideTypeof: {
+    case FeedbackSlotKind::kLoadGlobalInsideTypeof:
+    case FeedbackSlotKind::kJumpLoop: {
       if (feedback->IsSmi()) return InlineCacheState::MONOMORPHIC;
 
       DCHECK(feedback->IsWeakOrCleared());
@@ -742,6 +735,17 @@ InlineCacheState FeedbackNexus::ic_state() const {
                                           : InlineCacheState::MONOMORPHIC;
         }
       }
+      // TODO(1393773): Remove once the issue is solved.
+      Address vector_ptr = vector().ptr();
+      config_.isolate()->PushParamsAndDie(
+          reinterpret_cast<void*>(feedback.ptr()),
+          reinterpret_cast<void*>(extra.ptr()),
+          reinterpret_cast<void*>(vector_ptr),
+          reinterpret_cast<void*>(static_cast<intptr_t>(slot_.ToInt())),
+          reinterpret_cast<void*>(static_cast<intptr_t>(kind())),
+          // Include part of the feedback vector containing the slot.
+          reinterpret_cast<void*>(
+              vector_ptr + FeedbackVector::OffsetOfElementAt(slot_.ToInt())));
       UNREACHABLE();
     }
     case FeedbackSlotKind::kCall: {
@@ -811,12 +815,6 @@ InlineCacheState FeedbackNexus::ic_state() const {
 
       return InlineCacheState::MEGAMORPHIC;
     }
-    case FeedbackSlotKind::kTypeProfile: {
-      if (feedback == UninitializedSentinel()) {
-        return InlineCacheState::UNINITIALIZED;
-      }
-      return InlineCacheState::MONOMORPHIC;
-    }
 
     case FeedbackSlotKind::kCloneObject: {
       if (feedback == UninitializedSentinel()) {
@@ -834,7 +832,6 @@ InlineCacheState FeedbackNexus::ic_state() const {
     }
 
     case FeedbackSlotKind::kInvalid:
-    case FeedbackSlotKind::kKindsNumber:
       UNREACHABLE();
   }
   return InlineCacheState::UNINITIALIZED;
@@ -901,16 +898,18 @@ void FeedbackNexus::ConfigureCloneObject(Handle<Map> source_map,
         // Transition to POLYMORPHIC.
         Handle<WeakFixedArray> array =
             CreateArrayOfSize(2 * kCloneObjectPolymorphicEntrySize);
-        array->Set(0, HeapObjectReference::Weak(*feedback));
-        array->Set(1, GetFeedbackExtra());
-        array->Set(2, HeapObjectReference::Weak(*source_map));
-        array->Set(3, MaybeObject::FromObject(*result_map));
-        SetFeedback(*array, UPDATE_WRITE_BARRIER,
+        DisallowGarbageCollection no_gc;
+        auto raw_array = *array;
+        raw_array.Set(0, HeapObjectReference::Weak(*feedback));
+        raw_array.Set(1, GetFeedbackExtra());
+        raw_array.Set(2, HeapObjectReference::Weak(*source_map));
+        raw_array.Set(3, MaybeObject::FromObject(*result_map));
+        SetFeedback(raw_array, UPDATE_WRITE_BARRIER,
                     HeapObjectReference::ClearedValue(isolate));
       }
       break;
     case InlineCacheState::POLYMORPHIC: {
-      const int kMaxElements = FLAG_max_valid_polymorphic_map_count *
+      const int kMaxElements = v8_flags.max_valid_polymorphic_map_count *
                                kCloneObjectPolymorphicEntrySize;
       Handle<WeakFixedArray> array = Handle<WeakFixedArray>::cast(feedback);
       int i = 0;
@@ -1078,6 +1077,20 @@ int FeedbackNexus::ExtractMapsAndFeedback(
   return found;
 }
 
+MaybeObjectHandle FeedbackNexus::ExtractMegaDOMHandler() {
+  DCHECK(ic_state() == InlineCacheState::MEGADOM);
+  DisallowGarbageCollection no_gc;
+
+  auto pair = GetFeedbackPair();
+  MaybeObject maybe_handler = pair.second;
+  if (!maybe_handler->IsCleared()) {
+    MaybeObjectHandle handler = config()->NewHandle(maybe_handler);
+    return handler;
+  }
+
+  return MaybeObjectHandle();
+}
+
 int FeedbackNexus::ExtractMapsAndHandlers(
     std::vector<MapAndHandler>* maps_and_handlers,
     TryUpdateHandler map_handler) const {
@@ -1209,7 +1222,7 @@ KeyedAccessStoreMode FeedbackNexus::GetKeyedAccessStoreMode() const {
   for (const MapAndHandler& map_and_handler : maps_and_handlers) {
     const MaybeObjectHandle maybe_code_handler = map_and_handler.second;
     // The first handler that isn't the slow handler will have the bits we need.
-    Handle<Code> handler;
+    Builtin builtin_handler = Builtin::kNoBuiltinId;
     if (maybe_code_handler.object()->IsStoreHandler()) {
       Handle<StoreHandler> data_handler =
           Handle<StoreHandler>::cast(maybe_code_handler.object());
@@ -1221,8 +1234,8 @@ KeyedAccessStoreMode FeedbackNexus::GetKeyedAccessStoreMode() const {
         if (mode != STANDARD_STORE) return mode;
         continue;
       } else {
-        Code code = FromCodeT(CodeT::cast(data_handler->smi_handler()));
-        handler = config()->NewHandle(code);
+        Code code = Code::cast(data_handler->smi_handler());
+        builtin_handler = code.builtin_id();
       }
 
     } else if (maybe_code_handler.object()->IsSmi()) {
@@ -1240,19 +1253,14 @@ KeyedAccessStoreMode FeedbackNexus::GetKeyedAccessStoreMode() const {
       continue;
     } else {
       // Element store without prototype chain check.
-      if (V8_EXTERNAL_CODE_SPACE_BOOL) {
-        Code code = FromCodeT(CodeT::cast(*maybe_code_handler.object()));
-        handler = config()->NewHandle(code);
-      } else {
-        handler = Handle<Code>::cast(maybe_code_handler.object());
-      }
+      Code code = Code::cast(*maybe_code_handler.object());
+      builtin_handler = code.builtin_id();
     }
 
-    if (handler->is_builtin()) {
-      Builtin builtin = handler->builtin_id();
-      if (!BuiltinHasKeyedAccessStoreMode(builtin)) continue;
+    if (Builtins::IsBuiltinId(builtin_handler)) {
+      if (!BuiltinHasKeyedAccessStoreMode(builtin_handler)) continue;
 
-      mode = KeyedAccessStoreModeForBuiltin(builtin);
+      mode = KeyedAccessStoreModeForBuiltin(builtin_handler);
       break;
     }
   }
@@ -1307,120 +1315,6 @@ MaybeHandle<JSObject> FeedbackNexus::GetConstructorFeedback() const {
   return MaybeHandle<JSObject>();
 }
 
-namespace {
-
-bool InList(Handle<ArrayList> types, Handle<String> type) {
-  for (int i = 0; i < types->Length(); i++) {
-    Object obj = types->Get(i);
-    if (String::cast(obj).Equals(*type)) {
-      return true;
-    }
-  }
-  return false;
-}
-}  // anonymous namespace
-
-void FeedbackNexus::Collect(Handle<String> type, int position) {
-  DCHECK(IsTypeProfileKind(kind()));
-  DCHECK_GE(position, 0);
-  DCHECK(config()->can_write());
-  Isolate* isolate = GetIsolate();
-
-  MaybeObject const feedback = GetFeedback();
-
-  // Map source position to collection of types
-  Handle<SimpleNumberDictionary> types;
-
-  if (feedback == UninitializedSentinel()) {
-    types = SimpleNumberDictionary::New(isolate, 1);
-  } else {
-    types = handle(
-        SimpleNumberDictionary::cast(feedback->GetHeapObjectAssumeStrong()),
-        isolate);
-  }
-
-  Handle<ArrayList> position_specific_types;
-
-  InternalIndex entry = types->FindEntry(isolate, position);
-  if (entry.is_not_found()) {
-    position_specific_types = ArrayList::New(isolate, 1);
-    types = SimpleNumberDictionary::Set(
-        isolate, types, position,
-        ArrayList::Add(isolate, position_specific_types, type));
-  } else {
-    DCHECK(types->ValueAt(entry).IsArrayList());
-    position_specific_types =
-        handle(ArrayList::cast(types->ValueAt(entry)), isolate);
-    if (!InList(position_specific_types, type)) {  // Add type
-      types = SimpleNumberDictionary::Set(
-          isolate, types, position,
-          ArrayList::Add(isolate, position_specific_types, type));
-    }
-  }
-  SetFeedback(*types);
-}
-
-std::vector<int> FeedbackNexus::GetSourcePositions() const {
-  DCHECK(IsTypeProfileKind(kind()));
-  std::vector<int> source_positions;
-  Isolate* isolate = GetIsolate();
-
-  MaybeObject const feedback = GetFeedback();
-
-  if (feedback == UninitializedSentinel()) {
-    return source_positions;
-  }
-
-  Handle<SimpleNumberDictionary> types(
-      SimpleNumberDictionary::cast(feedback->GetHeapObjectAssumeStrong()),
-      isolate);
-
-  for (int index = SimpleNumberDictionary::kElementsStartIndex;
-       index < types->length(); index += SimpleNumberDictionary::kEntrySize) {
-    int key_index = index + SimpleNumberDictionary::kEntryKeyIndex;
-    Object key = types->get(key_index);
-    if (key.IsSmi()) {
-      int position = Smi::cast(key).value();
-      source_positions.push_back(position);
-    }
-  }
-  return source_positions;
-}
-
-std::vector<Handle<String>> FeedbackNexus::GetTypesForSourcePositions(
-    uint32_t position) const {
-  DCHECK(IsTypeProfileKind(kind()));
-  Isolate* isolate = GetIsolate();
-
-  MaybeObject const feedback = GetFeedback();
-  std::vector<Handle<String>> types_for_position;
-  if (feedback == UninitializedSentinel()) {
-    return types_for_position;
-  }
-
-  Handle<SimpleNumberDictionary> types(
-      SimpleNumberDictionary::cast(feedback->GetHeapObjectAssumeStrong()),
-      isolate);
-
-  InternalIndex entry = types->FindEntry(isolate, position);
-  if (entry.is_not_found()) return types_for_position;
-
-  DCHECK(types->ValueAt(entry).IsArrayList());
-  Handle<ArrayList> position_specific_types =
-      Handle<ArrayList>(ArrayList::cast(types->ValueAt(entry)), isolate);
-  for (int i = 0; i < position_specific_types->Length(); i++) {
-    Object t = position_specific_types->Get(i);
-    types_for_position.push_back(Handle<String>(String::cast(t), isolate));
-  }
-
-  return types_for_position;
-}
-
-void FeedbackNexus::ResetTypeProfile() {
-  DCHECK(IsTypeProfileKind(kind()));
-  SetFeedback(UninitializedSentinel());
-}
-
 FeedbackIterator::FeedbackIterator(const FeedbackNexus* nexus)
     : done_(false), index_(-1), state_(kOther) {
   DCHECK(
