@@ -22,9 +22,13 @@ namespace v8 {
 namespace internal {
 
 #ifdef DEBUG
-bool ScopeInfo::Equals(ScopeInfo other) const {
+bool ScopeInfo::Equals(ScopeInfo other, bool is_live_edit_compare) const {
   if (length() != other.length()) return false;
   for (int index = 0; index < length(); ++index) {
+    if (is_live_edit_compare && HasPositionInfo() &&
+        index >= PositionInfoIndex() && index <= PositionInfoIndex() + 1) {
+      continue;
+    }
     Object entry = get(index);
     Object other_entry = other.get(index);
     if (entry.IsSmi()) {
@@ -39,14 +43,20 @@ bool ScopeInfo::Equals(ScopeInfo other) const {
           return false;
         }
       } else if (entry.IsScopeInfo()) {
-        if (!ScopeInfo::cast(entry).Equals(ScopeInfo::cast(other_entry))) {
+        if (!is_live_edit_compare && !ScopeInfo::cast(entry).Equals(
+                                         ScopeInfo::cast(other_entry), false)) {
           return false;
         }
       } else if (entry.IsSourceTextModuleInfo()) {
-        if (!SourceTextModuleInfo::cast(entry).Equals(
+        if (!is_live_edit_compare &&
+            !SourceTextModuleInfo::cast(entry).Equals(
                 SourceTextModuleInfo::cast(other_entry))) {
           return false;
         }
+      } else if (entry.IsOddball()) {
+        if (Oddball::cast(entry).kind() != Oddball::cast(other_entry).kind()) {
+          return false;
+        }
       } else {
         UNREACHABLE();
       }
@@ -164,7 +174,7 @@ Handle<ScopeInfo> ScopeInfo::Create(IsolateT* isolate, Zone* zone, Scope* scope,
 
 // Make sure the Fields enum agrees with Torque-generated offsets.
 #define ASSERT_MATCHED_FIELD(name) \
-  STATIC_ASSERT(OffsetOfElementAt(k##name) == k##name##Offset);
+  static_assert(OffsetOfElementAt(k##name) == k##name##Offset);
   FOR_EACH_SCOPE_INFO_NUMERIC_FIELD(ASSERT_MATCHED_FIELD)
 #undef ASSERT_MATCHED_FIELD
 
@@ -480,13 +490,21 @@ Handle<ScopeInfo> ScopeInfo::CreateForNativeContext(Isolate* isolate) {
   return CreateForBootstrapping(isolate, BootstrappingType::kNative);
 }
 
+// static
+Handle<ScopeInfo> ScopeInfo::CreateForShadowRealmNativeContext(
+    Isolate* isolate) {
+  return CreateForBootstrapping(isolate, BootstrappingType::kShadowRealm);
+}
+
 // static
 Handle<ScopeInfo> ScopeInfo::CreateForBootstrapping(Isolate* isolate,
                                                     BootstrappingType type) {
   const int parameter_count = 0;
   const bool is_empty_function = type == BootstrappingType::kFunction;
-  const bool is_native_context = type == BootstrappingType::kNative;
+  const bool is_native_context = (type == BootstrappingType::kNative) ||
+                                 (type == BootstrappingType::kShadowRealm);
   const bool is_script = type == BootstrappingType::kScript;
+  const bool is_shadow_realm = type == BootstrappingType::kShadowRealm;
   const int context_local_count =
       is_empty_function || is_native_context ? 0 : 1;
   const bool has_inferred_function_name = is_empty_function;
@@ -501,10 +519,14 @@ Handle<ScopeInfo> ScopeInfo::CreateForBootstrapping(Isolate* isolate,
   Factory* factory = isolate->factory();
   Handle<ScopeInfo> scope_info =
       factory->NewScopeInfo(length, AllocationType::kReadOnly);
-
+  DisallowGarbageCollection _nogc;
   // Encode the flags.
+  DCHECK_IMPLIES(is_shadow_realm || is_script, !is_empty_function);
   int flags =
-      ScopeTypeBits::encode(is_empty_function ? FUNCTION_SCOPE : SCRIPT_SCOPE) |
+      ScopeTypeBits::encode(
+          is_empty_function
+              ? FUNCTION_SCOPE
+              : (is_shadow_realm ? SHADOW_REALM_SCOPE : SCRIPT_SCOPE)) |
       SloppyEvalCanExtendVarsBit::encode(false) |
       LanguageModeBit::encode(LanguageMode::kSloppy) |
       DeclarationScopeBit::encode(true) |
@@ -524,18 +546,20 @@ Handle<ScopeInfo> ScopeInfo::CreateForBootstrapping(Isolate* isolate,
       PrivateNameLookupSkipsOuterClassBit::encode(false) |
       HasContextExtensionSlotBit::encode(is_native_context) |
       IsReplModeScopeBit::encode(false) | HasLocalsBlockListBit::encode(false);
-  scope_info->set_flags(flags);
-  scope_info->set_parameter_count(parameter_count);
-  scope_info->set_context_local_count(context_local_count);
+  auto raw_scope_info = *scope_info;
+  raw_scope_info.set_flags(flags);
+  raw_scope_info.set_parameter_count(parameter_count);
+  raw_scope_info.set_context_local_count(context_local_count);
 
   int index = kVariablePartIndex;
 
   // Here we add info for context-allocated "this".
-  DCHECK_EQ(index, scope_info->ContextLocalNamesIndex());
+  DCHECK_EQ(index, raw_scope_info.ContextLocalNamesIndex());
+  ReadOnlyRoots roots(isolate);
   if (context_local_count) {
-    scope_info->set(index++, ReadOnlyRoots(isolate).this_string());
+    raw_scope_info.set(index++, roots.this_string());
   }
-  DCHECK_EQ(index, scope_info->ContextLocalInfosIndex());
+  DCHECK_EQ(index, raw_scope_info.ContextLocalInfosIndex());
   if (context_local_count > 0) {
     const uint32_t value =
         VariableModeBits::encode(VariableMode::kConst) |
@@ -543,30 +567,30 @@ Handle<ScopeInfo> ScopeInfo::CreateForBootstrapping(Isolate* isolate,
         MaybeAssignedFlagBit::encode(kNotAssigned) |
         ParameterNumberBits::encode(ParameterNumberBits::kMax) |
         IsStaticFlagBit::encode(IsStaticFlag::kNotStatic);
-    scope_info->set(index++, Smi::FromInt(value));
+    raw_scope_info.set(index++, Smi::FromInt(value));
   }
 
-  DCHECK_EQ(index, scope_info->FunctionVariableInfoIndex());
+  DCHECK_EQ(index, raw_scope_info.FunctionVariableInfoIndex());
   if (is_empty_function) {
-    scope_info->set(index++, *isolate->factory()->empty_string());
-    scope_info->set(index++, Smi::zero());
+    raw_scope_info.set(index++, roots.empty_string());
+    raw_scope_info.set(index++, Smi::zero());
   }
-  DCHECK_EQ(index, scope_info->InferredFunctionNameIndex());
+  DCHECK_EQ(index, raw_scope_info.InferredFunctionNameIndex());
   if (has_inferred_function_name) {
-    scope_info->set(index++, *isolate->factory()->empty_string());
+    raw_scope_info.set(index++, roots.empty_string());
   }
-  DCHECK_EQ(index, scope_info->PositionInfoIndex());
+  DCHECK_EQ(index, raw_scope_info.PositionInfoIndex());
   // Store dummy position to be in sync with the {scope_type}.
-  scope_info->set(index++, Smi::zero());
-  scope_info->set(index++, Smi::zero());
-  DCHECK_EQ(index, scope_info->OuterScopeInfoIndex());
-  DCHECK_EQ(index, scope_info->length());
-  DCHECK_EQ(scope_info->ParameterCount(), parameter_count);
+  raw_scope_info.set(index++, Smi::zero());
+  raw_scope_info.set(index++, Smi::zero());
+  DCHECK_EQ(index, raw_scope_info.OuterScopeInfoIndex());
+  DCHECK_EQ(index, raw_scope_info.length());
+  DCHECK_EQ(raw_scope_info.ParameterCount(), parameter_count);
   if (is_empty_function || is_native_context) {
-    DCHECK_EQ(scope_info->ContextLength(), 0);
+    DCHECK_EQ(raw_scope_info.ContextLength(), 0);
   } else {
-    DCHECK_EQ(scope_info->ContextLength(),
-              scope_info->ContextHeaderLength() + 1);
+    DCHECK_EQ(raw_scope_info.ContextLength(),
+              raw_scope_info.ContextHeaderLength() + 1);
   }
 
   return scope_info;
@@ -1098,6 +1122,19 @@ void ScopeInfo::ModuleVariable(int i, String* name, int* index,
   }
 }
 
+uint32_t ScopeInfo::Hash() {
+  // Hash ScopeInfo based on its start and end position.
+  // Note: Ideally we'd also have the script ID. But since we only use the
+  // hash in a debug-evaluate cache, we don't worry too much about collisions.
+  if (HasPositionInfo()) {
+    return static_cast<uint32_t>(
+        base::hash_combine(flags(), StartPosition(), EndPosition()));
+  }
+
+  return static_cast<uint32_t>(
+      base::hash_combine(flags(), context_local_count()));
+}
+
 std::ostream& operator<<(std::ostream& os, VariableAllocationInfo var_info) {
   switch (var_info) {
     case VariableAllocationInfo::NONE:
@@ -1119,9 +1156,11 @@ Handle<ModuleRequest> ModuleRequest::New(IsolateT* isolate,
                                          int position) {
   Handle<ModuleRequest> result = Handle<ModuleRequest>::cast(
       isolate->factory()->NewStruct(MODULE_REQUEST_TYPE, AllocationType::kOld));
-  result->set_specifier(*specifier);
-  result->set_import_assertions(*import_assertions);
-  result->set_position(position);
+  DisallowGarbageCollection no_gc;
+  auto raw = *result;
+  raw.set_specifier(*specifier);
+  raw.set_import_assertions(*import_assertions);
+  raw.set_position(position);
   return result;
 }
 
@@ -1141,13 +1180,15 @@ Handle<SourceTextModuleInfoEntry> SourceTextModuleInfoEntry::New(
   Handle<SourceTextModuleInfoEntry> result =
       Handle<SourceTextModuleInfoEntry>::cast(isolate->factory()->NewStruct(
           SOURCE_TEXT_MODULE_INFO_ENTRY_TYPE, AllocationType::kOld));
-  result->set_export_name(*export_name);
-  result->set_local_name(*local_name);
-  result->set_import_name(*import_name);
-  result->set_module_request(module_request);
-  result->set_cell_index(cell_index);
-  result->set_beg_pos(beg_pos);
-  result->set_end_pos(end_pos);
+  DisallowGarbageCollection no_gc;
+  auto raw = *result;
+  raw.set_export_name(*export_name);
+  raw.set_local_name(*local_name);
+  raw.set_import_name(*import_name);
+  raw.set_module_request(module_request);
+  raw.set_cell_index(cell_index);
+  raw.set_beg_pos(beg_pos);
+  raw.set_end_pos(end_pos);
   return result;
 }
 
