@@ -26,12 +26,8 @@ class Smi : public Object {
   // in that we want them to be constexprs.
   constexpr Smi() : Object() {}
   explicit constexpr Smi(Address ptr) : Object(ptr) {
-#if V8_CAN_HAVE_DCHECK_IN_CONSTEXPR
     DCHECK(HAS_SMI_TAG(ptr));
-#endif
   }
-  Smi* operator->() { return this; }
-  const Smi* operator->() const { return this; }
 
   // Returns the integer value.
   inline int value() const { return Internals::SmiValue(ptr()); }
@@ -47,9 +43,7 @@ class Smi : public Object {
 
   // Convert a value to a Smi object.
   static inline constexpr Smi FromInt(int value) {
-#if V8_CAN_HAVE_DCHECK_IN_CONSTEXPR
     DCHECK(Smi::IsValid(value));
-#endif
     return Smi(Internals::IntToSmi(value));
   }
 
@@ -69,16 +63,14 @@ class Smi : public Object {
   template <typename E,
             typename = typename std::enable_if<std::is_enum<E>::value>::type>
   static inline Smi FromEnum(E value) {
-    STATIC_ASSERT(sizeof(E) <= sizeof(int));
+    static_assert(sizeof(E) <= sizeof(int));
     return FromInt(static_cast<int>(value));
   }
 
   // Returns whether value can be represented in a Smi.
   static inline bool constexpr IsValid(intptr_t value) {
-#if V8_CAN_HAVE_DCHECK_IN_CONSTEXPR
-    DCHECK(Internals::IsValidSmi(value) ==
-           (value >= kMinValue && value <= kMaxValue));
-#endif
+    DCHECK_EQ(Internals::IsValidSmi(value),
+              value >= kMinValue && value <= kMaxValue);
     return Internals::IsValidSmi(value);
   }
 
@@ -95,18 +87,29 @@ class Smi : public Object {
   DECL_CAST(Smi)
 
   // Dispatched behavior.
-  V8_EXPORT_PRIVATE void SmiPrint(std::ostream& os) const;  // NOLINT
+  V8_EXPORT_PRIVATE void SmiPrint(std::ostream& os) const;
   DECL_VERIFIER(Smi)
 
-  // C++ does not allow us to have an object of type Smi within class Smi,
-  // so the kZero value has type Object. Consider it deprecated; new code
-  // should use zero() instead.
-  V8_EXPORT_PRIVATE static constexpr Object kZero = Object(0);
-  // If you need something with type Smi, call zero() instead. Since it is
-  // a constexpr, "calling" it is just as efficient as reading kZero.
+  // Since this is a constexpr, "calling" it is just as efficient
+  // as reading a constant.
   static inline constexpr Smi zero() { return Smi::FromInt(0); }
   static constexpr int kMinValue = kSmiMinValue;
   static constexpr int kMaxValue = kSmiMaxValue;
+
+  // Smi value for filling in not-yet initialized tagged field values with a
+  // valid tagged pointer. A field value equal to this doesn't necessarily
+  // indicate that a field is uninitialized, but an uninitialized field should
+  // definitely equal this value.
+  //
+  // This _has_ to be kNullAddress, so that an uninitialized field value read as
+  // an embedded pointer field is interpreted as nullptr. This is so that
+  // uninitialised embedded pointers are not forwarded to the embedder as part
+  // of embedder tracing (and similar mechanisms), as nullptrs are skipped for
+  // those cases and otherwise the embedder would try to dereference the
+  // uninitialized pointer value.
+  static constexpr Smi uninitialized_deserialization_value() {
+    return Smi(kNullAddress);
+  }
 };
 
 CAST_ACCESSOR(Smi)
