@@ -14,26 +14,28 @@
 namespace v8 {
 namespace internal {
 
-void CodeCreateEventRecord::UpdateCodeMap(CodeMap* code_map) {
-  code_map->AddCode(instruction_start, entry, instruction_size);
+void CodeCreateEventRecord::UpdateCodeMap(
+    InstructionStreamMap* instruction_stream_map) {
+  instruction_stream_map->AddCode(instruction_start, entry, instruction_size);
 }
 
-
-void CodeMoveEventRecord::UpdateCodeMap(CodeMap* code_map) {
-  code_map->MoveCode(from_instruction_start, to_instruction_start);
+void CodeMoveEventRecord::UpdateCodeMap(
+    InstructionStreamMap* instruction_stream_map) {
+  instruction_stream_map->MoveCode(from_instruction_start,
+                                   to_instruction_start);
 }
 
-
-void CodeDisableOptEventRecord::UpdateCodeMap(CodeMap* code_map) {
-  CodeEntry* entry = code_map->FindEntry(instruction_start);
+void CodeDisableOptEventRecord::UpdateCodeMap(
+    InstructionStreamMap* instruction_stream_map) {
+  CodeEntry* entry = instruction_stream_map->FindEntry(instruction_start);
   if (entry != nullptr) {
     entry->set_bailout_reason(bailout_reason);
   }
 }
 
-
-void CodeDeoptEventRecord::UpdateCodeMap(CodeMap* code_map) {
-  CodeEntry* entry = code_map->FindEntry(instruction_start);
+void CodeDeoptEventRecord::UpdateCodeMap(
+    InstructionStreamMap* instruction_stream_map) {
+  CodeEntry* entry = instruction_stream_map->FindEntry(instruction_start);
   if (entry != nullptr) {
     std::vector<CpuProfileDeoptFrame> frames_vector(
         deopt_frames, deopt_frames + deopt_frame_count);
@@ -42,9 +44,9 @@ void CodeDeoptEventRecord::UpdateCodeMap(CodeMap* code_map) {
   delete[] deopt_frames;
 }
 
-
-void ReportBuiltinEventRecord::UpdateCodeMap(CodeMap* code_map) {
-  CodeEntry* entry = code_map->FindEntry(instruction_start);
+void ReportBuiltinEventRecord::UpdateCodeMap(
+    InstructionStreamMap* instruction_stream_map) {
+  CodeEntry* entry = instruction_stream_map->FindEntry(instruction_start);
   if (entry) {
     entry->SetBuiltinId(builtin);
     return;
@@ -53,9 +55,9 @@ void ReportBuiltinEventRecord::UpdateCodeMap(CodeMap* code_map) {
   if (builtin == Builtin::kGenericJSToWasmWrapper) {
     // Make sure to add the generic js-to-wasm wrapper builtin, because that
     // one is supposed to show up in profiles.
-    entry = code_map->code_entries().Create(CodeEventListener::BUILTIN_TAG,
-                                            Builtins::name(builtin));
-    code_map->AddCode(instruction_start, entry, instruction_size);
+    entry = instruction_stream_map->code_entries().Create(
+        LogEventListener::CodeTag::kBuiltin, Builtins::name(builtin));
+    instruction_stream_map->AddCode(instruction_start, entry, instruction_size);
   }
 #endif  // V8_ENABLE_WEBASSEMBLY
 }
@@ -68,8 +70,9 @@ TickSample* SamplingEventsProcessor::StartTickSample() {
   return &evt->sample;
 }
 
-void CodeDeleteEventRecord::UpdateCodeMap(CodeMap* code_map) {
-  bool removed = code_map->RemoveCode(entry);
+void CodeDeleteEventRecord::UpdateCodeMap(
+    InstructionStreamMap* instruction_stream_map) {
+  bool removed = instruction_stream_map->RemoveCode(entry);
   CHECK(removed);
 }
 
