@@ -5,12 +5,14 @@
 #include "src/profiler/profile-generator.h"
 
 #include <algorithm>
+#include <vector>
 
 #include "include/v8-profiler.h"
 #include "src/base/lazy-instance.h"
 #include "src/codegen/source-position.h"
 #include "src/objects/shared-function-info-inl.h"
 #include "src/profiler/cpu-profiler.h"
+#include "src/profiler/output-stream-writer.h"
 #include "src/profiler/profile-generator-inl.h"
 #include "src/profiler/profiler-stats.h"
 #include "src/tracing/trace-event.h"
@@ -91,7 +93,7 @@ const char* const CodeEntry::kRootEntryName = "(root)";
 // static
 CodeEntry* CodeEntry::program_entry() {
   static base::LeakyObject<CodeEntry> kProgramEntry(
-      CodeEventListener::FUNCTION_TAG, CodeEntry::kProgramEntryName,
+      LogEventListener::CodeTag::kFunction, CodeEntry::kProgramEntryName,
       CodeEntry::kEmptyResourceName, v8::CpuProfileNode::kNoLineNumberInfo,
       v8::CpuProfileNode::kNoColumnNumberInfo, nullptr, false,
       CodeEntry::CodeType::OTHER);
@@ -101,7 +103,7 @@ CodeEntry* CodeEntry::program_entry() {
 // static
 CodeEntry* CodeEntry::idle_entry() {
   static base::LeakyObject<CodeEntry> kIdleEntry(
-      CodeEventListener::FUNCTION_TAG, CodeEntry::kIdleEntryName,
+      LogEventListener::CodeTag::kFunction, CodeEntry::kIdleEntryName,
       CodeEntry::kEmptyResourceName, v8::CpuProfileNode::kNoLineNumberInfo,
       v8::CpuProfileNode::kNoColumnNumberInfo, nullptr, false,
       CodeEntry::CodeType::OTHER);
@@ -111,8 +113,9 @@ CodeEntry* CodeEntry::idle_entry() {
 // static
 CodeEntry* CodeEntry::gc_entry() {
   static base::LeakyObject<CodeEntry> kGcEntry(
-      CodeEventListener::BUILTIN_TAG, CodeEntry::kGarbageCollectorEntryName,
-      CodeEntry::kEmptyResourceName, v8::CpuProfileNode::kNoLineNumberInfo,
+      LogEventListener::CodeTag::kBuiltin,
+      CodeEntry::kGarbageCollectorEntryName, CodeEntry::kEmptyResourceName,
+      v8::CpuProfileNode::kNoLineNumberInfo,
       v8::CpuProfileNode::kNoColumnNumberInfo, nullptr, false,
       CodeEntry::CodeType::OTHER);
   return kGcEntry.get();
@@ -121,7 +124,7 @@ CodeEntry* CodeEntry::gc_entry() {
 // static
 CodeEntry* CodeEntry::unresolved_entry() {
   static base::LeakyObject<CodeEntry> kUnresolvedEntry(
-      CodeEventListener::FUNCTION_TAG, CodeEntry::kUnresolvedFunctionName,
+      LogEventListener::CodeTag::kFunction, CodeEntry::kUnresolvedFunctionName,
       CodeEntry::kEmptyResourceName, v8::CpuProfileNode::kNoLineNumberInfo,
       v8::CpuProfileNode::kNoColumnNumberInfo, nullptr, false,
       CodeEntry::CodeType::OTHER);
@@ -131,7 +134,7 @@ CodeEntry* CodeEntry::unresolved_entry() {
 // static
 CodeEntry* CodeEntry::root_entry() {
   static base::LeakyObject<CodeEntry> kRootEntry(
-      CodeEventListener::FUNCTION_TAG, CodeEntry::kRootEntryName,
+      LogEventListener::CodeTag::kFunction, CodeEntry::kRootEntryName,
       CodeEntry::kEmptyResourceName, v8::CpuProfileNode::kNoLineNumberInfo,
       v8::CpuProfileNode::kNoColumnNumberInfo, nullptr, false,
       CodeEntry::CodeType::OTHER);
@@ -163,7 +166,8 @@ bool CodeEntry::IsSameFunctionAs(const CodeEntry* entry) const {
 }
 
 void CodeEntry::SetBuiltinId(Builtin id) {
-  bit_field_ = TagField::update(bit_field_, CodeEventListener::BUILTIN_TAG);
+  bit_field_ =
+      CodeTagField::update(bit_field_, LogEventListener::CodeTag::kBuiltin);
   bit_field_ = BuiltinField::update(bit_field_, id);
 }
 
@@ -339,34 +343,26 @@ CpuProfileNode::SourceType ProfileNode::source_type() const {
     return CpuProfileNode::kUnresolved;
 
   // Otherwise, resolve based on logger tag.
-  switch (entry_->tag()) {
-    case CodeEventListener::EVAL_TAG:
-    case CodeEventListener::SCRIPT_TAG:
-    case CodeEventListener::LAZY_COMPILE_TAG:
-    case CodeEventListener::FUNCTION_TAG:
+  switch (entry_->code_tag()) {
+    case LogEventListener::CodeTag::kEval:
+    case LogEventListener::CodeTag::kScript:
+    case LogEventListener::CodeTag::kFunction:
       return CpuProfileNode::kScript;
-    case CodeEventListener::BUILTIN_TAG:
-    case CodeEventListener::HANDLER_TAG:
-    case CodeEventListener::BYTECODE_HANDLER_TAG:
-    case CodeEventListener::NATIVE_FUNCTION_TAG:
-    case CodeEventListener::NATIVE_SCRIPT_TAG:
-    case CodeEventListener::NATIVE_LAZY_COMPILE_TAG:
+    case LogEventListener::CodeTag::kBuiltin:
+    case LogEventListener::CodeTag::kHandler:
+    case LogEventListener::CodeTag::kBytecodeHandler:
+    case LogEventListener::CodeTag::kNativeFunction:
+    case LogEventListener::CodeTag::kNativeScript:
       return CpuProfileNode::kBuiltin;
-    case CodeEventListener::CALLBACK_TAG:
+    case LogEventListener::CodeTag::kCallback:
       return CpuProfileNode::kCallback;
-    case CodeEventListener::REG_EXP_TAG:
-    case CodeEventListener::STUB_TAG:
-    case CodeEventListener::CODE_CREATION_EVENT:
-    case CodeEventListener::CODE_DISABLE_OPT_EVENT:
-    case CodeEventListener::CODE_MOVE_EVENT:
-    case CodeEventListener::CODE_DELETE_EVENT:
-    case CodeEventListener::CODE_MOVING_GC:
-    case CodeEventListener::SHARED_FUNC_MOVE_EVENT:
-    case CodeEventListener::SNAPSHOT_CODE_NAME_EVENT:
-    case CodeEventListener::TICK_EVENT:
-    case CodeEventListener::NUMBER_OF_LOG_EVENTS:
+    case LogEventListener::CodeTag::kRegExp:
+    case LogEventListener::CodeTag::kStub:
+    case LogEventListener::CodeTag::kLength:
       return CpuProfileNode::kInternal;
   }
+  return CpuProfileNode::kInternal;
+  UNREACHABLE();
 }
 
 void ProfileNode::CollectDeoptInfo(CodeEntry* entry) {
@@ -576,7 +572,7 @@ CpuProfile::CpuProfile(CpuProfiler* profiler, ProfilerId id, const char* title,
                        CpuProfilingOptions options,
                        std::unique_ptr<DiscardedSamplesDelegate> delegate)
     : title_(title),
-      options_(options),
+      options_(std::move(options)),
       delegate_(std::move(delegate)),
       start_time_(base::TimeTicks::Now()),
       top_down_(profiler->isolate(), profiler->code_entries()),
@@ -631,17 +627,16 @@ void CpuProfile::AddPath(base::TimeTicks timestamp,
   ProfileNode* top_frame_node =
       top_down_.AddPathFromEnd(path, src_line, update_stats, options_.mode());
 
+  bool is_buffer_full =
+      options_.max_samples() != CpuProfilingOptions::kNoSampleLimit &&
+      samples_.size() >= options_.max_samples();
   bool should_record_sample =
-      !timestamp.IsNull() && timestamp >= start_time_ &&
-      (options_.max_samples() == CpuProfilingOptions::kNoSampleLimit ||
-       samples_.size() < options_.max_samples());
+      !timestamp.IsNull() && timestamp >= start_time_ && !is_buffer_full;
 
   if (should_record_sample) {
     samples_.push_back(
         {top_frame_node, timestamp, src_line, state_tag, embedder_state_tag});
-  }
-
-  if (!should_record_sample && delegate_ != nullptr) {
+  } else if (is_buffer_full && delegate_ != nullptr) {
     const auto task_runner = V8::GetCurrentPlatform()->GetForegroundTaskRunner(
         reinterpret_cast<v8::Isolate*>(profiler_->isolate()));
 
@@ -769,6 +764,160 @@ void CpuProfile::FinishProfile() {
                               "ProfileChunk", id_, "data", std::move(value));
 }
 
+namespace {
+
+void FlattenNodesTree(const v8::CpuProfileNode* node,
+                      std::vector<const v8::CpuProfileNode*>* nodes) {
+  nodes->emplace_back(node);
+  const int childrenCount = node->GetChildrenCount();
+  for (int i = 0; i < childrenCount; i++)
+    FlattenNodesTree(node->GetChild(i), nodes);
+}
+
+}  // namespace
+
+void CpuProfileJSONSerializer::Serialize(v8::OutputStream* stream) {
+  DCHECK_NULL(writer_);
+  writer_ = new OutputStreamWriter(stream);
+  SerializeImpl();
+  delete writer_;
+  writer_ = nullptr;
+}
+
+void CpuProfileJSONSerializer::SerializePositionTicks(
+    const v8::CpuProfileNode* node, int lineCount) {
+  std::vector<v8::CpuProfileNode::LineTick> entries(lineCount);
+  if (node->GetLineTicks(&entries[0], lineCount)) {
+    for (int i = 0; i < lineCount; i++) {
+      writer_->AddCharacter('{');
+      writer_->AddString("\"line\":");
+      writer_->AddNumber(entries[i].line);
+      writer_->AddString(",\"ticks\":");
+      writer_->AddNumber(entries[i].hit_count);
+      writer_->AddCharacter('}');
+      if (i != (lineCount - 1)) writer_->AddCharacter(',');
+    }
+  }
+}
+
+void CpuProfileJSONSerializer::SerializeCallFrame(
+    const v8::CpuProfileNode* node) {
+  writer_->AddString("\"functionName\":\"");
+  writer_->AddString(node->GetFunctionNameStr());
+  writer_->AddString("\",\"lineNumber\":");
+  writer_->AddNumber(node->GetLineNumber() - 1);
+  writer_->AddString(",\"columnNumber\":");
+  writer_->AddNumber(node->GetColumnNumber() - 1);
+  writer_->AddString(",\"scriptId\":");
+  writer_->AddNumber(node->GetScriptId());
+  writer_->AddString(",\"url\":\"");
+  writer_->AddString(node->GetScriptResourceNameStr());
+  writer_->AddCharacter('"');
+}
+
+void CpuProfileJSONSerializer::SerializeChildren(const v8::CpuProfileNode* node,
+                                                 int childrenCount) {
+  for (int i = 0; i < childrenCount; i++) {
+    writer_->AddNumber(node->GetChild(i)->GetNodeId());
+    if (i != (childrenCount - 1)) writer_->AddCharacter(',');
+  }
+}
+
+void CpuProfileJSONSerializer::SerializeNode(const v8::CpuProfileNode* node) {
+  writer_->AddCharacter('{');
+  writer_->AddString("\"id\":");
+  writer_->AddNumber(node->GetNodeId());
+
+  writer_->AddString(",\"hitCount\":");
+  writer_->AddNumber(node->GetHitCount());
+
+  writer_->AddString(",\"callFrame\":{");
+  SerializeCallFrame(node);
+  writer_->AddCharacter('}');
+
+  const int childrenCount = node->GetChildrenCount();
+  if (childrenCount) {
+    writer_->AddString(",\"children\":[");
+    SerializeChildren(node, childrenCount);
+    writer_->AddCharacter(']');
+  }
+
+  const char* deoptReason = node->GetBailoutReason();
+  if (deoptReason && deoptReason[0] && strcmp(deoptReason, "no reason")) {
+    writer_->AddString(",\"deoptReason\":\"");
+    writer_->AddString(deoptReason);
+    writer_->AddCharacter('"');
+  }
+
+  unsigned lineCount = node->GetHitLineCount();
+  if (lineCount) {
+    writer_->AddString(",\"positionTicks\":[");
+    SerializePositionTicks(node, lineCount);
+    writer_->AddCharacter(']');
+  }
+  writer_->AddCharacter('}');
+}
+
+void CpuProfileJSONSerializer::SerializeNodes() {
+  std::vector<const v8::CpuProfileNode*> nodes;
+  FlattenNodesTree(
+      reinterpret_cast<const v8::CpuProfileNode*>(profile_->top_down()->root()),
+      &nodes);
+
+  for (size_t i = 0; i < nodes.size(); i++) {
+    SerializeNode(nodes.at(i));
+    if (writer_->aborted()) return;
+    if (i != (nodes.size() - 1)) writer_->AddCharacter(',');
+  }
+}
+
+void CpuProfileJSONSerializer::SerializeTimeDeltas() {
+  int count = profile_->samples_count();
+  uint64_t lastTime = profile_->start_time().since_origin().InMicroseconds();
+  for (int i = 0; i < count; i++) {
+    uint64_t ts = profile_->sample(i).timestamp.since_origin().InMicroseconds();
+    writer_->AddNumber(static_cast<int>(ts - lastTime));
+    if (i != (count - 1)) writer_->AddString(",");
+    lastTime = ts;
+  }
+}
+
+void CpuProfileJSONSerializer::SerializeSamples() {
+  int count = profile_->samples_count();
+  for (int i = 0; i < count; i++) {
+    writer_->AddNumber(profile_->sample(i).node->id());
+    if (i != (count - 1)) writer_->AddString(",");
+  }
+}
+
+void CpuProfileJSONSerializer::SerializeImpl() {
+  writer_->AddCharacter('{');
+  writer_->AddString("\"nodes\":[");
+  SerializeNodes();
+  writer_->AddString("]");
+
+  writer_->AddString(",\"startTime\":");
+  writer_->AddNumber(static_cast<unsigned>(
+      profile_->start_time().since_origin().InMicroseconds()));
+
+  writer_->AddString(",\"endTime\":");
+  writer_->AddNumber(static_cast<unsigned>(
+      profile_->end_time().since_origin().InMicroseconds()));
+
+  writer_->AddString(",\"samples\":[");
+  SerializeSamples();
+  if (writer_->aborted()) return;
+  writer_->AddCharacter(']');
+
+  writer_->AddString(",\"timeDeltas\":[");
+  SerializeTimeDeltas();
+  if (writer_->aborted()) return;
+  writer_->AddString("]");
+
+  writer_->AddCharacter('}');
+  writer_->Finalize();
+}
+
 void CpuProfile::Print() const {
   base::OS::Print("[Top down]:\n");
   top_down_.Print();
@@ -792,11 +941,12 @@ void CodeEntryStorage::DecRef(CodeEntry* entry) {
   }
 }
 
-CodeMap::CodeMap(CodeEntryStorage& storage) : code_entries_(storage) {}
+InstructionStreamMap::InstructionStreamMap(CodeEntryStorage& storage)
+    : code_entries_(storage) {}
 
-CodeMap::~CodeMap() { Clear(); }
+InstructionStreamMap::~InstructionStreamMap() { Clear(); }
 
-void CodeMap::Clear() {
+void InstructionStreamMap::Clear() {
   for (auto& slot : code_map_) {
     if (CodeEntry* entry = slot.second.entry) {
       code_entries_.DecRef(entry);
@@ -809,12 +959,13 @@ void CodeMap::Clear() {
   code_map_.clear();
 }
 
-void CodeMap::AddCode(Address addr, CodeEntry* entry, unsigned size) {
+void InstructionStreamMap::AddCode(Address addr, CodeEntry* entry,
+                                   unsigned size) {
   code_map_.emplace(addr, CodeEntryMapInfo{entry, size});
   entry->set_instruction_start(addr);
 }
 
-bool CodeMap::RemoveCode(CodeEntry* entry) {
+bool InstructionStreamMap::RemoveCode(CodeEntry* entry) {
   auto range = code_map_.equal_range(entry->instruction_start());
   for (auto i = range.first; i != range.second; ++i) {
     if (i->second.entry == entry) {
@@ -826,7 +977,7 @@ bool CodeMap::RemoveCode(CodeEntry* entry) {
   return false;
 }
 
-void CodeMap::ClearCodesInRange(Address start, Address end) {
+void InstructionStreamMap::ClearCodesInRange(Address start, Address end) {
   auto left = code_map_.upper_bound(start);
   if (left != code_map_.begin()) {
     --left;
@@ -839,7 +990,8 @@ void CodeMap::ClearCodesInRange(Address start, Address end) {
   code_map_.erase(left, right);
 }
 
-CodeEntry* CodeMap::FindEntry(Address addr, Address* out_instruction_start) {
+CodeEntry* InstructionStreamMap::FindEntry(Address addr,
+                                           Address* out_instruction_start) {
   // Note that an address may correspond to multiple CodeEntry objects. An
   // arbitrary selection is made (as per multimap spec) in the event of a
   // collision.
@@ -854,7 +1006,7 @@ CodeEntry* CodeMap::FindEntry(Address addr, Address* out_instruction_start) {
   return ret;
 }
 
-void CodeMap::MoveCode(Address from, Address to) {
+void InstructionStreamMap::MoveCode(Address from, Address to) {
   if (from == to) return;
 
   auto range = code_map_.equal_range(from);
@@ -877,14 +1029,14 @@ void CodeMap::MoveCode(Address from, Address to) {
   code_map_.erase(range.first, it);
 }
 
-void CodeMap::Print() {
+void InstructionStreamMap::Print() {
   for (const auto& pair : code_map_) {
     base::OS::Print("%p %5d %s\n", reinterpret_cast<void*>(pair.first),
                     pair.second.size, pair.second.entry->name());
   }
 }
 
-size_t CodeMap::GetEstimatedMemoryUsage() const {
+size_t InstructionStreamMap::GetEstimatedMemoryUsage() const {
   size_t map_size = 0;
   for (const auto& pair : code_map_) {
     map_size += sizeof(pair.first) + sizeof(pair.second) +
@@ -894,7 +1046,7 @@ size_t CodeMap::GetEstimatedMemoryUsage() const {
 }
 
 CpuProfilesCollection::CpuProfilesCollection(Isolate* isolate)
-    : profiler_(nullptr), current_profiles_semaphore_(1), isolate_(isolate) {
+    : profiler_(nullptr), current_profiles_mutex_(), isolate_(isolate) {
   USE(isolate_);
 }
 
@@ -906,16 +1058,15 @@ CpuProfilingResult CpuProfilesCollection::StartProfilingForTesting(
 CpuProfilingResult CpuProfilesCollection::StartProfiling(
     const char* title, CpuProfilingOptions options,
     std::unique_ptr<DiscardedSamplesDelegate> delegate) {
-  return StartProfiling(++last_id_, title, options, std::move(delegate));
+  return StartProfiling(++last_id_, title, std::move(options),
+                        std::move(delegate));
 }
 
 CpuProfilingResult CpuProfilesCollection::StartProfiling(
     ProfilerId id, const char* title, CpuProfilingOptions options,
     std::unique_ptr<DiscardedSamplesDelegate> delegate) {
-  current_profiles_semaphore_.Wait();
-
+  base::RecursiveMutexGuard profiles_guard{&current_profiles_mutex_};
   if (static_cast<int>(current_profiles_.size()) >= kMaxSimultaneousProfiles) {
-    current_profiles_semaphore_.Signal();
     return {
         0,
         CpuProfilingStatus::kErrorTooManyProfilers,
@@ -927,7 +1078,6 @@ CpuProfilingResult CpuProfilesCollection::StartProfiling(
          strcmp(profile->title(), title) == 0) ||
         profile->id() == id) {
       // Ignore attempts to start profile with the same title or id
-      current_profiles_semaphore_.Signal();
       // ... though return kAlreadyStarted to force it collect a sample.
       return {
           profile->id(),
@@ -936,10 +1086,9 @@ CpuProfilingResult CpuProfilesCollection::StartProfiling(
     }
   }
 
-  CpuProfile* profile =
-      new CpuProfile(profiler_, id, title, options, std::move(delegate));
+  CpuProfile* profile = new CpuProfile(profiler_, id, title, std::move(options),
+                                       std::move(delegate));
   current_profiles_.emplace_back(profile);
-  current_profiles_semaphore_.Signal();
 
   return {
       profile->id(),
@@ -948,7 +1097,7 @@ CpuProfilingResult CpuProfilesCollection::StartProfiling(
 }
 
 CpuProfile* CpuProfilesCollection::StopProfiling(ProfilerId id) {
-  current_profiles_semaphore_.Wait();
+  base::RecursiveMutexGuard profiles_guard{&current_profiles_mutex_};
   CpuProfile* profile = nullptr;
 
   auto it = std::find_if(
@@ -962,37 +1111,27 @@ CpuProfile* CpuProfilesCollection::StopProfiling(ProfilerId id) {
     // Convert reverse iterator to matching forward iterator.
     current_profiles_.erase(--(it.base()));
   }
-  current_profiles_semaphore_.Signal();
   return profile;
 }
 
 CpuProfile* CpuProfilesCollection::Lookup(const char* title) {
-  // Called from VM thread, and only it can mutate the list,
-  // so no locking is needed here.
-  DCHECK_EQ(ThreadId::Current(), isolate_->thread_id());
-  if (title == nullptr) {
-    return nullptr;
-  }
+  if (title == nullptr) return nullptr;
   // http://crbug/51594, edge case console.profile may provide an empty title
   // and must not crash
   const bool empty_title = title[0] == '\0';
+  base::RecursiveMutexGuard profiles_guard{&current_profiles_mutex_};
   auto it = std::find_if(
       current_profiles_.rbegin(), current_profiles_.rend(),
       [&](const std::unique_ptr<CpuProfile>& p) {
         return (empty_title ||
                 (p->title() != nullptr && strcmp(p->title(), title) == 0));
       });
-  if (it != current_profiles_.rend()) {
-    return it->get();
-  }
-
+  if (it != current_profiles_.rend()) return it->get();
   return nullptr;
 }
 
 bool CpuProfilesCollection::IsLastProfileLeft(ProfilerId id) {
-  // Called from VM thread, and only it can mutate the list,
-  // so no locking is needed here.
-  DCHECK_EQ(ThreadId::Current(), isolate_->thread_id());
+  base::RecursiveMutexGuard profiles_guard{&current_profiles_mutex_};
   if (current_profiles_.size() != 1) return false;
   return id == current_profiles_[0]->id();
 }
@@ -1017,7 +1156,7 @@ int64_t GreatestCommonDivisor(int64_t a, int64_t b) {
 
 }  // namespace
 
-base::TimeDelta CpuProfilesCollection::GetCommonSamplingInterval() const {
+base::TimeDelta CpuProfilesCollection::GetCommonSamplingInterval() {
   DCHECK(profiler_);
 
   int64_t base_sampling_interval_us =
@@ -1025,16 +1164,19 @@ base::TimeDelta CpuProfilesCollection::GetCommonSamplingInterval() const {
   if (base_sampling_interval_us == 0) return base::TimeDelta();
 
   int64_t interval_us = 0;
-  for (const auto& profile : current_profiles_) {
-    // Snap the profile's requested sampling interval to the next multiple of
-    // the base sampling interval.
-    int64_t profile_interval_us =
-        std::max<int64_t>(
-            (profile->sampling_interval_us() + base_sampling_interval_us - 1) /
-                base_sampling_interval_us,
-            1) *
-        base_sampling_interval_us;
-    interval_us = GreatestCommonDivisor(interval_us, profile_interval_us);
+  {
+    base::RecursiveMutexGuard profiles_guard{&current_profiles_mutex_};
+    for (const auto& profile : current_profiles_) {
+      // Snap the profile's requested sampling interval to the next multiple of
+      // the base sampling interval.
+      int64_t profile_interval_us =
+          std::max<int64_t>((profile->sampling_interval_us() +
+                             base_sampling_interval_us - 1) /
+                                base_sampling_interval_us,
+                            1) *
+          base_sampling_interval_us;
+      interval_us = GreatestCommonDivisor(interval_us, profile_interval_us);
+    }
   }
   return base::TimeDelta::FromMicroseconds(interval_us);
 }
@@ -1047,8 +1189,8 @@ void CpuProfilesCollection::AddPathToCurrentProfiles(
   // As starting / stopping profiles is rare relatively to this
   // method, we don't bother minimizing the duration of lock holding,
   // e.g. copying contents of the list to a local vector.
-  current_profiles_semaphore_.Wait();
   const ProfileStackTrace empty_path;
+  base::RecursiveMutexGuard profiles_guard{&current_profiles_mutex_};
   for (const std::unique_ptr<CpuProfile>& profile : current_profiles_) {
     ContextFilter& context_filter = profile->context_filter();
     // If the context filter check failed, omit the contents of the stack.
@@ -1067,16 +1209,14 @@ void CpuProfilesCollection::AddPathToCurrentProfiles(
                      accepts_embedder_context ? embedder_state_tag
                                               : EmbedderStateTag::EMPTY);
   }
-  current_profiles_semaphore_.Signal();
 }
 
 void CpuProfilesCollection::UpdateNativeContextAddressForCurrentProfiles(
     Address from, Address to) {
-  current_profiles_semaphore_.Wait();
+  base::RecursiveMutexGuard profiles_guard{&current_profiles_mutex_};
   for (const std::unique_ptr<CpuProfile>& profile : current_profiles_) {
     profile->context_filter().OnMoveEvent(from, to);
   }
-  current_profiles_semaphore_.Signal();
 }
 
 }  // namespace internal
