@@ -16,7 +16,7 @@ namespace internal {
 
 // ES#sec-names-and-keywords Names and Keywords
 // UnicodeIDStart, '$', '_' and '\'
-bool IsIdentifierStartSlow(uc32 c) {
+bool IsIdentifierStartSlow(base::uc32 c) {
   // cannot use u_isIDStart because it does not work for
   // Other_ID_Start characters.
   return u_hasBinaryProperty(c, UCHAR_ID_START) ||
@@ -25,7 +25,7 @@ bool IsIdentifierStartSlow(uc32 c) {
 
 // ES#sec-names-and-keywords Names and Keywords
 // UnicodeIDContinue, '$', '_', '\', ZWJ, and ZWNJ
-bool IsIdentifierPartSlow(uc32 c) {
+bool IsIdentifierPartSlow(base::uc32 c) {
   // Can't use u_isIDPart because it does not work for
   // Other_ID_Continue characters.
   return u_hasBinaryProperty(c, UCHAR_ID_CONTINUE) ||
@@ -35,7 +35,7 @@ bool IsIdentifierPartSlow(uc32 c) {
 
 // ES#sec-white-space White Space
 // gC=Zs, U+0009, U+000B, U+000C, U+FEFF
-bool IsWhiteSpaceSlow(uc32 c) {
+bool IsWhiteSpaceSlow(base::uc32 c) {
   return (u_charType(c) == U_SPACE_SEPARATOR) ||
          (c < 0x0D && (c == 0x09 || c == 0x0B || c == 0x0C)) || c == 0xFEFF;
 }
