@@ -53,12 +53,10 @@ class V8_EXPORT_PRIVATE StreamingProcessor {
 
   // Report the end of a chunk.
   virtual void OnFinishedChunk() = 0;
-  // Report the end of the stream. If the stream was successful, all
-  // received bytes are passed by parameter. If there has been an error, an
-  // empty array is passed.
-  virtual void OnFinishedStream(base::OwnedVector<uint8_t> bytes) = 0;
-  // Report an error detected in the StreamingDecoder.
-  virtual void OnError(const WasmError&) = 0;
+  // Report the end of the stream. This will be called even after an error has
+  // been detected. In any case, the parameter is the total received bytes.
+  virtual void OnFinishedStream(base::OwnedVector<const uint8_t> bytes,
+                                bool after_error) = 0;
   // Report the abortion of the stream.
   virtual void OnAbort() = 0;
 
@@ -81,34 +79,35 @@ class V8_EXPORT_PRIVATE StreamingDecoder {
 
   virtual void Abort() = 0;
 
-  // Notify the StreamingDecoder that compilation ended and the
+  // Notify the StreamingDecoder that the job was discarded and the
   // StreamingProcessor should not be called anymore.
-  virtual void NotifyCompilationEnded() = 0;
+  virtual void NotifyCompilationDiscarded() = 0;
 
   // Caching support.
-  // Sets the callback that is called after the module is fully compiled.
-  using ModuleCompiledCallback =
+  // Sets the callback that is called after a new chunk of the module is tiered
+  // up.
+  using MoreFunctionsCanBeSerializedCallback =
       std::function<void(const std::shared_ptr<NativeModule>&)>;
 
-  void SetModuleCompiledCallback(ModuleCompiledCallback callback) {
-    module_compiled_callback_ = callback;
+  void SetMoreFunctionsCanBeSerializedCallback(
+      MoreFunctionsCanBeSerializedCallback callback) {
+    more_functions_can_be_serialized_callback_ = std::move(callback);
   }
 
   // Passes previously compiled module bytes from the embedder's cache.
   // The content shouldn't be used until Finish(true) is called.
-  bool SetCompiledModuleBytes(
-      base::Vector<const uint8_t> compiled_module_bytes) {
-    compiled_module_bytes_ = compiled_module_bytes;
-    return true;
+  void SetCompiledModuleBytes(base::Vector<const uint8_t> bytes) {
+    compiled_module_bytes_ = bytes;
   }
 
   virtual void NotifyNativeModuleCreated(
       const std::shared_ptr<NativeModule>& native_module) = 0;
 
-  base::Vector<const char> url() { return base::VectorOf(url_); }
+  const std::string& url() const { return *url_; }
+  std::shared_ptr<const std::string> shared_url() const { return url_; }
 
   void SetUrl(base::Vector<const char> url) {
-    url_.assign(url.begin(), url.length());
+    url_->assign(url.begin(), url.size());
   }
 
   static std::unique_ptr<StreamingDecoder> CreateAsyncStreamingDecoder(
@@ -122,8 +121,9 @@ class V8_EXPORT_PRIVATE StreamingDecoder {
  protected:
   bool deserializing() const { return !compiled_module_bytes_.empty(); }
 
-  std::string url_;
-  ModuleCompiledCallback module_compiled_callback_;
+  const std::shared_ptr<std::string> url_ = std::make_shared<std::string>();
+  MoreFunctionsCanBeSerializedCallback
+      more_functions_can_be_serialized_callback_;
   // The content of `compiled_module_bytes_` shouldn't be used until
   // Finish(true) is called.
   base::Vector<const uint8_t> compiled_module_bytes_;
