@@ -29,24 +29,23 @@ class Simd128;
 // Format: kind, log2Size, code, machineType, shortName, typeName
 //
 // Some of these types are from proposals that are not standardized yet:
-// - "ref"/"optref" (a.k.a. "ref null") per
-//   https://github.com/WebAssembly/function-references
-// - "rtt" per https://github.com/WebAssembly/gc
+// - "ref"/"ref null" https://github.com/WebAssembly/function-references
+// - "rtt", "i8" and "i16" per https://github.com/WebAssembly/gc
 #define FOREACH_NUMERIC_VALUE_TYPE(V)    \
   V(I32, 2, I32, Int32, 'i', "i32")      \
   V(I64, 3, I64, Int64, 'l', "i64")      \
   V(F32, 2, F32, Float32, 'f', "f32")    \
   V(F64, 3, F64, Float64, 'd', "f64")    \
-  V(S128, 4, S128, Simd128, 's', "s128") \
+  V(S128, 4, S128, Simd128, 's', "v128") \
   V(I8, 0, I8, Int8, 'b', "i8")          \
   V(I16, 1, I16, Int16, 'h', "i16")
 
-#define FOREACH_VALUE_TYPE(V)                                    \
-  V(Void, -1, Void, None, 'v', "<void>")                         \
-  FOREACH_NUMERIC_VALUE_TYPE(V)                                  \
-  V(Rtt, kTaggedSizeLog2, Rtt, TaggedPointer, 't', "rtt")        \
-  V(Ref, kTaggedSizeLog2, Ref, AnyTagged, 'r', "ref")            \
-  V(OptRef, kTaggedSizeLog2, OptRef, AnyTagged, 'n', "ref null") \
+#define FOREACH_VALUE_TYPE(V)                                      \
+  V(Void, -1, Void, None, 'v', "<void>")                           \
+  FOREACH_NUMERIC_VALUE_TYPE(V)                                    \
+  V(Rtt, kTaggedSizeLog2, Rtt, TaggedPointer, 't', "rtt")          \
+  V(Ref, kTaggedSizeLog2, Ref, AnyTagged, 'r', "ref")              \
+  V(RefNull, kTaggedSizeLog2, RefNull, AnyTagged, 'n', "ref null") \
   V(Bottom, -1, Void, None, '*', "<bot>")
 
 constexpr int kMaxValueTypeSize = 16;  // bytes
@@ -63,9 +62,17 @@ class HeapType {
     kFunc = kV8MaxWasmTypes,  // shorthand: c
     kEq,                      // shorthand: q
     kI31,                     // shorthand: j
-    kData,                    // shorthand: o
+    kStruct,                  // shorthand: o
     kArray,                   // shorthand: g
-    kAny,                     // shorthand: a. Aka kExtern.
+    kAny,                     //
+    kExtern,                  // shorthand: a.
+    kString,                  // shorthand: w.
+    kStringViewWtf8,          // shorthand: x.
+    kStringViewWtf16,         // shorthand: y.
+    kStringViewIter,          // shorthand: z.
+    kNone,                    //
+    kNoFunc,                  //
+    kNoExtern,                //
     // This value is used to represent failures in the parsing of heap types and
     // does not correspond to a wasm heap type. It has to be last in this list.
     kBottom
@@ -80,12 +87,27 @@ class HeapType {
       case ValueTypeCode::kI31RefCode:
         return HeapType(kI31);
       case ValueTypeCode::kAnyRefCode:
-      case ValueTypeCode::kAnyRefCodeAlias:
         return HeapType(kAny);
-      case ValueTypeCode::kDataRefCode:
-        return HeapType(kData);
+      case ValueTypeCode::kExternRefCode:
+        return HeapType(kExtern);
+      case ValueTypeCode::kStructRefCode:
+        return HeapType(kStruct);
       case ValueTypeCode::kArrayRefCode:
         return HeapType(kArray);
+      case ValueTypeCode::kStringRefCode:
+        return HeapType(kString);
+      case ValueTypeCode::kStringViewWtf8Code:
+        return HeapType(kStringViewWtf8);
+      case ValueTypeCode::kStringViewWtf16Code:
+        return HeapType(kStringViewWtf16);
+      case ValueTypeCode::kStringViewIterCode:
+        return HeapType(kStringViewIter);
+      case ValueTypeCode::kNoneCode:
+        return HeapType(kNone);
+      case ValueTypeCode::kNoExternCode:
+        return HeapType(kNoExtern);
+      case ValueTypeCode::kNoFuncCode:
+        return HeapType(kNoFunc);
       default:
         return HeapType(kBottom);
     }
@@ -134,12 +156,30 @@ class HeapType {
         return std::string("eq");
       case kI31:
         return std::string("i31");
-      case kData:
-        return std::string("data");
+      case kStruct:
+        return std::string("struct");
       case kArray:
         return std::string("array");
+      case kExtern:
+        return std::string("extern");
       case kAny:
-        return std::string(FLAG_experimental_wasm_gc ? "any" : "extern");
+        return std::string("any");
+      case kString:
+        return std::string("string");
+      case kStringViewWtf8:
+        return std::string("stringview_wtf8");
+      case kStringViewWtf16:
+        return std::string("stringview_wtf16");
+      case kStringViewIter:
+        return std::string("stringview_iter");
+      case kNone:
+        return std::string("none");
+      case kNoExtern:
+        return std::string("noextern");
+      case kNoFunc:
+        return std::string("nofunc");
+      case kBottom:
+        return std::string("<bot>");
       default:
         return std::to_string(representation_);
     }
@@ -157,12 +197,28 @@ class HeapType {
         return mask | kEqRefCode;
       case kI31:
         return mask | kI31RefCode;
-      case kData:
-        return mask | kDataRefCode;
+      case kStruct:
+        return mask | kStructRefCode;
       case kArray:
         return mask | kArrayRefCode;
+      case kExtern:
+        return mask | kExternRefCode;
       case kAny:
         return mask | kAnyRefCode;
+      case kString:
+        return mask | kStringRefCode;
+      case kStringViewWtf8:
+        return mask | kStringViewWtf8Code;
+      case kStringViewWtf16:
+        return mask | kStringViewWtf16Code;
+      case kStringViewIter:
+        return mask | kStringViewIterCode;
+      case kNone:
+        return mask | kNoneCode;
+      case kNoExtern:
+        return mask | kNoExternCode;
+      case kNoFunc:
+        return mask | kNoFuncCode;
       default:
         return static_cast<int32_t>(representation_);
     }
@@ -201,11 +257,11 @@ constexpr bool is_numeric(ValueKind kind) {
 }
 
 constexpr bool is_reference(ValueKind kind) {
-  return kind == kRef || kind == kOptRef || kind == kRtt;
+  return kind == kRef || kind == kRefNull || kind == kRtt;
 }
 
 constexpr bool is_object_reference(ValueKind kind) {
-  return kind == kRef || kind == kOptRef;
+  return kind == kRef || kind == kRefNull;
 }
 
 constexpr int value_kind_size_log2(ValueKind kind) {
@@ -303,14 +359,32 @@ class ValueType {
     DCHECK(kind == kBottom || kind <= kI16);
     return ValueType(KindField::encode(kind));
   }
-  static constexpr ValueType Ref(uint32_t heap_type, Nullability nullability) {
+  static constexpr ValueType Ref(uint32_t heap_type) {
+    DCHECK(HeapType(heap_type).is_valid());
+    return ValueType(KindField::encode(kRef) |
+                     HeapTypeField::encode(heap_type));
+  }
+  static constexpr ValueType Ref(HeapType heap_type) {
+    return Ref(heap_type.representation());
+  }
+  static constexpr ValueType RefNull(uint32_t heap_type) {
+    DCHECK(HeapType(heap_type).is_valid());
+    return ValueType(KindField::encode(kRefNull) |
+                     HeapTypeField::encode(heap_type));
+  }
+  static constexpr ValueType RefNull(HeapType heap_type) {
+    return RefNull(heap_type.representation());
+  }
+  static constexpr ValueType RefMaybeNull(uint32_t heap_type,
+                                          Nullability nullability) {
     DCHECK(HeapType(heap_type).is_valid());
     return ValueType(
-        KindField::encode(nullability == kNullable ? kOptRef : kRef) |
+        KindField::encode(nullability == kNullable ? kRefNull : kRef) |
         HeapTypeField::encode(heap_type));
   }
-  static constexpr ValueType Ref(HeapType heap_type, Nullability nullability) {
-    return Ref(heap_type.representation(), nullability);
+  static constexpr ValueType RefMaybeNull(HeapType heap_type,
+                                          Nullability nullability) {
+    return RefMaybeNull(heap_type.representation(), nullability);
   }
 
   static constexpr ValueType Rtt(uint32_t type_index) {
@@ -320,7 +394,7 @@ class ValueType {
   }
 
   static constexpr ValueType FromIndex(ValueKind kind, uint32_t index) {
-    DCHECK(kind == kOptRef || kind == kRef || kind == kRtt);
+    DCHECK(kind == kRefNull || kind == kRef || kind == kRtt);
     return ValueType(KindField::encode(kind) | HeapTypeField::encode(index));
   }
 
@@ -330,6 +404,7 @@ class ValueType {
   }
 
   /******************************** Type checks *******************************/
+  // Includes s128.
   constexpr bool is_numeric() const { return wasm::is_numeric(kind()); }
 
   constexpr bool is_reference() const { return wasm::is_reference(kind()); }
@@ -338,10 +413,11 @@ class ValueType {
     return wasm::is_object_reference(kind());
   }
 
-  constexpr bool is_nullable() const { return kind() == kOptRef; }
+  constexpr bool is_nullable() const { return kind() == kRefNull; }
+  constexpr bool is_non_nullable() const { return kind() == kRef; }
 
   constexpr bool is_reference_to(uint32_t htype) const {
-    return (kind() == kRef || kind() == kOptRef) &&
+    return (kind() == kRef || kind() == kRefNull) &&
            heap_representation() == htype;
   }
 
@@ -355,17 +431,28 @@ class ValueType {
 
   constexpr bool is_bottom() const { return kind() == kBottom; }
 
+  // These can occur as the result of type propagation, but never in
+  // reachable control flow.
+  constexpr bool is_uninhabited() const {
+    return is_non_nullable() && (is_reference_to(HeapType::kNone) ||
+                                 is_reference_to(HeapType::kNoExtern) ||
+                                 is_reference_to(HeapType::kNoFunc));
+  }
+
   constexpr bool is_packed() const { return wasm::is_packed(kind()); }
 
   constexpr ValueType Unpacked() const {
     return is_packed() ? Primitive(kI32) : *this;
   }
 
-  // Returns the version of this type that does not allow null values. Handles
-  // bottom.
+  // If {this} is (ref null $t), returns (ref $t). Otherwise, returns {this}.
   constexpr ValueType AsNonNull() const {
-    DCHECK(is_object_reference() || is_bottom());
-    return is_nullable() ? Ref(heap_type(), kNonNullable) : *this;
+    return is_nullable() ? Ref(heap_type()) : *this;
+  }
+
+  // If {this} is (ref $t), returns (ref null $t). Otherwise, returns {this}.
+  constexpr ValueType AsNullable() const {
+    return is_non_nullable() ? RefNull(heap_type()) : *this;
   }
 
   /***************************** Field Accessors ******************************/
@@ -385,7 +472,7 @@ class ValueType {
   }
   constexpr Nullability nullability() const {
     DCHECK(is_object_reference());
-    return kind() == kOptRef ? kNullable : kNonNullable;
+    return kind() == kRefNull ? kNullable : kNonNullable;
   }
 
   // Useful when serializing this type to store it into a runtime object.
@@ -437,7 +524,7 @@ class ValueType {
       case MachineRepresentation::kFloat64:
         return Primitive(kF64);
       case MachineRepresentation::kTaggedPointer:
-        return Ref(HeapType::kAny, kNullable);
+        return RefNull(HeapType::kAny);
       case MachineRepresentation::kSimd128:
         return Primitive(kS128);
       default:
@@ -452,41 +539,54 @@ class ValueType {
   // For compatibility with the reftypes and exception-handling proposals, this
   // function prioritizes shorthand encodings
   // (e.g., Ref(HeapType::kFunc, kNullable).value_type_code will return
-  // kFuncrefCode and not kOptRefCode).
+  // kFuncrefCode and not kRefNullCode).
   constexpr ValueTypeCode value_type_code() const {
     DCHECK_NE(kBottom, kind());
     switch (kind()) {
-      case kOptRef:
+      case kRefNull:
         switch (heap_representation()) {
           case HeapType::kFunc:
             return kFuncRefCode;
           case HeapType::kEq:
             return kEqRefCode;
+          case HeapType::kExtern:
+            return kExternRefCode;
           case HeapType::kAny:
             return kAnyRefCode;
-          default:
-            return kOptRefCode;
-        }
-      case kRef:
-        switch (heap_representation()) {
           case HeapType::kI31:
             return kI31RefCode;
-          case HeapType::kData:
-            return kDataRefCode;
+          case HeapType::kStruct:
+            return kStructRefCode;
           case HeapType::kArray:
             return kArrayRefCode;
+          case HeapType::kString:
+            return kStringRefCode;
+          case HeapType::kStringViewWtf8:
+            return kStringViewWtf8Code;
+          case HeapType::kStringViewWtf16:
+            return kStringViewWtf16Code;
+          case HeapType::kStringViewIter:
+            return kStringViewIterCode;
+          case HeapType::kNone:
+            return kNoneCode;
+          case HeapType::kNoExtern:
+            return kNoExternCode;
+          case HeapType::kNoFunc:
+            return kNoFuncCode;
           default:
-            return kRefCode;
+            return kRefNullCode;
         }
-      case kVoid:
-        return kVoidCode;
-      case kRtt:
-        return kRttCode;
+      case kRef:
+        return kRefCode;
 #define NUMERIC_TYPE_CASE(kind, ...) \
   case k##kind:                      \
     return k##kind##Code;
         FOREACH_NUMERIC_VALUE_TYPE(NUMERIC_TYPE_CASE)
 #undef NUMERIC_TYPE_CASE
+      // The RTT value type can not be used in WebAssembly and is a
+      // compiler-internal type only.
+      case kRtt:
+      case kVoid:
       case kBottom:
         // Unreachable code
         return kVoidCode;
@@ -496,18 +596,7 @@ class ValueType {
   // Returns true iff the heap type is needed to encode this type in the wasm
   // binary format, taking into account available type shorthands.
   constexpr bool encoding_needs_heap_type() const {
-    switch (kind()) {
-      case kRef:
-        return heap_representation() != HeapType::kI31 &&
-               heap_representation() != HeapType::kArray &&
-               heap_representation() != HeapType::kData;
-      case kOptRef:
-        return heap_representation() != HeapType::kFunc &&
-               heap_representation() != HeapType::kEq &&
-               heap_representation() != HeapType::kAny;
-      default:
-        return false;
-    }
+    return kind() == kRef || (kind() == kRefNull && heap_type().is_index());
   }
 
   /****************************** Pretty-printing *****************************/
@@ -517,12 +606,26 @@ class ValueType {
     std::ostringstream buf;
     switch (kind()) {
       case kRef:
-      case kOptRef:
-        if (encoding_needs_heap_type()) {
-          buf << "(ref " << (kind() == kOptRef ? "null " : "")
-              << heap_type().name() << ")";
+        buf << "(ref " << heap_type().name() << ")";
+        break;
+      case kRefNull:
+        if (heap_type().is_generic()) {
+          switch (heap_type().representation()) {
+            case HeapType::kNone:
+              buf << "nullref";
+              break;
+            case HeapType::kNoExtern:
+              buf << "nullexternref";
+              break;
+            case HeapType::kNoFunc:
+              buf << "nullfuncref";
+              break;
+            default:
+              buf << heap_type().name() << "ref";
+              break;
+          }
         } else {
-          buf << heap_type().name() << "ref";
+          buf << "(ref null " << heap_type().name() << ")";
         }
         break;
       case kRtt:
@@ -550,6 +653,8 @@ class ValueType {
   static constexpr int kKindBits = 5;
   static constexpr int kHeapTypeBits = 20;
 
+  static const intptr_t kBitFieldOffset;
+
  private:
   // {hash_value} directly reads {bit_field_}.
   friend size_t hash_value(ValueType type);
@@ -575,6 +680,10 @@ class ValueType {
 
   uint32_t bit_field_;
 };
+ASSERT_TRIVIALLY_COPYABLE(ValueType);
+
+inline constexpr intptr_t ValueType::kBitFieldOffset =
+    offsetof(ValueType, bit_field_);
 
 static_assert(sizeof(ValueType) <= kUInt32Size,
               "ValueType is small and can be passed by value");
@@ -602,22 +711,28 @@ constexpr ValueType kWasmI16 = ValueType::Primitive(kI16);
 constexpr ValueType kWasmVoid = ValueType::Primitive(kVoid);
 constexpr ValueType kWasmBottom = ValueType::Primitive(kBottom);
 // Established reference-type and wasm-gc proposal shorthands.
-constexpr ValueType kWasmFuncRef = ValueType::Ref(HeapType::kFunc, kNullable);
-constexpr ValueType kWasmAnyRef = ValueType::Ref(HeapType::kAny, kNullable);
-constexpr ValueType kWasmEqRef = ValueType::Ref(HeapType::kEq, kNullable);
-constexpr ValueType kWasmI31Ref = ValueType::Ref(HeapType::kI31, kNonNullable);
-constexpr ValueType kWasmDataRef =
-    ValueType::Ref(HeapType::kData, kNonNullable);
-constexpr ValueType kWasmArrayRef =
-    ValueType::Ref(HeapType::kArray, kNonNullable);
+constexpr ValueType kWasmFuncRef = ValueType::RefNull(HeapType::kFunc);
+constexpr ValueType kWasmAnyRef = ValueType::RefNull(HeapType::kAny);
+constexpr ValueType kWasmExternRef = ValueType::RefNull(HeapType::kExtern);
+constexpr ValueType kWasmEqRef = ValueType::RefNull(HeapType::kEq);
+constexpr ValueType kWasmI31Ref = ValueType::RefNull(HeapType::kI31);
+constexpr ValueType kWasmStructRef = ValueType::RefNull(HeapType::kStruct);
+constexpr ValueType kWasmArrayRef = ValueType::RefNull(HeapType::kArray);
+constexpr ValueType kWasmStringRef = ValueType::RefNull(HeapType::kString);
+constexpr ValueType kWasmStringViewWtf8 =
+    ValueType::RefNull(HeapType::kStringViewWtf8);
+constexpr ValueType kWasmStringViewWtf16 =
+    ValueType::RefNull(HeapType::kStringViewWtf16);
+constexpr ValueType kWasmStringViewIter =
+    ValueType::RefNull(HeapType::kStringViewIter);
+constexpr ValueType kWasmNullRef = ValueType::RefNull(HeapType::kNone);
+constexpr ValueType kWasmNullExternRef =
+    ValueType::RefNull(HeapType::kNoExtern);
+constexpr ValueType kWasmNullFuncRef = ValueType::RefNull(HeapType::kNoFunc);
 
 // Constants used by the generic js-to-wasm wrapper.
 constexpr int kWasmValueKindBitsMask = (1u << ValueType::kKindBits) - 1;
 
-// This is used in wasm.tq.
-constexpr ValueType kWasmAnyNonNullableRef =
-    ValueType::Ref(HeapType::kAny, kNonNullable);
-
 #define FOREACH_WASMVALUE_CTYPES(V) \
   V(kI32, int32_t)                  \
   V(kI64, int64_t)                  \
@@ -657,8 +772,8 @@ class LoadType {
       : val_(val) {}
 
   constexpr LoadTypeValue value() const { return val_; }
-  constexpr unsigned size_log_2() const { return kLoadSizeLog2[val_]; }
-  constexpr unsigned size() const { return 1 << size_log_2(); }
+  constexpr uint8_t size_log_2() const { return kLoadSizeLog2[val_]; }
+  constexpr uint8_t size() const { return kLoadSize[val_]; }
   constexpr ValueType value_type() const { return kValueType[val_]; }
   constexpr MachineType mem_type() const { return kMemType[val_]; }
 
@@ -686,6 +801,15 @@ class LoadType {
  private:
   const LoadTypeValue val_;
 
+  static constexpr uint8_t kLoadSize[] = {
+  // MSVC wants a static_cast here.
+#define LOAD_SIZE(_, __, memtype) \
+  static_cast<uint8_t>(           \
+      ElementSizeInBytes(MachineType::memtype().representation())),
+      FOREACH_LOAD_TYPE(LOAD_SIZE)
+#undef LOAD_SIZE
+  };
+
   static constexpr uint8_t kLoadSizeLog2[] = {
   // MSVC wants a static_cast here.
 #define LOAD_SIZE(_, __, memtype) \
