@@ -11,12 +11,14 @@
 
 #include <stdint.h>
 
-#include "src/common/globals.h"
+#include "src/base/macros.h"
 
 namespace v8 {
 namespace internal {
 namespace wasm {
 
+using Address = uintptr_t;
+
 V8_EXPORT_PRIVATE void f32_trunc_wrapper(Address data);
 
 V8_EXPORT_PRIVATE void f32_floor_wrapper(Address data);
@@ -116,6 +118,12 @@ void array_copy_wrapper(Address raw_instance, Address raw_dst_array,
                         uint32_t dst_index, Address raw_src_array,
                         uint32_t src_index, uint32_t length);
 
+// The initial value is passed as an int64_t on the stack. Cannot handle s128
+// other than 0.
+void array_fill_wrapper(Address raw_array, uint32_t index, uint32_t length,
+                        uint32_t emit_write_barrier, uint32_t raw_type,
+                        Address initial_value_addr);
+
 using WasmTrapCallbackForTesting = void (*)();
 
 V8_EXPORT_PRIVATE void set_trap_callback_for_testing(
