@@ -28,20 +28,23 @@ std::ostream& operator<<(std::ostream& os, const FunctionSig& sig) {
   return os;
 }
 
-// TODO(7748): Once we have a story for JS interaction of structs/arrays, this
-// function should become independent of module. Remove 'module' parameter in
-// this function as well as all transitive callees that no longer need it
-// (In essence, revert
-// https://chromium-review.googlesource.com/c/v8/v8/+/2413251).
-bool IsJSCompatibleSignature(const FunctionSig* sig, const WasmModule* module,
-                             const WasmFeatures& enabled_features) {
+bool IsJSCompatibleSignature(const FunctionSig* sig) {
   for (auto type : sig->all()) {
-    // TODO(7748): Allow structs, arrays, and rtts when their JS-interaction is
-    // decided on.
-    if (type == kWasmS128 || type.is_rtt() ||
-        (type.has_index() && !module->has_signature(type.ref_index()))) {
+    // Structs and arrays may only be passed via externref.
+    // Rtts are implicit and can not be used explicitly.
+    if (type == kWasmS128 || type.is_rtt()) {
       return false;
     }
+    if (type.is_object_reference()) {
+      switch (type.heap_type().representation()) {
+        case HeapType::kStringViewWtf8:
+        case HeapType::kStringViewWtf16:
+        case HeapType::kStringViewIter:
+          return false;
+        default:
+          break;
+      }
+    }
   }
   return true;
 }
