@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 # Copyright 2018 the V8 project authors. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
@@ -15,12 +15,12 @@ import sys
 
 def asm_to_inl_asm(in_filename, out_filename):
   with open(in_filename, 'r') as infile, open(out_filename, 'wb') as outfile:
-    outfile.write('__asm__(\n')
+    outfile.write(b'__asm__(\n')
     for line in infile:
       # Escape " in .S file before outputing it to inline asm file.
       line = line.replace('"', '\\"')
-      outfile.write('  "%s\\n"\n' % line.rstrip())
-    outfile.write(');\n')
+      outfile.write(b'  "%s\\n"\n' % line.rstrip().encode('utf8'))
+    outfile.write(b');\n')
   return 0
 
 if __name__ == '__main__':
