@@ -1,12 +1,8 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # Copyright 2014 the V8 project authors. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
-# for py2/py3 compatibility
-from __future__ import print_function
-
-from collections import namedtuple
 import json
 import os
 import platform
@@ -22,12 +18,20 @@ import mock
 # Requires python-coverage and python-mock. Native python coverage
 # version >= 3.7.1 should be installed to get the best speed.
 
-BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-RUN_PERF = os.path.join(BASE_DIR, 'run_perf.py')
-TEST_DATA = os.path.join(BASE_DIR, 'unittests', 'testdata')
+TOOLS_ROOT = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
+sys.path.insert(0, TOOLS_ROOT)
+import run_perf
+from testrunner.local import command
+from testrunner.objects.output import Output, NULL_OUTPUT
+
+
+RUN_PERF = os.path.join(TOOLS_ROOT, 'run_perf.py')
+TEST_DATA = os.path.join(TOOLS_ROOT, 'unittests', 'testdata')
 
 TEST_WORKSPACE = os.path.join(tempfile.gettempdir(), 'test-v8-run-perf')
 
+SORT_KEY = lambda x: x['graphs']
+
 V8_JSON = {
   'path': ['.'],
   'owners': ['username@chromium.org'],
@@ -43,6 +47,35 @@ V8_JSON = {
   ]
 }
 
+V8_VARIANTS_JSON = {
+    'path': ['.'],
+    'owners': ['username@chromium.org'],
+    'binary': 'd7',
+    'timeout': 60,
+    'flags': ['--flag'],
+    'main': 'run.js',
+    'run_count': 1,
+    'results_regexp': '%s: (.+)$',
+    'variants': [{
+        'name': 'default',
+        'flags': [],
+    }, {
+        'name': 'VariantA',
+        'flags': ['--variant-a-flag'],
+    }, {
+        'name': 'VariantB',
+        'flags': ['--variant-b-flag'],
+    }],
+    'tests': [
+        {
+            'name': 'Richards',
+        },
+        {
+            'name': 'DeltaBlue',
+        },
+    ]
+}
+
 V8_NESTED_SUITES_JSON = {
   'path': ['.'],
   'owners': ['username@chromium.org'],
@@ -90,17 +123,21 @@ V8_GENERIC_JSON = {
   'units': 'ms',
 }
 
+
+class UnitTest(unittest.TestCase):
+  def testBuildDirectory(self):
+    base_path = os.path.join(TEST_DATA, 'builddirs', 'dir1', 'out')
+    expected_path = os.path.join(base_path, 'build')
+    self.assertEqual(expected_path,
+                     run_perf.find_build_directory(base_path, 'x64'))
+
+
 class PerfTest(unittest.TestCase):
   @classmethod
   def setUpClass(cls):
-    sys.path.insert(0, BASE_DIR)
     cls._cov = coverage.coverage(
-        include=([os.path.join(BASE_DIR, 'run_perf.py')]))
+        include=([os.path.join(TOOLS_ROOT, 'run_perf.py')]))
     cls._cov.start()
-    import run_perf
-    from testrunner.local import command
-    from testrunner.objects.output import Output, NULL_OUTPUT
-    global command, run_perf, Output, NULL_OUTPUT
 
   @classmethod
   def tearDownClass(cls):
@@ -124,13 +161,17 @@ class PerfTest(unittest.TestCase):
     with open(self._test_input, 'w') as f:
       f.write(json.dumps(json_content))
 
-  def _MockCommand(self, *args, **kwargs):
+  def _MockCommand(self, raw_dirs, raw_outputs, *args, **kwargs):
+    on_bots = kwargs.pop('on_bots', False)
     # Fake output for each test run.
-    test_outputs = [Output(stdout=arg,
-                           timed_out=kwargs.get('timed_out', False),
-                           exit_code=kwargs.get('exit_code', 0),
-                           duration=42)
-                    for arg in args[1]]
+    test_outputs = [
+        Output(
+            stdout=output,
+            timed_out=kwargs.get('timed_out', False),
+            exit_code=kwargs.get('exit_code', 0),
+            duration=42) for output in raw_outputs
+    ]
+
     def create_cmd(*args, **kwargs):
       cmd = mock.MagicMock()
       def execute(*args, **kwargs):
@@ -142,10 +183,27 @@ class PerfTest(unittest.TestCase):
         run_perf.command, 'PosixCommand',
         mock.MagicMock(side_effect=create_cmd)).start()
 
+    build_dir = 'Release' if on_bots else 'x64.release'
+    out_dirs = ['out', 'out-secondary']
+    return_values = [
+      os.path.join(os.path.dirname(TOOLS_ROOT), out, build_dir)
+      for out in out_dirs
+    ]
+    mock.patch.object(
+        run_perf, 'find_build_directory',
+        mock.MagicMock(side_effect=return_values)).start()
+
     # Check that d8 is called from the correct cwd for each test run.
-    dirs = [os.path.join(TEST_WORKSPACE, arg) for arg in args[0]]
-    def chdir(*args, **kwargs):
-      self.assertEqual(dirs.pop(), args[0])
+    dirs = [os.path.join(TEST_WORKSPACE, dir) for dir in raw_dirs]
+
+    def chdir(dir, *args, **kwargs):
+      if not dirs:
+        raise Exception("Missing test chdir '%s'" % dir)
+      expected_dir = dirs.pop()
+      self.assertEqual(
+          expected_dir, dir,
+          "Unexpected chdir: expected='%s' got='%s'" % (expected_dir, dir))
+
     os.chdir = mock.MagicMock(side_effect=chdir)
 
     subprocess.check_call = mock.MagicMock()
@@ -165,13 +223,19 @@ class PerfTest(unittest.TestCase):
     with open(file_name or self._test_output) as f:
       return json.load(f)
 
+  def _VerifyResultTraces(self, traces, file_name=None):
+    sorted_expected = sorted(traces, key=SORT_KEY)
+    sorted_results = sorted(
+        self._LoadResults(file_name)['traces'], key=SORT_KEY)
+    self.assertListEqual(sorted_expected, sorted_results)
+
   def _VerifyResults(self, suite, units, traces, file_name=None):
     self.assertListEqual(sorted([
       {'units': units,
        'graphs': [suite, trace['name']],
        'results': trace['results'],
-       'stddev': trace['stddev']} for trace in traces]),
-      sorted(self._LoadResults(file_name)['traces']))
+       'stddev': trace['stddev']} for trace in traces], key=SORT_KEY),
+      sorted(self._LoadResults(file_name)['traces'], key=SORT_KEY))
 
   def _VerifyRunnableDurations(self, runs, timeout, file_name=None):
     self.assertListEqual([
@@ -186,21 +250,23 @@ class PerfTest(unittest.TestCase):
     self.assertListEqual(errors, self._LoadResults()['errors'])
 
   def _VerifyMock(self, binary, *args, **kwargs):
-    shell = os.path.join(os.path.dirname(BASE_DIR), binary)
+    shell = os.path.join(os.path.dirname(TOOLS_ROOT), binary)
     command.Command.assert_called_with(
         cmd_prefix=[],
         shell=shell,
         args=list(args),
-        timeout=kwargs.get('timeout', 60))
+        timeout=kwargs.get('timeout', 60),
+        handle_sigterm=True)
 
   def _VerifyMockMultiple(self, *args, **kwargs):
     self.assertEqual(len(args), len(command.Command.call_args_list))
     for arg, actual in zip(args, command.Command.call_args_list):
       expected = {
         'cmd_prefix': [],
-        'shell': os.path.join(os.path.dirname(BASE_DIR), arg[0]),
+        'shell': os.path.join(os.path.dirname(TOOLS_ROOT), arg[0]),
         'args': list(arg[1:]),
-        'timeout': kwargs.get('timeout', 60)
+        'timeout': kwargs.get('timeout', 60),
+        'handle_sigterm': True,
       }
       self.assertTupleEqual((expected, ), actual)
 
@@ -217,6 +283,60 @@ class PerfTest(unittest.TestCase):
     self._VerifyMock(
         os.path.join('out', 'x64.release', 'd7'), '--flag', 'run.js')
 
+  def testOneRunVariants(self):
+    self._WriteTestInput(V8_VARIANTS_JSON)
+    self._MockCommand(['.', '.', '.'], [
+        'x\nRichards: 3.3\nDeltaBlue: 3000\ny\n',
+        'x\nRichards: 2.2\nDeltaBlue: 2000\ny\n',
+        'x\nRichards: 1.1\nDeltaBlue: 1000\ny\n'
+    ])
+    self.assertEqual(0, self._CallMain())
+    self._VerifyResultTraces([
+        {
+            'units': 'score',
+            'graphs': ['test', 'default', 'Richards'],
+            'results': [1.1],
+            'stddev': ''
+        },
+        {
+            'units': 'score',
+            'graphs': ['test', 'default', 'DeltaBlue'],
+            'results': [1000],
+            'stddev': ''
+        },
+        {
+            'units': 'score',
+            'graphs': ['test', 'VariantA', 'Richards'],
+            'results': [2.2],
+            'stddev': ''
+        },
+        {
+            'units': 'score',
+            'graphs': ['test', 'VariantA', 'DeltaBlue'],
+            'results': [2000],
+            'stddev': ''
+        },
+        {
+            'units': 'score',
+            'graphs': ['test', 'VariantB', 'Richards'],
+            'results': [3.3],
+            'stddev': ''
+        },
+        {
+            'units': 'score',
+            'graphs': ['test', 'VariantB', 'DeltaBlue'],
+            'results': [3000],
+            'stddev': ''
+        },
+    ])
+    self._VerifyErrors([])
+    self._VerifyMockMultiple(
+        (os.path.join('out', 'x64.release', 'd7'), '--flag', 'run.js'),
+        (os.path.join('out', 'x64.release',
+                      'd7'), '--flag', '--variant-a-flag', 'run.js'),
+        (os.path.join('out', 'x64.release',
+                      'd7'), '--flag', '--variant-b-flag', 'run.js'))
+
   def testOneRunWithTestFlags(self):
     test_input = dict(V8_JSON)
     test_input['test_flags'] = ['2', 'test_name']
@@ -340,7 +460,7 @@ class PerfTest(unittest.TestCase):
        'graphs': ['test', 'DeltaBlue'],
        'results': [200.0],
        'stddev': ''},
-      ]), sorted(self._LoadResults()['traces']))
+      ], key=SORT_KEY), sorted(self._LoadResults()['traces'], key=SORT_KEY))
     self._VerifyErrors([])
     self._VerifyMockMultiple(
         (os.path.join('out', 'x64.release', 'd7'), '--flag', 'run.js'),
@@ -351,9 +471,10 @@ class PerfTest(unittest.TestCase):
         (os.path.join('out', 'x64.release', 'd8'),
          '--flag', '--flag2', 'run.js'))
 
+
   def testOneRunStdDevRegExp(self):
     test_input = dict(V8_JSON)
-    test_input['stddev_regexp'] = '^%s\-stddev: (.+)$'
+    test_input['stddev_regexp'] = r'^%s-stddev: (.+)$'
     self._WriteTestInput(test_input)
     self._MockCommand(['.'], ['Richards: 1.234\nRichards-stddev: 0.23\n'
                               'DeltaBlue: 10657567\nDeltaBlue-stddev: 106\n'])
@@ -368,7 +489,7 @@ class PerfTest(unittest.TestCase):
 
   def testTwoRunsStdDevRegExp(self):
     test_input = dict(V8_JSON)
-    test_input['stddev_regexp'] = '^%s\-stddev: (.+)$'
+    test_input['stddev_regexp'] = r'^%s-stddev: (.+)$'
     test_input['run_count'] = 2
     self._WriteTestInput(test_input)
     self._MockCommand(['.'], ['Richards: 3\nRichards-stddev: 0.7\n'
@@ -380,23 +501,25 @@ class PerfTest(unittest.TestCase):
       {'name': 'Richards', 'results': [2.0, 3.0], 'stddev': '0.7'},
       {'name': 'DeltaBlue', 'results': [5.0, 6.0], 'stddev': '0.8'},
     ])
-    self._VerifyErrors(
-        ['Test test/Richards should only run once since a stddev is provided '
-         'by the test.',
-         'Test test/DeltaBlue should only run once since a stddev is provided '
-         'by the test.',
-         'Regexp "^DeltaBlue\-stddev: (.+)$" did not match for test '
-         'test/DeltaBlue.'])
+    self._VerifyErrors([
+        'Test test/Richards should only run once since a stddev is provided '
+        'by the test.',
+        'Test test/DeltaBlue should only run once since a stddev is provided '
+        'by the test.',
+        r'Regexp "^DeltaBlue-stddev: (.+)$" did not match for test '
+        r'test/DeltaBlue.'
+    ])
     self._VerifyMock(
         os.path.join('out', 'x64.release', 'd7'), '--flag', 'run.js')
 
   def testBuildbot(self):
     self._WriteTestInput(V8_JSON)
-    self._MockCommand(['.'], ['Richards: 1.234\nDeltaBlue: 10657567\n'])
+    self._MockCommand(['.'], ['Richards: 1.234\nDeltaBlue: 10657567\n'],
+                      on_bots=True)
     mock.patch.object(
         run_perf.Platform, 'ReadBuildConfig',
         mock.MagicMock(return_value={'is_android': False})).start()
-    self.assertEqual(0, self._CallMain('--buildbot'))
+    self.assertEqual(0, self._CallMain())
     self._VerifyResults('test', 'score', [
       {'name': 'Richards', 'results': [1.234], 'stddev': ''},
       {'name': 'DeltaBlue', 'results': [10657567.0], 'stddev': ''},
@@ -408,11 +531,12 @@ class PerfTest(unittest.TestCase):
     test_input = dict(V8_JSON)
     test_input['total'] = True
     self._WriteTestInput(test_input)
-    self._MockCommand(['.'], ['Richards: 1.234\nDeltaBlue: 10657567\n'])
+    self._MockCommand(['.'], ['Richards: 1.234\nDeltaBlue: 10657567\n'],
+                      on_bots=True)
     mock.patch.object(
         run_perf.Platform, 'ReadBuildConfig',
         mock.MagicMock(return_value={'is_android': False})).start()
-    self.assertEqual(0, self._CallMain('--buildbot'))
+    self.assertEqual(0, self._CallMain())
     self._VerifyResults('test', 'score', [
       {'name': 'Richards', 'results': [1.234], 'stddev': ''},
       {'name': 'DeltaBlue', 'results': [10657567.0], 'stddev': ''},
@@ -425,11 +549,12 @@ class PerfTest(unittest.TestCase):
     test_input = dict(V8_JSON)
     test_input['total'] = True
     self._WriteTestInput(test_input)
-    self._MockCommand(['.'], ['x\nRichards: bla\nDeltaBlue: 10657567\ny\n'])
+    self._MockCommand(['.'], ['x\nRichards: bla\nDeltaBlue: 10657567\ny\n'],
+                      on_bots=True)
     mock.patch.object(
         run_perf.Platform, 'ReadBuildConfig',
         mock.MagicMock(return_value={'is_android': False})).start()
-    self.assertEqual(1, self._CallMain('--buildbot'))
+    self.assertEqual(1, self._CallMain())
     self._VerifyResults('test', 'score', [
       {'name': 'DeltaBlue', 'results': [10657567.0], 'stddev': ''},
     ])
@@ -452,6 +577,37 @@ class PerfTest(unittest.TestCase):
     self._VerifyMock(
         os.path.join('out', 'x64.release', 'd7'), '--flag', 'run.js')
 
+  def testFilterInvalidRegexp(self):
+    self._WriteTestInput(V8_JSON)
+    self._MockCommand(['.'], ['x\nRichards: 1.234\nDeltaBlue: 10657567\ny\n'])
+    self.assertNotEqual(0, self._CallMain("--filter=((("))
+    self._VerifyMock(os.path.join('out', 'Release', 'd7'), '--flag', 'run.js')
+
+  def testFilterRegexpMatchAll(self):
+    self._WriteTestInput(V8_JSON)
+    self._MockCommand(['.'], ['x\nRichards: 1.234\nDeltaBlue: 10657567\ny\n'])
+    self.assertEqual(0, self._CallMain("--filter=test"))
+    self._VerifyResults('test', 'score', [
+        {
+            'name': 'Richards',
+            'results': [1.234],
+            'stddev': ''
+        },
+        {
+            'name': 'DeltaBlue',
+            'results': [10657567.0],
+            'stddev': ''
+        },
+    ])
+    self._VerifyMock(
+        os.path.join('out', 'x64.release', 'd7'), '--flag', 'run.js')
+
+  def testFilterRegexpSkipAll(self):
+    self._WriteTestInput(V8_JSON)
+    self._MockCommand(['.'], ['x\nRichards: 1.234\nDeltaBlue: 10657567\ny\n'])
+    self.assertEqual(0, self._CallMain("--filter=NonExistingName"))
+    self._VerifyResults('test', 'score', [])
+
   def testOneRunCrashed(self):
     test_input = dict(V8_JSON)
     test_input['retry_count'] = 1
@@ -482,11 +638,12 @@ class PerfTest(unittest.TestCase):
     mock.patch('run_perf.AndroidPlatform.PreExecution').start()
     mock.patch('run_perf.AndroidPlatform.PostExecution').start()
     mock.patch('run_perf.AndroidPlatform.PreTests').start()
+    mock.patch('run_perf.find_build_directory').start()
     mock.patch(
         'run_perf.AndroidPlatform.Run',
         return_value=(Output(stdout='Richards: 1.234\nDeltaBlue: 10657567\n'),
                       NULL_OUTPUT)).start()
-    mock.patch('testrunner.local.android._Driver', autospec=True).start()
+    mock.patch('testrunner.local.android.Driver', autospec=True).start()
     mock.patch(
         'run_perf.Platform.ReadBuildConfig',
         return_value={'is_android': True}).start()
@@ -573,7 +730,7 @@ class PerfTest(unittest.TestCase):
         'results': [2.1, 2.1],
         'stddev': '',
       },
-    ]), sorted(results['traces']))
+    ], key=SORT_KEY), sorted(results['traces'], key=SORT_KEY))
 
   def testResultsProcessor(self):
     results = self._RunPerf('d8_mocked2.py', 'test2.json')
