@@ -864,6 +864,10 @@ OutgoingMessage.prototype.write = function write(chunk, encoding, callback) {
 };
 
 function onError(msg, err, callback) {
+  if (msg.destroyed) {
+    return;
+  }
+
   const triggerAsyncId = msg.socket ? msg.socket[async_id_symbol] : undefined;
   defaultTriggerAsyncIdScope(triggerAsyncId,
                              process.nextTick,
@@ -875,7 +879,7 @@ function onError(msg, err, callback) {
 
 function emitErrorNt(msg, err, callback) {
   callback(err);
-  if (typeof msg.emit === 'function' && !msg._closed) {
+  if (typeof msg.emit === 'function' && !msg.destroyed) {
     msg.emit('error', err);
   }
 }
@@ -1168,6 +1172,8 @@ OutgoingMessage.prototype._flushOutput = function _flushOutput(socket) {
   // Refs: https://github.com/nodejs/node/pull/30958
   for (let i = 0; i < outputLength; i++) {
     const { data, encoding, callback } = outputData[i];
+    // Avoid any potential ref to Buffer in new generation from old generation
+    outputData[i].data = null;
     ret = socket.write(data, encoding, callback);
   }
   socket.uncork();
