@@ -22,8 +22,6 @@
 'use strict';
 
 const {
-  ArrayPrototypeSlice,
-  ArrayPrototypeSort,
   ObjectDefineProperty,
 } = primordials;
 
@@ -118,7 +116,7 @@ function get(url, options, cb) {
 
 module.exports = {
   _connectionListener,
-  METHODS: ArrayPrototypeSort(ArrayPrototypeSlice(methods)),
+  METHODS: methods.toSorted(),
   STATUS_CODES,
   Agent: httpAgent.Agent,
   ClientRequest,
