@@ -31,14 +31,24 @@ process.removeListener('removeListener', stopListeningIfSignal);
 
 const {
   createWorkerStdio,
+  kStdioWantsMoreDataCallback,
 } = require('internal/worker/io');
 
 let workerStdio;
 function lazyWorkerStdio() {
-  if (!workerStdio) workerStdio = createWorkerStdio();
+  if (workerStdio === undefined) {
+    workerStdio = createWorkerStdio();
+    process.on('exit', flushSync);
+  }
+
   return workerStdio;
 }
 
+function flushSync() {
+  workerStdio.stdout[kStdioWantsMoreDataCallback]();
+  workerStdio.stderr[kStdioWantsMoreDataCallback]();
+}
+
 function getStdout() { return lazyWorkerStdio().stdout; }
 
 function getStderr() { return lazyWorkerStdio().stderr; }
