@@ -61,7 +61,12 @@ const advanced = {
   *parseChannelMessages(channel, readData) {
     if (readData.length === 0) return;
 
-    ArrayPrototypePush(channel[kMessageBuffer], readData);
+    if (channel[kMessageBufferSize] && channel[kMessageBuffer][0].length < 4) {
+      // Message length split into two buffers, so let's concatenate it.
+      channel[kMessageBuffer][0] = Buffer.concat([channel[kMessageBuffer][0], readData]);
+    } else {
+      ArrayPrototypePush(channel[kMessageBuffer], readData);
+    }
     channel[kMessageBufferSize] += readData.length;
 
     // Index 0 should always be present because we just pushed data into it.
