@@ -228,11 +228,6 @@ function copyPrototype(src, dest, prefix) {
   copyPrototype(original.prototype, primordials, `${name}Prototype`);
 });
 
-// Define Symbol.dispose and Symbol.asyncDispose
-// Until these are defined by the environment.
-// TODO(MoLow): Remove this polyfill once Symbol.dispose and Symbol.asyncDispose are available in V8.
-primordials.SymbolDispose ??= primordials.SymbolFor('nodejs.dispose');
-primordials.SymbolAsyncDispose ??= primordials.SymbolFor('nodejs.asyncDispose');
 
 // Create copies of intrinsic objects that require a valid `this` to call
 // static methods.
@@ -461,7 +456,7 @@ const SafePromise = makeSafe(
  * rejected). The resolved value cannot be modified from the callback.
  * Prefer using async functions when possible.
  * @param {Promise<any>} thisPromise
- * @param {() => void) | undefined | null} onFinally The callback to execute
+ * @param {(() => void) | undefined | null} onFinally The callback to execute
  *        when the Promise is settled (fulfilled or rejected).
  * @returns {Promise} A Promise for the completion of the callback.
  */
