@@ -3,32 +3,70 @@ const {
   ArrayPrototypePush,
   ArrayPrototypeSlice,
   Error,
+  FunctionPrototypeBind,
   FunctionPrototypeCall,
+  Int32Array,
   ObjectDefineProperty,
   ObjectGetOwnPropertyDescriptor,
   ObjectGetPrototypeOf,
+  ObjectKeys,
   Proxy,
   ReflectApply,
   ReflectConstruct,
   ReflectGet,
   SafeMap,
+  StringPrototypeSlice,
+  StringPrototypeStartsWith,
+  globalThis: {
+    Atomics: {
+      store: AtomicsStore,
+      wait: AtomicsWait,
+    },
+    SharedArrayBuffer,
+  },
 } = primordials;
 const {
   codes: {
     ERR_INVALID_ARG_TYPE,
     ERR_INVALID_ARG_VALUE,
+    ERR_INVALID_STATE,
   },
 } = require('internal/errors');
-const { kEmptyObject } = require('internal/util');
+const esmLoader = require('internal/modules/esm/loader');
+const { getOptionValue } = require('internal/options');
+const { fileURLToPath, toPathIfFileURL, URL } = require('internal/url');
+const {
+  emitExperimentalWarning,
+  getStructuredStack,
+  kEmptyObject,
+} = require('internal/util');
+let debug = require('internal/util/debuglog').debuglog('test_runner', (fn) => {
+  debug = fn;
+});
 const {
   validateBoolean,
   validateFunction,
   validateInteger,
   validateObject,
+  validateOneOf,
+  validateString,
 } = require('internal/validators');
 const { MockTimers } = require('internal/test_runner/mock/mock_timers');
-
+const { strictEqual, notStrictEqual } = require('assert');
+const { Module } = require('internal/modules/cjs/loader');
+const { MessageChannel } = require('worker_threads');
+const { _load, _nodeModulePaths, _resolveFilename, isBuiltin } = Module;
 function kDefaultFunction() {}
+const enableModuleMocking = getOptionValue('--experimental-test-module-mocks');
+const kMockSearchParam = 'node-test-mock';
+const kMockSuccess = 1;
+const kMockExists = 2;
+const kMockUnknownMessage = 3;
+const kWaitTimeout = 5_000;
+const kBadExportsMessage = 'Cannot create mock because named exports ' +
+  'cannot be applied to the provided default export.';
+const kSupportedFormats = ['builtin', 'commonjs', 'module'];
+let sharedModuleState;
 
 class MockFunctionContext {
   #calls;
@@ -132,10 +170,108 @@ class MockFunctionContext {
   }
 }
 
-const { nextImpl, restore, trackCall } = MockFunctionContext.prototype;
+const {
+  nextImpl,
+  restore: restoreFn,
+  trackCall,
+} = MockFunctionContext.prototype;
 delete MockFunctionContext.prototype.trackCall;
 delete MockFunctionContext.prototype.nextImpl;
 
+class MockModuleContext {
+  #restore;
+  #sharedState;
+
+  constructor({
+    baseURL,
+    cache,
+    caller,
+    defaultExport,
+    format,
+    fullPath,
+    hasDefaultExport,
+    namedExports,
+    sharedState,
+  }) {
+    const ack = new Int32Array(new SharedArrayBuffer(4));
+    const config = {
+      __proto__: null,
+      cache,
+      defaultExport,
+      hasDefaultExport,
+      namedExports,
+      caller: toPathIfFileURL(caller),
+    };
+
+    sharedState.mockMap.set(baseURL, config);
+    sharedState.mockMap.set(fullPath, config);
+
+    this.#sharedState = sharedState;
+    this.#restore = {
+      __proto__: null,
+      ack,
+      baseURL,
+      cached: fullPath in Module._cache,
+      format,
+      fullPath,
+      value: Module._cache[fullPath],
+    };
+
+    sharedState.loaderPort.postMessage({
+      __proto__: null,
+      type: 'node:test:register',
+      payload: {
+        __proto__: null,
+        ack,
+        baseURL,
+        cache,
+        exportNames: ObjectKeys(namedExports),
+        hasDefaultExport,
+        format,
+      },
+    });
+    waitForAck(ack);
+    delete Module._cache[fullPath];
+    sharedState.mockExports.set(baseURL, {
+      __proto__: null,
+      defaultExport,
+      namedExports,
+    });
+  }
+
+  restore() {
+    if (this.#restore === undefined) {
+      return;
+    }
+
+    // Delete the mock CJS cache entry. If the module was previously in the
+    // cache then restore the old value.
+    delete Module._cache[this.#restore.fullPath];
+
+    if (this.#restore.cached) {
+      Module._cache[this.#restore.fullPath] = this.#restore.value;
+    }
+
+    AtomicsStore(this.#restore.ack, 0, 0);
+    this.#sharedState.loaderPort.postMessage({
+      __proto__: null,
+      type: 'node:test:unregister',
+      payload: {
+        __proto__: null,
+        ack: this.#restore.ack,
+        baseURL: this.#restore.baseURL,
+      },
+    });
+    waitForAck(this.#restore.ack);
+
+    this.#sharedState.mockMap.delete(this.#restore.baseURL);
+    this.#sharedState.mockMap.delete(this.#restore.fullPath);
+    this.#restore = undefined;
+  }
+}
+
+const { restore: restoreModule } = MockModuleContext.prototype;
+
 class MockTracker {
   #mocks = [];
   #timers;
@@ -350,6 +486,74 @@ class MockTracker {
     });
   }
 
+  module(specifier, options = kEmptyObject) {
+    emitExperimentalWarning('Module mocking');
+    validateString(specifier, 'specifier');
+    validateObject(options, 'options');
+    debug('module mock entry, specifier = "%s", options = %o', specifier, options);
+
+    const {
+      cache = false,
+      namedExports = kEmptyObject,
+      defaultExport,
+    } = options;
+    const hasDefaultExport = 'defaultExport' in options;
+
+    validateBoolean(cache, 'options.cache');
+    validateObject(namedExports, 'options.namedExports');
+
+    const sharedState = setupSharedModuleState();
+    const mockSpecifier = StringPrototypeStartsWith(specifier, 'node:') ?
+      StringPrototypeSlice(specifier, 5) : specifier;
+
+    // Get the file that called this function. We need four stack frames:
+    // vm context -> getStructuredStack() -> this function -> actual caller.
+    const caller = getStructuredStack()[3]?.getFileName();
+    const { format, url } = sharedState.moduleLoader.resolveSync(
+      mockSpecifier, caller, null,
+    );
+    debug('module mock, url = "%s", format = "%s", caller = "%s"', url, format, caller);
+    if (format) { // Format is not yet known for ambiguous files when detection is enabled.
+      validateOneOf(format, 'format', kSupportedFormats);
+    }
+    const baseURL = URL.parse(url);
+
+    if (!baseURL) {
+      throw new ERR_INVALID_ARG_VALUE(
+        'specifier', specifier, 'cannot compute URL',
+      );
+    }
+
+    if (baseURL.searchParams.has(kMockSearchParam)) {
+      throw new ERR_INVALID_STATE(
+        `Cannot mock '${specifier}.' The module is already mocked.`,
+      );
+    }
+
+    const fullPath = StringPrototypeStartsWith(url, 'file://') ?
+      fileURLToPath(url) : null;
+    const ctx = new MockModuleContext({
+      __proto__: null,
+      baseURL: baseURL.href,
+      cache,
+      caller,
+      defaultExport,
+      format,
+      fullPath,
+      hasDefaultExport,
+      namedExports,
+      sharedState,
+      specifier: mockSpecifier,
+    });
+
+    ArrayPrototypePush(this.#mocks, {
+      __proto__: null,
+      ctx,
+      restore: restoreModule,
+    });
+    return ctx;
+  }
+
   /**
    * Resets the mock tracker, restoring all mocks and clearing timers.
    */
@@ -364,7 +568,9 @@ class MockTracker {
    */
   restoreAll() {
     for (let i = 0; i < this.#mocks.length; i++) {
-      FunctionPrototypeCall(restore, this.#mocks[i]);
+      const { ctx, restore } = this.#mocks[i];
+
+      FunctionPrototypeCall(restore, ctx);
     }
   }
 
@@ -430,11 +636,101 @@ class MockTracker {
       },
     });
 
-    ArrayPrototypePush(this.#mocks, ctx);
+    ArrayPrototypePush(this.#mocks, {
+      __proto__: null,
+      ctx,
+      restore: restoreFn,
+    });
     return mock;
   }
 }
 
+function setupSharedModuleState() {
+  if (sharedModuleState === undefined) {
+    const { mock } = require('test');
+    const mockExports = new SafeMap();
+    const { port1, port2 } = new MessageChannel();
+    const moduleLoader = esmLoader.getOrInitializeCascadedLoader();
+
+    moduleLoader.register(
+      'internal/test_runner/mock/loader',
+      'node:',
+      { __proto__: null, port: port2 },
+      [port2],
+      true,
+    );
+
+    sharedModuleState = {
+      __proto__: null,
+      loaderPort: port1,
+      mockExports,
+      mockMap: new SafeMap(),
+      moduleLoader,
+    };
+    mock._mockExports = mockExports;
+    Module._load = FunctionPrototypeBind(cjsMockModuleLoad, sharedModuleState);
+  }
+
+  return sharedModuleState;
+}
+
+function cjsMockModuleLoad(request, parent, isMain) {
+  let resolved;
+
+  if (isBuiltin(request)) {
+    resolved = ensureNodeScheme(request);
+  } else {
+    resolved = _resolveFilename(request, parent, isMain);
+  }
+
+  const config = this.mockMap.get(resolved);
+  if (config === undefined) {
+    return _load(request, parent, isMain);
+  }
+
+  const {
+    cache,
+    caller,
+    defaultExport,
+    hasDefaultExport,
+    namedExports,
+  } = config;
+
+  if (cache && Module._cache[resolved]) {
+    // The CJS cache entry is deleted when the mock is configured. If it has
+    // been repopulated, return the exports from that entry.
+    return Module._cache[resolved].exports;
+  }
+
+  // eslint-disable-next-line node-core/set-proto-to-null-in-object
+  const modExports = hasDefaultExport ? defaultExport : {};
+  const exportNames = ObjectKeys(namedExports);
+
+  if ((typeof modExports !== 'object' || modExports === null) &&
+      exportNames.length > 0) {
+    // eslint-disable-next-line no-restricted-syntax
+    throw new Error(kBadExportsMessage);
+  }
+
+  for (let i = 0; i < exportNames.length; ++i) {
+    const name = exportNames[i];
+    const descriptor = ObjectGetOwnPropertyDescriptor(namedExports, name);
+    ObjectDefineProperty(modExports, name, descriptor);
+  }
+
+  if (cache) {
+    const entry = new Module(resolved, caller);
+
+    entry.exports = modExports;
+    entry.filename = resolved;
+    entry.loaded = true;
+    entry.paths = _nodeModulePaths(entry.path);
+    Module._cache[resolved] = entry;
+  }
+
+  return modExports;
+}
+
 function validateStringOrSymbol(value, name) {
   if (typeof value !== 'string' && typeof value !== 'symbol') {
     throw new ERR_INVALID_ARG_TYPE(name, ['string', 'symbol'], value);
@@ -466,4 +762,31 @@ function findMethodOnPrototypeChain(instance, methodName) {
   return descriptor;
 }
 
-module.exports = { MockTracker };
+function waitForAck(buf) {
+  const result = AtomicsWait(buf, 0, 0, kWaitTimeout);
+
+  notStrictEqual(result, 'timed-out', 'test mocking synchronization failed');
+  strictEqual(buf[0], kMockSuccess);
+}
+
+function ensureNodeScheme(specifier) {
+  if (!StringPrototypeStartsWith(specifier, 'node:')) {
+    return `node:${specifier}`;
+  }
+
+  return specifier;
+}
+
+if (!enableModuleMocking) {
+  delete MockTracker.prototype.module;
+}
+
+module.exports = {
+  ensureNodeScheme,
+  kBadExportsMessage,
+  kMockSearchParam,
+  kMockSuccess,
+  kMockExists,
+  kMockUnknownMessage,
+  MockTracker,
+};
