@@ -77,19 +77,15 @@ const {
   MathTrunc,
   NumberIsFinite,
   NumberMIN_SAFE_INTEGER,
-  ObjectCreate,
   ReflectApply,
   Symbol,
 } = primordials;
 
+const binding = internalBinding('timers');
 const {
-  scheduleTimer,
-  toggleTimerRef,
-  getLibuvNow,
   immediateInfo,
   timeoutInfo,
-  toggleImmediateRef,
-} = internalBinding('timers');
+} = binding;
 
 const {
   getDefaultTriggerAsyncId,
@@ -154,7 +150,12 @@ const timerListQueue = new PriorityQueue(compareTimersLists, setPosition);
 //
 // - key = time in milliseconds
 // - value = linked list
-const timerListMap = ObjectCreate(null);
+const timerListMap = { __proto__: null };
+
+// This stores all the known timer async ids to allow users to clearTimeout and
+// clearInterval using those ids, to match the spec and the rest of the web
+// platform.
+const knownTimersById = { __proto__: null };
 
 function initAsyncResource(resource, type) {
   const asyncId = resource[async_id_symbol] = newAsyncId();
@@ -307,13 +308,17 @@ class ImmediateList {
 const immediateQueue = new ImmediateList();
 
 function incRefCount() {
-  if (timeoutInfo[0]++ === 0)
-    toggleTimerRef(true);
+  if (timeoutInfo[0]++ === 0) {
+    // We need to use the binding as the receiver for fast API calls.
+    binding.toggleTimerRef(true);
+  }
 }
 
 function decRefCount() {
-  if (--timeoutInfo[0] === 0)
-    toggleTimerRef(false);
+  if (--timeoutInfo[0] === 0) {
+    // We need to use the binding as the receiver for fast API calls.
+    binding.toggleTimerRef(false);
+  }
 }
 
 // Schedule or re-schedule a timer.
@@ -357,7 +362,8 @@ function insertGuarded(item, refed, start) {
   item[kRefed] = refed;
 }
 
-function insert(item, msecs, start = getLibuvNow()) {
+// We need to use the binding as the receiver for fast API calls.
+function insert(item, msecs, start = binding.getLibuvNow()) {
   // Truncate so that accuracy of sub-millisecond timers is not assumed.
   msecs = MathTrunc(msecs);
   item._idleStart = start;
@@ -371,7 +377,8 @@ function insert(item, msecs, start = getLibuvNow()) {
     timerListQueue.insert(list);
 
     if (nextExpiry > expiry) {
-      scheduleTimer(msecs);
+      // We need to use the binding as the receiver for fast API calls.
+      binding.scheduleTimer(msecs);
       nextExpiry = expiry;
     }
   }
@@ -548,6 +555,9 @@ function getTimerCallbacks(runNextTicks) {
         if (!timer._destroyed) {
           timer._destroyed = true;
 
+          if (timer[kHasPrimitive])
+            delete knownTimersById[asyncId];
+
           if (timer[kRefed])
             timeoutInfo[0]--;
 
@@ -560,8 +570,10 @@ function getTimerCallbacks(runNextTicks) {
       emitBefore(asyncId, timer[trigger_async_id_symbol], timer);
 
       let start;
-      if (timer._repeat)
-        start = getLibuvNow();
+      if (timer._repeat) {
+        // We need to use the binding as the receiver for fast API calls.
+        start = binding.getLibuvNow();
+      }
 
       try {
         const args = timer._timerArgs;
@@ -576,6 +588,9 @@ function getTimerCallbacks(runNextTicks) {
         } else if (!timer._idleNext && !timer._idlePrev && !timer._destroyed) {
           timer._destroyed = true;
 
+          if (timer[kHasPrimitive])
+            delete knownTimersById[asyncId];
+
           if (timer[kRefed])
             timeoutInfo[0]--;
 
@@ -628,8 +643,11 @@ class Immediate {
   ref() {
     if (this[kRefed] === false) {
       this[kRefed] = true;
-      if (immediateInfo[kRefCount]++ === 0)
-        toggleImmediateRef(true);
+
+      if (immediateInfo[kRefCount]++ === 0) {
+        // We need to use the binding as the receiver for fast API calls.
+        binding.toggleImmediateRef(true);
+      }
     }
     return this;
   }
@@ -637,8 +655,10 @@ class Immediate {
   unref() {
     if (this[kRefed] === true) {
       this[kRefed] = false;
-      if (--immediateInfo[kRefCount] === 0)
-        toggleImmediateRef(false);
+      if (--immediateInfo[kRefCount] === 0) {
+        // We need to use the binding as the receiver for fast API calls.
+        binding.toggleImmediateRef(false);
+      }
     }
     return this;
   }
@@ -674,4 +694,5 @@ module.exports = {
   timerListQueue,
   decRefCount,
   incRefCount,
+  knownTimersById,
 };
