@@ -3,8 +3,10 @@
 const {
   ArrayIsArray,
   ArrayPrototypeForEach,
+  Boolean,
   SafeMap,
   SafeSet,
+  SafeWeakMap,
   StringPrototypeStartsWith,
 } = primordials;
 
@@ -13,10 +15,11 @@ const { kEmptyObject } = require('internal/util');
 const { TIMEOUT_MAX } = require('internal/timers');
 
 const EventEmitter = require('events');
+const { addAbortListener } = require('internal/events/abort_listener');
 const { watch } = require('fs');
-const { fileURLToPath } = require('url');
+const { fileURLToPath } = require('internal/url');
 const { resolve, dirname } = require('path');
-const { setTimeout } = require('timers');
+const { setTimeout, clearTimeout } = require('timers');
 
 const supportsRecursiveWatching = process.platform === 'win32' ||
   process.platform === 'darwin';
@@ -24,24 +27,28 @@ const supportsRecursiveWatching = process.platform === 'win32' ||
 class FilesWatcher extends EventEmitter {
   #watchers = new SafeMap();
   #filteredFiles = new SafeSet();
-  #debouncing = new SafeSet();
-  #depencencyOwners = new SafeMap();
+  #dependencyOwners = new SafeMap();
   #ownerDependencies = new SafeMap();
+  #debounceOwners = new SafeSet();
+  #debounceTimer;
   #debounce;
   #mode;
   #signal;
+  #passthroughIPC = false;
+  #ipcHandlers = new SafeWeakMap();
 
   constructor({ debounce = 200, mode = 'filter', signal } = kEmptyObject) {
-    super();
+    super({ __proto__: null, captureRejections: true });
 
     validateNumber(debounce, 'options.debounce', 0, TIMEOUT_MAX);
     validateOneOf(mode, 'options.mode', ['filter', 'all']);
     this.#debounce = debounce;
     this.#mode = mode;
     this.#signal = signal;
+    this.#passthroughIPC = Boolean(process.send);
 
     if (signal) {
-      EventEmitter.addAbortListener(signal, () => this.clear());
+      addAbortListener(signal, () => this.clear());
     }
   }
 
@@ -73,18 +80,21 @@ class FilesWatcher extends EventEmitter {
     watcher.handle.close();
   }
 
-  #onChange(trigger) {
-    if (this.#debouncing.has(trigger)) {
-      return;
-    }
+  #onChange(trigger, eventType) {
     if (this.#mode === 'filter' && !this.#filteredFiles.has(trigger)) {
       return;
     }
-    this.#debouncing.add(trigger);
-    const owners = this.#depencencyOwners.get(trigger);
-    setTimeout(() => {
-      this.#debouncing.delete(trigger);
-      this.emit('changed', { owners });
+    const owners = this.#dependencyOwners.get(trigger);
+    if (owners) {
+      for (const owner of owners) {
+        this.#debounceOwners.add(owner);
+      }
+    }
+    clearTimeout(this.#debounceTimer);
+    this.#debounceTimer = setTimeout(() => {
+      this.#debounceTimer = null;
+      this.emit('changed', { owners: this.#debounceOwners, eventType });
+      this.#debounceOwners.clear();
     }, this.#debounce).unref();
   }
 
@@ -97,8 +107,11 @@ class FilesWatcher extends EventEmitter {
       return;
     }
     const watcher = watch(path, { recursive, signal: this.#signal });
-    watcher.on('change', (eventType, fileName) => this
-      .#onChange(recursive ? resolve(path, fileName) : path));
+    watcher.on('change', (eventType, fileName) => {
+      // `fileName` can be `null` if it cannot be determined. See
+      // https://github.com/nodejs/node/pull/49891#issuecomment-1744673430.
+      this.#onChange(recursive ? resolve(path, fileName ?? '') : path, eventType);
+    });
     this.#watchers.set(path, { handle: watcher, recursive });
     if (recursive) {
       this.#removeWatchedChildren(path);
@@ -116,15 +129,39 @@ class FilesWatcher extends EventEmitter {
     }
     this.#filteredFiles.add(file);
     if (owner) {
-      const owners = this.#depencencyOwners.get(file) ?? new SafeSet();
+      const owners = this.#dependencyOwners.get(file) ?? new SafeSet();
       const dependencies = this.#ownerDependencies.get(file) ?? new SafeSet();
       owners.add(owner);
       dependencies.add(file);
-      this.#depencencyOwners.set(file, owners);
+      this.#dependencyOwners.set(file, owners);
       this.#ownerDependencies.set(owner, dependencies);
     }
   }
+
+
+  #setupIPC(child) {
+    const handlers = {
+      __proto__: null,
+      parentToChild: (message) => child.send(message),
+      childToParent: (message) => process.send(message),
+    };
+    this.#ipcHandlers.set(child, handlers);
+    process.on('message', handlers.parentToChild);
+    child.on('message', handlers.childToParent);
+  }
+
+  destroyIPC(child) {
+    const handlers = this.#ipcHandlers.get(child);
+    if (this.#passthroughIPC && handlers !== undefined) {
+      process.off('message', handlers.parentToChild);
+      child.off('message', handlers.childToParent);
+    }
+  }
+
   watchChildProcessModules(child, key = null) {
+    if (this.#passthroughIPC) {
+      this.#setupIPC(child);
+    }
     if (this.#mode !== 'filter') {
       return;
     }
@@ -145,10 +182,10 @@ class FilesWatcher extends EventEmitter {
     owners.forEach((owner) => {
       this.#ownerDependencies.get(owner)?.forEach((dependency) => {
         this.#filteredFiles.delete(dependency);
-        this.#depencencyOwners.delete(dependency);
+        this.#dependencyOwners.delete(dependency);
       });
       this.#filteredFiles.delete(owner);
-      this.#depencencyOwners.delete(owner);
+      this.#dependencyOwners.delete(owner);
       this.#ownerDependencies.delete(owner);
     });
   }
@@ -159,7 +196,7 @@ class FilesWatcher extends EventEmitter {
     this.#watchers.forEach(this.#unwatch);
     this.#watchers.clear();
     this.#filteredFiles.clear();
-    this.#depencencyOwners.clear();
+    this.#dependencyOwners.clear();
     this.#ownerDependencies.clear();
   }
 }
