@@ -22,6 +22,7 @@
 'use strict';
 
 const {
+  ArrayPrototypeIncludes,
   ArrayPrototypeJoin,
   ArrayPrototypeSlice,
   FunctionPrototypeBind,
@@ -34,6 +35,7 @@ const {
   StringPrototypeSlice,
   StringPrototypeSplit,
   StringPrototypeToLowerCase,
+  StringPrototypeToUpperCase,
 } = primordials;
 
 const {
@@ -70,6 +72,17 @@ function isPosixPathSeparator(code) {
   return code === CHAR_FORWARD_SLASH;
 }
 
+const WINDOWS_RESERVED_NAMES = [
+  'CON', 'PRN', 'AUX', 'NUL',
+  'COM1', 'COM2', 'COM3', 'COM4', 'COM5', 'COM6', 'COM7', 'COM8', 'COM9',
+  'LPT1', 'LPT2', 'LPT3', 'LPT4', 'LPT5', 'LPT6', 'LPT7', 'LPT8', 'LPT9',
+];
+
+function isWindowsReservedName(path, colonIndex) {
+  const devicePart = StringPrototypeToUpperCase(StringPrototypeSlice(path, 0, colonIndex));
+  return ArrayPrototypeIncludes(WINDOWS_RESERVED_NAMES, devicePart);
+}
+
 function isWindowsDeviceRoot(code) {
   return (code >= CHAR_UPPERCASE_A && code <= CHAR_UPPERCASE_Z) ||
          (code >= CHAR_LOWERCASE_A && code <= CHAR_LOWERCASE_Z);
@@ -400,16 +413,21 @@ const win32 = {
       } else {
         rootEnd = 1;
       }
-    } else if (isWindowsDeviceRoot(code) &&
-               StringPrototypeCharCodeAt(path, 1) === CHAR_COLON) {
-      // Possible device root
-      device = StringPrototypeSlice(path, 0, 2);
-      rootEnd = 2;
-      if (len > 2 && isPathSeparator(StringPrototypeCharCodeAt(path, 2))) {
-        // Treat separator following drive name as an absolute path
-        // indicator
-        isAbsolute = true;
-        rootEnd = 3;
+    } else {
+      const colonIndex = StringPrototypeIndexOf(path, ':');
+      if (colonIndex > 0) {
+        if (isWindowsDeviceRoot(code) && colonIndex === 1) {
+          device = StringPrototypeSlice(path, 0, 2);
+          rootEnd = 2;
+          if (len > 2 && isPathSeparator(StringPrototypeCharCodeAt(path, 2))) {
+            isAbsolute = true;
+            rootEnd = 3;
+          }
+        } else if (isWindowsReservedName(path, colonIndex)) {
+          device = StringPrototypeSlice(path, 0, colonIndex + 1);
+          rootEnd = colonIndex + 1;
+
+        }
       }
     }
 
@@ -433,12 +451,17 @@ const win32 = {
         return `.\\${tail}`;
       }
       let index = StringPrototypeIndexOf(path, ':');
+
       do {
         if (index === len - 1 || isPathSeparator(StringPrototypeCharCodeAt(path, index + 1))) {
           return `.\\${tail}`;
         }
       } while ((index = StringPrototypeIndexOf(path, ':', index + 1)) !== -1);
     }
+    const colonIndex = StringPrototypeIndexOf(path, ':');
+    if (isWindowsReservedName(path, colonIndex)) {
+      return `.\\${device ?? ''}${tail}`;
+    }
     if (device === undefined) {
       return isAbsolute ? `\\${tail}` : tail;
     }
