@@ -23,17 +23,21 @@
 
 const {
   ArrayBuffer,
-  Error,
-  MathMax,
-  NumberIsFinite,
+  ArrayPrototypeForEach,
+  ArrayPrototypeMap,
+  ArrayPrototypePush,
+  FunctionPrototypeBind,
+  MathMaxApply,
   NumberIsNaN,
   ObjectDefineProperties,
   ObjectDefineProperty,
   ObjectFreeze,
-  ObjectGetPrototypeOf,
   ObjectKeys,
   ObjectSetPrototypeOf,
+  ReflectApply,
+  StringPrototypeStartsWith,
   Symbol,
+  TypedArrayPrototypeFill,
   Uint32Array,
 } = primordials;
 
@@ -43,27 +47,35 @@ const {
     ERR_BUFFER_TOO_LARGE,
     ERR_INVALID_ARG_TYPE,
     ERR_OUT_OF_RANGE,
-    ERR_ZLIB_INITIALIZATION_FAILED,
   },
-  hideStackFrames
+  genericNodeError,
 } = require('internal/errors');
-const Transform = require('_stream_transform');
+const { Transform, finished } = require('stream');
 const {
-  deprecate
+  deprecate,
 } = require('internal/util');
 const {
   isArrayBufferView,
-  isAnyArrayBuffer
+  isAnyArrayBuffer,
+  isUint8Array,
 } = require('internal/util/types');
 const binding = internalBinding('zlib');
+const { crc32: crc32Native } = binding;
 const assert = require('internal/assert');
 const {
   Buffer,
-  kMaxLength
+  kMaxLength,
 } = require('buffer');
 const { owner_symbol } = require('internal/async_hooks').symbols;
+const {
+  checkRangesOrGetDefault,
+  validateFunction,
+  validateUint32,
+  validateFiniteNumber,
+} = require('internal/validators');
 
 const kFlushFlag = Symbol('kFlushFlag');
+const kError = Symbol('kError');
 
 const constants = internalBinding('constants').zlib;
 const {
@@ -78,7 +90,7 @@ const {
   BROTLI_DECODE, BROTLI_ENCODE,
   // Brotli operations (~flush levels)
   BROTLI_OPERATION_PROCESS, BROTLI_OPERATION_FLUSH,
-  BROTLI_OPERATION_FINISH
+  BROTLI_OPERATION_FINISH, BROTLI_OPERATION_EMIT_METADATA,
 } = constants;
 
 // Translation table for return codes.
@@ -91,7 +103,7 @@ const codes = {
   Z_DATA_ERROR: constants.Z_DATA_ERROR,
   Z_MEM_ERROR: constants.Z_MEM_ERROR,
   Z_BUF_ERROR: constants.Z_BUF_ERROR,
-  Z_VERSION_ERROR: constants.Z_VERSION_ERROR
+  Z_VERSION_ERROR: constants.Z_VERSION_ERROR,
 };
 
 for (const ckey of ObjectKeys(codes)) {
@@ -99,12 +111,10 @@ for (const ckey of ObjectKeys(codes)) {
 }
 
 function zlibBuffer(engine, buffer, callback) {
-  if (typeof callback !== 'function')
-    throw new ERR_INVALID_ARG_TYPE('callback', 'function', callback);
-  // Streams do not support non-Buffer ArrayBufferViews yet. Convert it to a
+  validateFunction(callback, 'callback');
+  // Streams do not support non-Uint8Array ArrayBufferViews yet. Convert it to a
   // Buffer without copying.
-  if (isArrayBufferView(buffer) &&
-      ObjectGetPrototypeOf(buffer) !== Buffer.prototype) {
+  if (isArrayBufferView(buffer) && !isUint8Array(buffer)) {
     buffer = Buffer.from(buffer.buffer, buffer.byteOffset, buffer.byteLength);
   } else if (isAnyArrayBuffer(buffer)) {
     buffer = Buffer.from(buffer);
@@ -122,7 +132,7 @@ function zlibBufferOnData(chunk) {
   if (!this.buffers)
     this.buffers = [chunk];
   else
-    this.buffers.push(chunk);
+    ArrayPrototypePush(this.buffers, chunk);
   this.nread += chunk.length;
   if (this.nread > this._maxOutputLength) {
     this.close();
@@ -161,7 +171,7 @@ function zlibBufferSync(engine, buffer) {
       throw new ERR_INVALID_ARG_TYPE(
         'buffer',
         ['string', 'Buffer', 'TypedArray', 'DataView', 'ArrayBuffer'],
-        buffer
+        buffer,
       );
     }
   }
@@ -175,59 +185,20 @@ function zlibOnError(message, errno, code) {
   const self = this[owner_symbol];
   // There is no way to cleanly recover.
   // Continuing only obscures problems.
-  _close(self);
-  self._hadError = true;
 
-  // eslint-disable-next-line no-restricted-syntax
-  const error = new Error(message);
+  const error = genericNodeError(message, { errno, code });
   error.errno = errno;
   error.code = code;
-  self.emit('error', error);
+  self.destroy(error);
+  self[kError] = error;
 }
 
-// 1. Returns false for undefined and NaN
-// 2. Returns true for finite numbers
-// 3. Throws ERR_INVALID_ARG_TYPE for non-numbers
-// 4. Throws ERR_OUT_OF_RANGE for infinite numbers
-const checkFiniteNumber = hideStackFrames((number, name) => {
-  // Common case
-  if (number === undefined) {
-    return false;
-  }
-
-  if (NumberIsFinite(number)) {
-    return true; // Is a valid number
-  }
-
-  if (NumberIsNaN(number)) {
-    return false;
-  }
-
-  // Other non-numbers
-  if (typeof number !== 'number') {
-    throw new ERR_INVALID_ARG_TYPE(name, 'number', number);
-  }
-
-  // Infinite numbers
-  throw new ERR_OUT_OF_RANGE(name, 'a finite number', number);
-});
-
-// 1. Returns def for number when it's undefined or NaN
-// 2. Returns number for finite numbers >= lower and <= upper
-// 3. Throws ERR_INVALID_ARG_TYPE for non-numbers
-// 4. Throws ERR_OUT_OF_RANGE for infinite numbers or numbers > upper or < lower
-const checkRangesOrGetDefault = hideStackFrames(
-  (number, name, lower, upper, def) => {
-    if (!checkFiniteNumber(number, name)) {
-      return def;
-    }
-    if (number < lower || number > upper) {
-      throw new ERR_OUT_OF_RANGE(name,
-                                 `>= ${lower} and <= ${upper}`, number);
-    }
-    return number;
-  }
-);
+const FLUSH_BOUND = [
+  [ Z_NO_FLUSH, Z_BLOCK ],
+  [ BROTLI_OPERATION_PROCESS, BROTLI_OPERATION_EMIT_METADATA ],
+];
+const FLUSH_BOUND_IDX_NORMAL = 0;
+const FLUSH_BOUND_IDX_BROTLI = 1;
 
 // The base class for all Zlib-style streams.
 function ZlibBase(opts, mode, handle, { flush, finishFlush, fullFlush }) {
@@ -238,9 +209,16 @@ function ZlibBase(opts, mode, handle, { flush, finishFlush, fullFlush }) {
   assert(typeof mode === 'number');
   assert(mode >= DEFLATE && mode <= BROTLI_ENCODE);
 
+  let flushBoundIdx;
+  if (mode !== BROTLI_ENCODE && mode !== BROTLI_DECODE) {
+    flushBoundIdx = FLUSH_BOUND_IDX_NORMAL;
+  } else {
+    flushBoundIdx = FLUSH_BOUND_IDX_BROTLI;
+  }
+
   if (opts) {
     chunkSize = opts.chunkSize;
-    if (!checkFiniteNumber(chunkSize, 'options.chunkSize')) {
+    if (!validateFiniteNumber(chunkSize, 'options.chunkSize')) {
       chunkSize = Z_DEFAULT_CHUNK;
     } else if (chunkSize < Z_MIN_CHUNK) {
       throw new ERR_OUT_OF_RANGE('options.chunkSize',
@@ -249,11 +227,12 @@ function ZlibBase(opts, mode, handle, { flush, finishFlush, fullFlush }) {
 
     flush = checkRangesOrGetDefault(
       opts.flush, 'options.flush',
-      Z_NO_FLUSH, Z_BLOCK, flush);
+      FLUSH_BOUND[flushBoundIdx][0], FLUSH_BOUND[flushBoundIdx][1], flush);
 
     finishFlush = checkRangesOrGetDefault(
       opts.finishFlush, 'options.finishFlush',
-      Z_NO_FLUSH, Z_BLOCK, finishFlush);
+      FLUSH_BOUND[flushBoundIdx][0], FLUSH_BOUND[flushBoundIdx][1],
+      finishFlush);
 
     maxOutputLength = checkRangesOrGetDefault(
       opts.maxOutputLength, 'options.maxOutputLength',
@@ -267,8 +246,8 @@ function ZlibBase(opts, mode, handle, { flush, finishFlush, fullFlush }) {
     }
   }
 
-  Transform.call(this, opts);
-  this._hadError = false;
+  ReflectApply(Transform, this, [{ autoDestroy: true, ...opts }]);
+  this[kError] = null;
   this.bytesWritten = 0;
   this._handle = handle;
   handle[owner_symbol] = this;
@@ -281,7 +260,6 @@ function ZlibBase(opts, mode, handle, { flush, finishFlush, fullFlush }) {
   this._defaultFlushFlag = flush;
   this._finishFlushFlag = finishFlush;
   this._defaultFullFlushFlag = fullFlush;
-  this.once('end', _close.bind(null, this));
   this._info = opts && opts.info;
   this._maxOutputLength = maxOutputLength;
 }
@@ -289,11 +267,12 @@ ObjectSetPrototypeOf(ZlibBase.prototype, Transform.prototype);
 ObjectSetPrototypeOf(ZlibBase, Transform);
 
 ObjectDefineProperty(ZlibBase.prototype, '_closed', {
+  __proto__: null,
   configurable: true,
   enumerable: true,
   get() {
     return !this._handle;
-  }
+  },
 });
 
 // `bytesRead` made sense as a name when looking from the zlib engine's
@@ -301,6 +280,7 @@ ObjectDefineProperty(ZlibBase.prototype, '_closed', {
 // that have this concept, where it stands for the number of bytes read
 // *from* the stream (that is, net.Socket/tls.Socket & file system streams).
 ObjectDefineProperty(ZlibBase.prototype, 'bytesRead', {
+  __proto__: null,
   configurable: true,
   enumerable: true,
   get: deprecate(function() {
@@ -310,7 +290,7 @@ ObjectDefineProperty(ZlibBase.prototype, 'bytesRead', {
   set: deprecate(function(value) {
     this.bytesWritten = value;
   }, 'Setting zlib.bytesRead is deprecated. ' +
-     'This feature will be removed in the future.', 'DEP0108')
+     'This feature will be removed in the future.', 'DEP0108'),
 });
 
 ZlibBase.prototype.reset = function() {
@@ -325,6 +305,11 @@ ZlibBase.prototype._flush = function(callback) {
   this._transform(Buffer.alloc(0), '', callback);
 };
 
+// Force Transform compat behavior.
+ZlibBase.prototype._final = function(callback) {
+  callback();
+};
+
 // If a flush is scheduled while another flush is still pending, a way to figure
 // out which one is the "stronger" flush is needed.
 // This is currently only used to figure out which flush flag to use for the
@@ -333,11 +318,10 @@ ZlibBase.prototype._flush = function(callback) {
 // Z_NO_FLUSH (< Z_TREES) < Z_BLOCK < Z_PARTIAL_FLUSH <
 //     Z_SYNC_FLUSH < Z_FULL_FLUSH < Z_FINISH
 const flushiness = [];
-let i = 0;
 const kFlushFlagList = [Z_NO_FLUSH, Z_BLOCK, Z_PARTIAL_FLUSH,
                         Z_SYNC_FLUSH, Z_FULL_FLUSH, Z_FINISH];
-for (const flushFlag of kFlushFlagList) {
-  flushiness[flushFlag] = i++;
+for (let i = 0; i < kFlushFlagList.length; i++) {
+  flushiness[kFlushFlagList[i]] = i;
 }
 
 function maxFlush(a, b) {
@@ -374,7 +358,7 @@ ZlibBase.prototype.flush = function(kind, callback) {
 };
 
 ZlibBase.prototype.close = function(callback) {
-  _close(this, callback);
+  if (callback) finished(this, callback);
   this.destroy();
 };
 
@@ -437,6 +421,8 @@ function processChunkSync(self, chunk, flushFlag) {
                      availOutBefore); // out_len
     if (error)
       throw error;
+    else if (self[kError])
+      throw self[kError];
 
     availOutAfter = state[0];
     availInAfter = state[1];
@@ -451,7 +437,7 @@ function processChunkSync(self, chunk, flushFlag) {
       if (!buffers)
         buffers = [out];
       else
-        buffers.push(out);
+        ArrayPrototypePush(buffers, out);
       nread += out.byteLength;
 
       if (nread > self._maxOutputLength) {
@@ -519,7 +505,7 @@ function processCallback() {
   const self = this[owner_symbol];
   const state = self._writeState;
 
-  if (self._hadError || self.destroyed) {
+  if (self.destroyed) {
     this.buffer = null;
     this.cb();
     return;
@@ -532,10 +518,11 @@ function processCallback() {
   self.bytesWritten += inDelta;
 
   const have = handle.availOutBefore - availOutAfter;
+  let streamBufferIsFull = false;
   if (have > 0) {
     const out = self._outBuffer.slice(self._outOffset, self._outOffset + have);
     self._outOffset += have;
-    self.push(out);
+    streamBufferIsFull = !self.push(out);
   } else {
     assert(have === 0, 'have should not go down');
   }
@@ -560,13 +547,29 @@ function processCallback() {
     handle.inOff += inDelta;
     handle.availInBefore = availInAfter;
 
-    this.write(handle.flushFlag,
-               this.buffer, // in
-               handle.inOff, // in_off
-               handle.availInBefore, // in_len
-               self._outBuffer, // out
-               self._outOffset, // out_off
-               self._chunkSize); // out_len
+
+    if (!streamBufferIsFull) {
+      this.write(handle.flushFlag,
+                 this.buffer, // in
+                 handle.inOff, // in_off
+                 handle.availInBefore, // in_len
+                 self._outBuffer, // out
+                 self._outOffset, // out_off
+                 self._chunkSize); // out_len
+    } else {
+      const oldRead = self._read;
+      self._read = (n) => {
+        self._read = oldRead;
+        this.write(handle.flushFlag,
+                   this.buffer, // in
+                   handle.inOff, // in_off
+                   handle.availInBefore, // in_len
+                   self._outBuffer, // out
+                   self._outOffset, // out_off
+                   self._chunkSize); // out_len
+        self._read(n);
+      };
+    }
     return;
   }
 
@@ -585,10 +588,7 @@ function processCallback() {
   this.cb();
 }
 
-function _close(engine, callback) {
-  if (callback)
-    process.nextTick(callback);
-
+function _close(engine) {
   // Caller may invoke .close after a zlib error (which will null _handle).
   if (!engine._handle)
     return;
@@ -600,7 +600,7 @@ function _close(engine, callback) {
 const zlibDefaultOpts = {
   flush: Z_NO_FLUSH,
   finishFlush: Z_FINISH,
-  fullFlush: Z_FULL_FLUSH
+  fullFlush: Z_FULL_FLUSH,
 };
 // Base class for all streams actually backed by zlib and using zlib-specific
 // parameters.
@@ -648,7 +648,7 @@ function Zlib(opts, mode) {
         throw new ERR_INVALID_ARG_TYPE(
           'options.dictionary',
           ['Buffer', 'TypedArray', 'DataView', 'ArrayBuffer'],
-          dictionary
+          dictionary,
         );
       }
     }
@@ -667,7 +667,7 @@ function Zlib(opts, mode) {
               processCallback,
               dictionary);
 
-  ZlibBase.call(this, opts, mode, handle, zlibDefaultOpts);
+  ReflectApply(ZlibBase, this, [opts, mode, handle, zlibDefaultOpts]);
 
   this._level = level;
   this._strategy = strategy;
@@ -682,7 +682,7 @@ ObjectSetPrototypeOf(Zlib, ZlibBase);
 function paramsAfterFlushCallback(level, strategy, callback) {
   assert(this._handle, 'zlib binding closed');
   this._handle.params(level, strategy);
-  if (!this._hadError) {
+  if (!this.destroyed) {
     this._level = level;
     this._strategy = strategy;
     if (callback) callback();
@@ -695,7 +695,8 @@ Zlib.prototype.params = function params(level, strategy, callback) {
 
   if (this._level !== level || this._strategy !== strategy) {
     this.flush(Z_SYNC_FLUSH,
-               paramsAfterFlushCallback.bind(this, level, strategy, callback));
+               FunctionPrototypeBind(paramsAfterFlushCallback, this,
+                                     level, strategy, callback));
   } else {
     process.nextTick(callback);
   }
@@ -706,7 +707,7 @@ Zlib.prototype.params = function params(level, strategy, callback) {
 function Deflate(opts) {
   if (!(this instanceof Deflate))
     return new Deflate(opts);
-  Zlib.call(this, opts, DEFLATE);
+  ReflectApply(Zlib, this, [opts, DEFLATE]);
 }
 ObjectSetPrototypeOf(Deflate.prototype, Zlib.prototype);
 ObjectSetPrototypeOf(Deflate, Zlib);
@@ -714,7 +715,7 @@ ObjectSetPrototypeOf(Deflate, Zlib);
 function Inflate(opts) {
   if (!(this instanceof Inflate))
     return new Inflate(opts);
-  Zlib.call(this, opts, INFLATE);
+  ReflectApply(Zlib, this, [opts, INFLATE]);
 }
 ObjectSetPrototypeOf(Inflate.prototype, Zlib.prototype);
 ObjectSetPrototypeOf(Inflate, Zlib);
@@ -722,7 +723,7 @@ ObjectSetPrototypeOf(Inflate, Zlib);
 function Gzip(opts) {
   if (!(this instanceof Gzip))
     return new Gzip(opts);
-  Zlib.call(this, opts, GZIP);
+  ReflectApply(Zlib, this, [opts, GZIP]);
 }
 ObjectSetPrototypeOf(Gzip.prototype, Zlib.prototype);
 ObjectSetPrototypeOf(Gzip, Zlib);
@@ -730,7 +731,7 @@ ObjectSetPrototypeOf(Gzip, Zlib);
 function Gunzip(opts) {
   if (!(this instanceof Gunzip))
     return new Gunzip(opts);
-  Zlib.call(this, opts, GUNZIP);
+  ReflectApply(Zlib, this, [opts, GUNZIP]);
 }
 ObjectSetPrototypeOf(Gunzip.prototype, Zlib.prototype);
 ObjectSetPrototypeOf(Gunzip, Zlib);
@@ -739,7 +740,7 @@ function DeflateRaw(opts) {
   if (opts && opts.windowBits === 8) opts.windowBits = 9;
   if (!(this instanceof DeflateRaw))
     return new DeflateRaw(opts);
-  Zlib.call(this, opts, DEFLATERAW);
+  ReflectApply(Zlib, this, [opts, DEFLATERAW]);
 }
 ObjectSetPrototypeOf(DeflateRaw.prototype, Zlib.prototype);
 ObjectSetPrototypeOf(DeflateRaw, Zlib);
@@ -747,7 +748,7 @@ ObjectSetPrototypeOf(DeflateRaw, Zlib);
 function InflateRaw(opts) {
   if (!(this instanceof InflateRaw))
     return new InflateRaw(opts);
-  Zlib.call(this, opts, INFLATERAW);
+  ReflectApply(Zlib, this, [opts, INFLATERAW]);
 }
 ObjectSetPrototypeOf(InflateRaw.prototype, Zlib.prototype);
 ObjectSetPrototypeOf(InflateRaw, Zlib);
@@ -755,7 +756,7 @@ ObjectSetPrototypeOf(InflateRaw, Zlib);
 function Unzip(opts) {
   if (!(this instanceof Unzip))
     return new Unzip(opts);
-  Zlib.call(this, opts, UNZIP);
+  ReflectApply(Zlib, this, [opts, UNZIP]);
 }
 ObjectSetPrototypeOf(Unzip.prototype, Zlib.prototype);
 ObjectSetPrototypeOf(Unzip, Zlib);
@@ -775,23 +776,26 @@ function createConvenienceMethod(ctor, sync) {
   };
 }
 
-const kMaxBrotliParam = MathMax(...ObjectKeys(constants).map((key) => {
-  return key.startsWith('BROTLI_PARAM_') ? constants[key] : 0;
-}));
+const kMaxBrotliParam = MathMaxApply(ArrayPrototypeMap(
+  ObjectKeys(constants),
+  (key) => (StringPrototypeStartsWith(key, 'BROTLI_PARAM_') ?
+    constants[key] :
+    0),
+));
 
 const brotliInitParamsArray = new Uint32Array(kMaxBrotliParam + 1);
 
 const brotliDefaultOpts = {
   flush: BROTLI_OPERATION_PROCESS,
   finishFlush: BROTLI_OPERATION_FINISH,
-  fullFlush: BROTLI_OPERATION_FLUSH
+  fullFlush: BROTLI_OPERATION_FLUSH,
 };
 function Brotli(opts, mode) {
   assert(mode === BROTLI_DECODE || mode === BROTLI_ENCODE);
 
-  brotliInitParamsArray.fill(-1);
-  if (opts && opts.params) {
-    for (const origKey of ObjectKeys(opts.params)) {
+  TypedArrayPrototypeFill(brotliInitParamsArray, -1);
+  if (opts?.params) {
+    ArrayPrototypeForEach(ObjectKeys(opts.params), (origKey) => {
       const key = +origKey;
       if (NumberIsNaN(key) || key < 0 || key > kMaxBrotliParam ||
           (brotliInitParamsArray[key] | 0) !== -1) {
@@ -804,23 +808,16 @@ function Brotli(opts, mode) {
                                        'number', opts.params[origKey]);
       }
       brotliInitParamsArray[key] = value;
-    }
+    });
   }
 
   const handle = mode === BROTLI_DECODE ?
     new binding.BrotliDecoder(mode) : new binding.BrotliEncoder(mode);
 
   this._writeState = new Uint32Array(2);
-  // TODO(addaleax): Sometimes we generate better error codes in C++ land,
-  // e.g. ERR_BROTLI_PARAM_SET_FAILED -- it's hard to access them with
-  // the current bindings setup, though.
-  if (!handle.init(brotliInitParamsArray,
-                   this._writeState,
-                   processCallback)) {
-    throw new ERR_ZLIB_INITIALIZATION_FAILED();
-  }
+  handle.init(brotliInitParamsArray, this._writeState, processCallback);
 
-  ZlibBase.call(this, opts, mode, handle, brotliDefaultOpts);
+  ReflectApply(ZlibBase, this, [opts, mode, handle, brotliDefaultOpts]);
 }
 ObjectSetPrototypeOf(Brotli.prototype, Zlib.prototype);
 ObjectSetPrototypeOf(Brotli, Zlib);
@@ -828,7 +825,7 @@ ObjectSetPrototypeOf(Brotli, Zlib);
 function BrotliCompress(opts) {
   if (!(this instanceof BrotliCompress))
     return new BrotliCompress(opts);
-  Brotli.call(this, opts, BROTLI_ENCODE);
+  ReflectApply(Brotli, this, [opts, BROTLI_ENCODE]);
 }
 ObjectSetPrototypeOf(BrotliCompress.prototype, Brotli.prototype);
 ObjectSetPrototypeOf(BrotliCompress, Brotli);
@@ -836,7 +833,7 @@ ObjectSetPrototypeOf(BrotliCompress, Brotli);
 function BrotliDecompress(opts) {
   if (!(this instanceof BrotliDecompress))
     return new BrotliDecompress(opts);
-  Brotli.call(this, opts, BROTLI_DECODE);
+  ReflectApply(Brotli, this, [opts, BROTLI_DECODE]);
 }
 ObjectSetPrototypeOf(BrotliDecompress.prototype, Brotli.prototype);
 ObjectSetPrototypeOf(BrotliDecompress, Brotli);
@@ -844,22 +841,33 @@ ObjectSetPrototypeOf(BrotliDecompress, Brotli);
 
 function createProperty(ctor) {
   return {
+    __proto__: null,
     configurable: true,
     enumerable: true,
     value: function(options) {
       return new ctor(options);
-    }
+    },
   };
 }
 
+function crc32(data, value = 0) {
+  if (typeof data !== 'string' && !isArrayBufferView(data)) {
+    throw new ERR_INVALID_ARG_TYPE('data', ['Buffer', 'TypedArray', 'DataView', 'string'], data);
+  }
+  validateUint32(value, 'value');
+  return crc32Native(data, value);
+}
+
 // Legacy alias on the C++ wrapper object. This is not public API, so we may
 // want to runtime-deprecate it at some point. There's no hurry, though.
 ObjectDefineProperty(binding.Zlib.prototype, 'jsref', {
+  __proto__: null,
   get() { return this[owner_symbol]; },
-  set(v) { return this[owner_symbol] = v; }
+  set(v) { return this[owner_symbol] = v; },
 });
 
 module.exports = {
+  crc32,
   Deflate,
   Inflate,
   Gzip,
@@ -903,22 +911,25 @@ ObjectDefineProperties(module.exports, {
   createBrotliCompress: createProperty(BrotliCompress),
   createBrotliDecompress: createProperty(BrotliDecompress),
   constants: {
+    __proto__: null,
     configurable: false,
     enumerable: true,
-    value: constants
+    value: constants,
   },
   codes: {
+    __proto__: null,
     enumerable: true,
     writable: false,
-    value: ObjectFreeze(codes)
-  }
+    value: ObjectFreeze(codes),
+  },
 });
 
 // These should be considered deprecated
 // expose all the zlib constants
 for (const bkey of ObjectKeys(constants)) {
-  if (bkey.startsWith('BROTLI')) continue;
+  if (StringPrototypeStartsWith(bkey, 'BROTLI')) continue;
   ObjectDefineProperty(module.exports, bkey, {
-    enumerable: false, value: constants[bkey], writable: false
+    __proto__: null,
+    enumerable: false, value: constants[bkey], writable: false,
   });
 }
