@@ -230,7 +230,10 @@ void SetIsolateErrorHandlers(v8::Isolate* isolate, const IsolateSettings& s) {
   auto* fatal_error_cb = s.fatal_error_callback ?
       s.fatal_error_callback : OnFatalError;
   isolate->SetFatalErrorHandler(fatal_error_cb);
-  isolate->SetOOMErrorHandler(OOMErrorHandler);
+
+  auto* oom_error_cb =
+      s.oom_error_callback ? s.oom_error_callback : OOMErrorHandler;
+  isolate->SetOOMErrorHandler(oom_error_cb);
 
   if ((s.flags & SHOULD_NOT_SET_PREPARE_STACK_TRACE_CALLBACK) == 0) {
     auto* prepare_stack_trace_cb = s.prepare_stack_trace_callback ?
