@@ -69,6 +69,12 @@ class Histogram : public MemoryRetainer {
 
 class HistogramImpl {
  public:
+  enum InternalFields {
+    kSlot = BaseObject::kSlot,
+    kImplField = BaseObject::kInternalFieldCount,
+    kInternalFieldCount
+  };
+
   explicit HistogramImpl(
       const Histogram::Options& options = Histogram::Options {});
   explicit HistogramImpl(std::shared_ptr<Histogram> histogram);
@@ -77,15 +83,60 @@ class HistogramImpl {
 
   const std::shared_ptr<Histogram>& histogram() const { return histogram_; }
 
+  static void DoReset(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetCountBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetMinBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetMaxBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetExceedsBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetCount(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetMin(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetMax(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetMean(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetExceeds(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetStddev(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetPercentile(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetPercentileBigInt(
+      const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetPercentiles(const v8::FunctionCallbackInfo<v8::Value>& args);
+  static void GetPercentilesBigInt(
+      const v8::FunctionCallbackInfo<v8::Value>& args);
+
+  static void FastReset(v8::Local<v8::Value> receiver);
+  static double FastGetCount(v8::Local<v8::Value> receiver);
+  static double FastGetMin(v8::Local<v8::Value> receiver);
+  static double FastGetMax(v8::Local<v8::Value> receiver);
+  static double FastGetMean(v8::Local<v8::Value> receiver);
+  static double FastGetExceeds(v8::Local<v8::Value> receiver);
+  static double FastGetStddev(v8::Local<v8::Value> receiver);
+  static double FastGetPercentile(v8::Local<v8::Value> receiver,
+                                  const double percentile);
+
+  static void AddMethods(v8::Isolate* isolate,
+                         v8::Local<v8::FunctionTemplate> tmpl);
+
+  static void RegisterExternalReferences(ExternalReferenceRegistry* registry);
+
+  static HistogramImpl* FromJSObject(v8::Local<v8::Value> value);
+
  private:
   std::shared_ptr<Histogram> histogram_;
+
+  static v8::CFunction fast_reset_;
+  static v8::CFunction fast_get_count_;
+  static v8::CFunction fast_get_min_;
+  static v8::CFunction fast_get_max_;
+  static v8::CFunction fast_get_mean_;
+  static v8::CFunction fast_get_exceeds_;
+  static v8::CFunction fast_get_stddev_;
+  static v8::CFunction fast_get_percentile_;
 };
 
-class HistogramBase : public BaseObject, public HistogramImpl {
+class HistogramBase final : public BaseObject, public HistogramImpl {
  public:
   static v8::Local<v8::FunctionTemplate> GetConstructorTemplate(
-    Environment* env);
-  static void Initialize(Environment* env, v8::Local<v8::Object> target);
+      IsolateData* isolate_data);
+  static void Initialize(IsolateData* isolate_data,
+                         v8::Local<v8::ObjectTemplate> target);
   static void RegisterExternalReferences(ExternalReferenceRegistry* registry);
 
   static BaseObjectPtr<HistogramBase> Create(
@@ -102,30 +153,16 @@ class HistogramBase : public BaseObject, public HistogramImpl {
   SET_MEMORY_INFO_NAME(HistogramBase)
   SET_SELF_SIZE(HistogramBase)
 
-  static void GetCountBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMinBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMaxBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetExceedsBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
-
-  static void GetCount(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMin(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMax(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMean(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetExceeds(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetStddev(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetPercentile(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetPercentileBigInt(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetPercentiles(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetPercentilesBigInt(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void DoReset(const v8::FunctionCallbackInfo<v8::Value>& args);
   static void Record(const v8::FunctionCallbackInfo<v8::Value>& args);
   static void RecordDelta(const v8::FunctionCallbackInfo<v8::Value>& args);
   static void Add(const v8::FunctionCallbackInfo<v8::Value>& args);
 
+  static void FastRecord(
+      v8::Local<v8::Value> receiver,
+      const int64_t value,
+      v8::FastApiCallbackOptions& options);  // NOLINT(runtime/references)
+  static void FastRecordDelta(v8::Local<v8::Value> receiver);
+
   HistogramBase(
       Environment* env,
       v8::Local<v8::Object> wrap,
@@ -161,9 +198,13 @@ class HistogramBase : public BaseObject, public HistogramImpl {
    private:
     std::shared_ptr<Histogram> histogram_;
   };
+
+ private:
+  static v8::CFunction fast_record_;
+  static v8::CFunction fast_record_delta_;
 };
 
-class IntervalHistogram : public HandleWrap, public HistogramImpl {
+class IntervalHistogram final : public HandleWrap, public HistogramImpl {
  public:
   enum class StartFlags {
     NONE,
@@ -189,29 +230,12 @@ class IntervalHistogram : public HandleWrap, public HistogramImpl {
       std::function<void(Histogram&)> on_interval,
       const Histogram::Options& options = Histogram::Options {});
 
-  static void GetCountBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMinBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMaxBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetExceedsBigInt(const v8::FunctionCallbackInfo<v8::Value>& args);
-
-  static void GetCount(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMin(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMax(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetMean(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetExceeds(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetStddev(const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetPercentile(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetPercentileBigInt(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetPercentiles(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void GetPercentilesBigInt(
-      const v8::FunctionCallbackInfo<v8::Value>& args);
-  static void DoReset(const v8::FunctionCallbackInfo<v8::Value>& args);
   static void Start(const v8::FunctionCallbackInfo<v8::Value>& args);
   static void Stop(const v8::FunctionCallbackInfo<v8::Value>& args);
 
+  static void FastStart(v8::Local<v8::Value> receiver, bool reset);
+  static void FastStop(v8::Local<v8::Value> receiver);
+
   TransferMode GetTransferMode() const override {
     return TransferMode::kCloneable;
   }
@@ -230,6 +254,9 @@ class IntervalHistogram : public HandleWrap, public HistogramImpl {
   int32_t interval_ = 0;
   std::function<void(Histogram&)> on_interval_;
   uv_timer_t timer_;
+
+  static v8::CFunction fast_start_;
+  static v8::CFunction fast_stop_;
 };
 
 }  // namespace node
