@@ -92,35 +92,57 @@ static const size_t kHeapCodeStatisticsPropertiesCount =
     HEAP_CODE_STATISTICS_PROPERTIES(V);
 #undef V
 
-BindingData::BindingData(Realm* realm, Local<Object> obj)
+BindingData::BindingData(Realm* realm,
+                         Local<Object> obj,
+                         InternalFieldInfo* info)
     : SnapshotableObject(realm, obj, type_int),
-      heap_statistics_buffer(realm->isolate(), kHeapStatisticsPropertiesCount),
-      heap_space_statistics_buffer(realm->isolate(),
-                                   kHeapSpaceStatisticsPropertiesCount),
-      heap_code_statistics_buffer(realm->isolate(),
-                                  kHeapCodeStatisticsPropertiesCount) {
+      heap_statistics_buffer(realm->isolate(),
+                             kHeapStatisticsPropertiesCount,
+                             MAYBE_FIELD_PTR(info, heap_statistics_buffer)),
+      heap_space_statistics_buffer(
+          realm->isolate(),
+          kHeapSpaceStatisticsPropertiesCount,
+          MAYBE_FIELD_PTR(info, heap_space_statistics_buffer)),
+      heap_code_statistics_buffer(
+          realm->isolate(),
+          kHeapCodeStatisticsPropertiesCount,
+          MAYBE_FIELD_PTR(info, heap_code_statistics_buffer)) {
   Local<Context> context = realm->context();
-  obj->Set(context,
-           FIXED_ONE_BYTE_STRING(realm->isolate(), "heapStatisticsBuffer"),
-           heap_statistics_buffer.GetJSArray())
-      .Check();
-  obj->Set(context,
+  if (info == nullptr) {
+    obj->Set(context,
+             FIXED_ONE_BYTE_STRING(realm->isolate(), "heapStatisticsBuffer"),
+             heap_statistics_buffer.GetJSArray())
+        .Check();
+    obj->Set(
+           context,
            FIXED_ONE_BYTE_STRING(realm->isolate(), "heapCodeStatisticsBuffer"),
            heap_code_statistics_buffer.GetJSArray())
-      .Check();
-  obj->Set(context,
+        .Check();
+    obj->Set(
+           context,
            FIXED_ONE_BYTE_STRING(realm->isolate(), "heapSpaceStatisticsBuffer"),
            heap_space_statistics_buffer.GetJSArray())
-      .Check();
+        .Check();
+  } else {
+    heap_statistics_buffer.Deserialize(realm->context());
+    heap_code_statistics_buffer.Deserialize(realm->context());
+    heap_space_statistics_buffer.Deserialize(realm->context());
+  }
+  heap_statistics_buffer.MakeWeak();
+  heap_space_statistics_buffer.MakeWeak();
+  heap_code_statistics_buffer.MakeWeak();
 }
 
 bool BindingData::PrepareForSerialization(Local<Context> context,
                                           v8::SnapshotCreator* creator) {
-  // We'll just re-initialize the buffers in the constructor since their
-  // contents can be thrown away once consumed in the previous call.
-  heap_statistics_buffer.Release();
-  heap_space_statistics_buffer.Release();
-  heap_code_statistics_buffer.Release();
+  DCHECK_NULL(internal_field_info_);
+  internal_field_info_ = InternalFieldInfoBase::New<InternalFieldInfo>(type());
+  internal_field_info_->heap_statistics_buffer =
+      heap_statistics_buffer.Serialize(context, creator);
+  internal_field_info_->heap_space_statistics_buffer =
+      heap_space_statistics_buffer.Serialize(context, creator);
+  internal_field_info_->heap_code_statistics_buffer =
+      heap_code_statistics_buffer.Serialize(context, creator);
   // Return true because we need to maintain the reference to the binding from
   // JS land.
   return true;
@@ -130,17 +152,20 @@ void BindingData::Deserialize(Local<Context> context,
                               Local<Object> holder,
                               int index,
                               InternalFieldInfoBase* info) {
-  DCHECK_EQ(index, BaseObject::kEmbedderType);
+  DCHECK_IS_SNAPSHOT_SLOT(index);
   HandleScope scope(context->GetIsolate());
   Realm* realm = Realm::GetCurrent(context);
-  BindingData* binding = realm->AddBindingData<BindingData>(context, holder);
+  // Recreate the buffer in the constructor.
+  InternalFieldInfo* casted_info = static_cast<InternalFieldInfo*>(info);
+  BindingData* binding =
+      realm->AddBindingData<BindingData>(holder, casted_info);
   CHECK_NOT_NULL(binding);
 }
 
 InternalFieldInfoBase* BindingData::Serialize(int index) {
-  DCHECK_EQ(index, BaseObject::kEmbedderType);
-  InternalFieldInfo* info =
-      InternalFieldInfoBase::New<InternalFieldInfo>(type());
+  DCHECK_IS_SNAPSHOT_SLOT(index);
+  InternalFieldInfo* info = internal_field_info_;
+  internal_field_info_ = nullptr;
   return info;
 }
 
@@ -344,7 +369,7 @@ void GCProfiler::New(const FunctionCallbackInfo<Value>& args) {
 void GCProfiler::Start(const FunctionCallbackInfo<Value>& args) {
   Environment* env = Environment::GetCurrent(args);
   GCProfiler* profiler;
-  ASSIGN_OR_RETURN_UNWRAP(&profiler, args.Holder());
+  ASSIGN_OR_RETURN_UNWRAP(&profiler, args.This());
   if (profiler->state != GCProfiler::GCProfilerState::kInitialized) {
     return;
   }
@@ -369,7 +394,7 @@ void GCProfiler::Start(const FunctionCallbackInfo<Value>& args) {
 void GCProfiler::Stop(const FunctionCallbackInfo<v8::Value>& args) {
   Environment* env = Environment::GetCurrent(args);
   GCProfiler* profiler;
-  ASSIGN_OR_RETURN_UNWRAP(&profiler, args.Holder());
+  ASSIGN_OR_RETURN_UNWRAP(&profiler, args.This());
   if (profiler->state != GCProfiler::GCProfilerState::kStarted) {
     return;
   }
@@ -397,8 +422,7 @@ void Initialize(Local<Object> target,
                 void* priv) {
   Realm* realm = Realm::GetCurrent(context);
   Environment* env = realm->env();
-  BindingData* const binding_data =
-      realm->AddBindingData<BindingData>(context, target);
+  BindingData* const binding_data = realm->AddBindingData<BindingData>(target);
   if (binding_data == nullptr) return;
 
   SetMethodNoSideEffect(
