@@ -1,4 +1,5 @@
 #include "base64-inl.h"
+#include "simdutf.h"
 
 #include <cstddef>
 #include <cstring>
@@ -6,14 +7,16 @@
 #include "gtest/gtest.h"
 
 using node::base64_decode;
-using node::base64_encode;
 
 TEST(Base64Test, Encode) {
   auto test = [](const char* string, const char* base64_string) {
     const size_t len = strlen(base64_string);
+    const size_t slen = strlen(string);
     char* const buffer = new char[len + 1];
     buffer[len] = 0;
-    base64_encode(string, strlen(string), buffer, len);
+    CHECK(len >= simdutf::base64_length_from_binary(slen) &&
+          "not enough space provided for base64 encode");
+    simdutf::binary_to_base64(string, slen, buffer);
     EXPECT_STREQ(base64_string, buffer);
     delete[] buffer;
   };
@@ -47,9 +50,13 @@ TEST(Base64Test, Encode) {
 TEST(Base64Test, EncodeURL) {
   auto test = [](const char* string, const char* base64_string) {
     const size_t len = strlen(base64_string);
+    const size_t slen = strlen(string);
     char* const buffer = new char[len + 1];
     buffer[len] = 0;
-    base64_encode(string, strlen(string), buffer, len, node::Base64Mode::URL);
+    CHECK(len >=
+              simdutf::base64_length_from_binary(slen, simdutf::base64_url) &&
+          "not enough space provided for base64 encode");
+    simdutf::binary_to_base64(string, slen, buffer, simdutf::base64_url);
     EXPECT_STREQ(base64_string, buffer);
     delete[] buffer;
   };
