@@ -3,7 +3,6 @@
 const common = require('../common');
 const assert = require('assert');
 const fs = require('fs');
-const path = require('path');
 
 if (!common.isWindows)
   common.skip('This test is for Windows only.');
@@ -18,7 +17,7 @@ const DATA_VALUE = 'hello';
 const RESERVED_CHARACTERS = '<>"|?*';
 
 [...RESERVED_CHARACTERS].forEach((ch) => {
-  const pathname = path.join(tmpdir.path, `somefile_${ch}`);
+  const pathname = tmpdir.resolve(`somefile_${ch}`);
   assert.throws(
     () => {
       fs.writeFileSync(pathname, DATA_VALUE);
@@ -29,7 +28,7 @@ const RESERVED_CHARACTERS = '<>"|?*';
 
 // Test for ':' (NTFS data streams).
 // Refs: https://msdn.microsoft.com/en-us/library/windows/desktop/bb540537.aspx
-const pathname = path.join(tmpdir.path, 'foo:bar');
+const pathname = tmpdir.resolve('foo:bar');
 fs.writeFileSync(pathname, DATA_VALUE);
 
 let content = '';
