@@ -25,6 +25,8 @@ if (versionParts[0] === '4' && versionParts[1] >= 2) {
   assert.strictEqual(process.release.lts, 'Gallium');
 } else if (versionParts[0] === '18' && versionParts[1] >= 12) {
   assert.strictEqual(process.release.lts, 'Hydrogen');
+} else if (versionParts[0] === '20' && versionParts[1] >= 9) {
+  assert.strictEqual(process.release.lts, 'Iron');
 } else {
   assert.strictEqual(process.release.lts, undefined);
 }
