@@ -405,7 +405,7 @@ putIn.run([
   'var custom = "test";',
 ]);
 testMe.complete('cus', common.mustCall(function(error, data) {
-  assert.deepStrictEqual(data, [['custom'], 'cus']);
+  assert.deepStrictEqual(data, [['CustomEvent', 'custom'], 'cus']);
 }));
 
 // Make sure tab completion doesn't crash REPL with half-baked proxy objects.
@@ -584,7 +584,7 @@ testMe.complete('obj.', common.mustCall(function(error, data) {
 
     data[0].forEach((key) => {
       if (!key || key === 'ele.biu') return;
-      assert.notStrictEqual(ele[key.substr(4)], undefined);
+      assert.notStrictEqual(ele[key.slice(4)], undefined);
     });
   }));
 });
