@@ -13,7 +13,6 @@ if (common.isWindows)
 common.skipIfWorker(); // Worker inspector never has a server running
 
 common.expectWarning('Warning',
-                     'process.on(SIGPROF) is reserved while debugging',
-                     common.noWarnCode);
+                     'process.on(SIGPROF) is reserved while debugging');
 
 process.on('SIGPROF', () => {});
