@@ -1,7 +1,7 @@
-// Flags: --experimental-worker
 'use strict';
 
 const common = require('../common');
+const assert = require('assert');
 const { Worker } = require('worker_threads');
 
 {
@@ -13,15 +13,16 @@ const { Worker } = require('worker_threads');
     Symbol('test'),
     {},
     [],
-    () => {}
+    () => {},
   ].forEach((val) => {
-    common.expectsError(
+    assert.throws(
       () => new Worker(val),
       {
         code: 'ERR_INVALID_ARG_TYPE',
-        type: TypeError,
-        message: 'The "filename" argument must be of type string. ' +
-                 `Received type ${typeof val}`
+        name: 'TypeError',
+        message: 'The "filename" argument must be of type string ' +
+                 'or an instance of URL.' +
+                 common.invalidArgTypeHelper(val)
       }
     );
   });
